/*  bzip2 handling functions
 *  Copyright (c) 2001  Lucian Langa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <unistd.h>
#include <pwd.h>
#include <wait.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "interface.h"
#include "conf.h"
#include "gnochive.h"
#include "zip.h"

#define TIMEOUT_GTK 5

extern gchivemain *gnochivemain;

extern GdkPixmap 	*dirpix,*docpix,*xpmpix,*makpix;
extern GdkPixmap	*rpmpix,*comppix,*dotpix,*lockpix;
extern GdkPixmap *hdrpix,*sympix,*rdpix,*rpm1pix;

extern GdkBitmap 	*dirmask,*docmask,*xpmmask,*makmask;
extern GdkBitmap	*rpmmask,*compmask,*dotmask,*lockmask;
extern GdkBitmap *hdrmask,*symmask,*rdmask,*rpm1mask;

extern gchar *arc_type;

extern GtkWidget *arcmenu1;

extern gchar *directory_read;
extern gchar *directory_up;

extern gfloat step;
extern float progr;
extern gchar *opd_archive_name;
extern int last_select_row;

static guint mytime(gpointer data);
 
/*
 *	0 - no error, file opened
 *	1 - error not an archive
 * FIXME! maybe add a message window to tell the user
 * that archive cannot be displayed as a filesystem
 * but offer him the possibility to extract the contents
 */
int
bzip2_open(pager *page, gchar *file_name)
{
	gint status=0;
	gint line = 0;
	gint arcsize;
	gchar *arcfile = NULL;
	gchar *myfile = NULL;
	gchar *clist_line[9] = {"","","","","","","","",""};
	FILE *tempfile;
	pid_t pid;
	gchar *temp_name = g_concat_dir_and_file(TMPDIR,"gchiveXXXXXX");
	gchar *temp_name2 = g_concat_dir_and_file(TMPDIR,"gchiveXXXXXX");
	
	/* FIXME! this is not very smart */
	(void *)mkstemp(temp_name);
	(void *)mkstemp(temp_name2);
	
	myfile = g_strdup(file_name);
	pid = gnochive_exec_arch("/tmp/gzipview", "gzipview",
		gnochivemain->gc_cfg->archs.bz2_path,
		"-d", "-c", myfile, temp_name, NULL);
	wait(0);
	tempfile = fopen(temp_name, "r");
	if (tempfile)
	{
	fseek(tempfile, 0L, SEEK_END);
	arcsize = ftell(tempfile);
	fclose(tempfile);

	gtk_clist_clear(GTK_CLIST(page->clist));
	
	if (arcsize==0) {
		clist_line[1] = directory_read;
		gtk_clist_append(GTK_CLIST(page->clist), clist_line);
		clist_line[1] = directory_up;
		gtk_clist_append(GTK_CLIST(page->clist),clist_line);
		gtk_clist_set_pixmap(GTK_CLIST(page->clist), 1, 0, dotpix, dotmask);
		pid = gnochive_exec_arch("/tmp/zpvw", "zpvw",
			gnochivemain->gc_cfg->archs.bz2_path,
			"-dvc", myfile, temp_name2, NULL);
		wait(0);
		g_print("%s\n", temp_name2);
		tempfile = fopen(temp_name2, "r");
		fseek(tempfile,0L,SEEK_END);
		arcsize = ftell(tempfile);
		arcfile = malloc(arcsize);
		fseek(tempfile, 0L, SEEK_SET);
		fread(arcfile, arcsize, 1, tempfile);
		fclose(tempfile);
		/*i=98;j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) {
			xfr[j] = arcfile[i];
			i++;j++;
		}
		xfr[j] = '\0';
		clist_line[7] = g_strdup(xfr);
		j=0;
		while (arcfile[i]==0x20) i++;
		for (q=0;q<2;q++) 
			{
			while (arcfile[i]!=0x20) {
					    xfr[j]=arcfile[i];
					    i++;j++;
					    }
			xfr[j]=' ';
			i++;j++;
			}
		while (arcfile[i]!=0x20) {
					    xfr[j]=arcfile[i];
					    i++;j++;
					    }
		xfr[j]='\0';
		clist_line[5]=g_strdup(xfr);
		j=0;
		while (arcfile[i]==0x20) i++;
		while (arcfile[i]!=0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[4] = g_strdup(xfr);
		j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) {
					    xfr[j]=arcfile[i];
					    i++;j++;
					    }
					    xfr[j]='\0';
		clist_line[3]=g_strdup(xfr);
		j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[8] = g_strdup(xfr);
		j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x0A) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[1] = g_strdup(xfr);
		gtk_clist_append(GTK_CLIST(page->clist),clist_line);
		page->current_lines = 3;
		page->selected_rows = g_malloc0(page->current_lines);
		g_free(clist_line[1]);g_free(clist_line[3]);
		g_free(clist_line[4]);g_free(clist_line[5]);
		g_free(clist_line[7]);g_free(clist_line[8]);
		status=0;*/
		gtk_clist_append(GTK_CLIST(gnochivemain->active_page->clist),clist_line);
		gnochivemain->active_page->current_lines=line;
		gnochivemain->active_page->selected_rows=g_malloc0(gnochivemain->active_page->current_lines);
		status=1;
	}
}
else {
	g_free(myfile);
	status=0;
}
g_free(temp_name);
g_free(temp_name2);
remove(temp_name);
remove(temp_name2);
return status;
}

int
tbz2_open(pager *page, gchar *file_name)
{
	int status = 0;
	int vline=0;
	gchar *myfile = NULL;
	gchar *arcfile=NULL;
	gint i,j,w,line = 0, e=0, k;
	guint time_id, arcsize;
	FILE *tempfile;
	gchar xfr[0x100];
	gchar *xfrwd[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
	gchar *clist_line[9] = {"","","","","","","","",""};
	gchar *dirs[512];
	gboolean is_dr, dirprs;
	gchar *bzcommand=NULL;
	gchar *tdir = NULL, *rbase2 = NULL;
	gchar *temp_name = g_concat_dir_and_file(TMPDIR,"gchiveXXXXXX");
	
	(void *)mkstemp(temp_name);
	
	step = 0.003;
	progr = 0.1;
		
	time_id = gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)mytime,NULL);
	myfile = g_strdup(file_name);
	bzcommand = g_strconcat(gnochivemain->gc_cfg->tbz2parm,"tvf",NULL);
	(void *)gnochive_exec_arch("/tmp/zpvw", "zpvw",
			gnochivemain->gc_cfg->archs.tar_path,
			bzcommand, myfile, temp_name, NULL);
	while(gtk_events_pending())
		gtk_main_iteration();
	gtk_clist_freeze(GTK_CLIST(page->clist));
	gtk_clist_clear(GTK_CLIST(page->clist));
	clist_line[1] = directory_read;
	gtk_clist_append(GTK_CLIST(page->clist), clist_line);
	clist_line[1] = directory_up;
	gtk_clist_append(GTK_CLIST(page->clist),clist_line);
	gtk_clist_set_pixmap(GTK_CLIST(page->clist),
						1, 0, dotpix, dotmask);
	vline = line = 2;
	while (!waitpid(0,NULL,WNOHANG))
	{
		gtk_main_iteration();
	}
	while(gtk_events_pending())
		gtk_main_iteration();
	tempfile=fopen(temp_name,"rb");
	if (tempfile)
	{
		fseek(tempfile,0L,SEEK_END);
		arcsize=ftell(tempfile);
		arcfile=g_malloc(arcsize);
		fseek(tempfile,0L,SEEK_SET);
		fread(arcfile,1,arcsize,tempfile);
		fclose(tempfile);
		i=0;
		while (i<=arcsize-2) {
				j=0;line++;
				while (arcfile[i]!=0x20) {
					xfr[j]=arcfile[i];
					i++;j++;
				}
				xfr[j] = '\0';
				xfrwd[0] = g_strdup(xfr);
				(gchar *)clist_line[6] = xfrwd[0];
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) i++;
				while (arcfile[i] == 0x20) i++;
				j=0;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
							xfr[j] = '\0';
				xfrwd[1] = g_strdup(xfr);
				(gchar *)clist_line[3] = xfrwd[1];
				j=0;
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
				xfr[j]=' ';i++;j++;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
							xfr[j]='\0';
				xfrwd[2]=g_strdup(xfr);
				(gchar *)clist_line[5]=xfrwd[2];
				j=0;
				while (arcfile[i]==0x20) i++;
				while (arcfile[i]!=0x0A) {
							xfr[j]=arcfile[i];
							i++;j++;
							}
							xfr[j]='\0';
				i++;
				xfrwd[3]=g_strdup(xfr);
				(gchar *)clist_line[1]=xfrwd[3];							
				is_dr = sort_dirs(clist_line[1]);
				tdir = extract_root_word(clist_line[1], 1);	
				if (e==0)
				{
					if (is_dr) 
					{
						dirs[e] = extract_root_word(clist_line[1], 1);
						e++;
						(gchar *)clist_line[1] = tdir;
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, dirpix, dirmask);
						vline++;
						is_dr = FALSE;
					}
					else
					{
						if (!process_line(clist_line[1])) {
							gtk_clist_append(GTK_CLIST(page->clist), clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(page->clist),
								vline, 0, docpix, docmask);
							vline++;
						}
						else
						{
							rbase2 = extract_root_word(clist_line[1], 1);
							dirprs = FALSE;
							for (k=0;k<e;k++)
							{
								if (!cmpare(dirs[k], rbase2))
									dirprs = TRUE;
							}
							g_free(rbase2);
							if (!dirprs)
							{
								dirs[e] = extract_root_word(clist_line[1], 1);
								e++;
								/* found a new dir and append it in the clist */
								(gchar *)clist_line[1] = extract_root_word(clist_line[1], 1);
								(gchar *)clist_line[3] = g_strdup("0");
								(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
								gtk_clist_append(GTK_CLIST(page->clist), clist_line);
								gtk_clist_set_pixmap(GTK_CLIST(page->clist),
									vline, 0, dirpix, dirmask);
								vline++;
							}
						}
					}
				}
				else {
				if (is_dr) 
				{
					is_dr = FALSE;
					dirprs = FALSE;
					for (k=0;k<e;k++)
					{
						if (!cmpare(dirs[k], tdir))
							dirprs = TRUE;
					}
					if (!dirprs)
					{
						dirs[e] = extract_root_word(clist_line[1], 1);
						e++;
						/* found a new dir and append it in the clist */
						(gchar *)clist_line[1] = tdir;
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, dirpix, dirmask);
						vline++;
					}
				}
				else
				{
					if (!process_line(clist_line[1])) {
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, docpix, docmask);
						vline++;
					}
					else
					{
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(dirs[k], tdir))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(clist_line[1], 1);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = tdir;
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							gtk_clist_append(GTK_CLIST(page->clist), clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(page->clist),
								vline, 0, dirpix, dirmask);
							vline++;
						}
					}
				}
			}
			for (w=0;w<=3;w++)
				g_free(xfrwd[w]);
			}
			gtk_clist_thaw(GTK_CLIST(page->clist));
			gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
			gtk_timeout_remove(time_id);
			page->current_lines =line;
			page->selected_rows =
							g_malloc0(page->current_lines);
			page->ad->data = arcfile;
			page->ad->root = g_strdup("\0");
			page->ad->depth = 1;
			status=1;
	}
	else status=0;
	g_free(bzcommand);
	g_free(myfile);
	g_free(temp_name);
	remove(temp_name);
	return status;
}

void
bzip_cd		(GtkWidget *clst, gchar *child)
{
	int vline = 0, line = 0;
	int i = 0, j;
	gchar *tword = NULL;
	gchar *temp = NULL;
	gchar *arcfile;
	gchar xfr[100];
	gchar *clist_line[9] = {"", "", "", "", "", "", "", "", ""};
	gchar *dirs[512], *tdir = NULL;
	gint row, arcsize, e=0, k;
	gboolean is_dr = FALSE, dirprs = FALSE;
		
	line = vline = 2;	
	while (gtk_events_pending())
		gtk_main_iteration();
	
	gtk_clist_freeze(GTK_CLIST(clst));
	gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
	g_free(gnochivemain->active_page->selected_rows);
	
	gtk_clist_set_auto_sort(GTK_CLIST(clst), 1);
	gtk_clist_set_sort_column(GTK_CLIST(clst), 2);
	gtk_clist_set_sort_type(GTK_CLIST(clst), GTK_SORT_DESCENDING);
	gtk_clist_sort(GTK_CLIST(clst));
	
	arcfile = gnochivemain->active_page->ad->data;
	arcsize = strlen(arcfile);
	arcsize-=1;
	i=0;
	while (i<=arcsize-2) {
		j=0;
		while (arcfile[i] != 0x20) {
			xfr[j] = arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[6] = g_strdup(xfr);
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) i++;
		while (arcfile[i] == 0x20) i++;
		j=0;
		while (arcfile[i]!=0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[3]=g_strdup(xfr);
		j=0;
		while (arcfile[i]==0x20) i++;
		while (arcfile[i]!=0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]=' ';i++;j++;
		while (arcfile[i]!=0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		clist_line[5] = g_strdup(xfr);
		j=0;
		while (arcfile[i]==0x20) i++;
		while (arcfile[i]!=0x0A) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j]='\0';
		i++;
		if (cmpare(xfr, child) == 0)
		{
			is_dr = sort_dirs(xfr);
			if (e==0 && strcmp(xfr, child) != 0)
			{
				if (is_dr) 
				{
					dirs[e] = extract_root_word(xfr,
						gnochivemain->active_page->ad->depth);
					e++;
					clist_line[1] = trim_root_word(child, xfr);
					clist_line[2] = "dir";
					row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
					gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
					g_free(clist_line[1]);
					vline++;
					is_dr = FALSE;
				}
				else
				{
					gchar *rbase2 = trim_root_word(child, xfr);
					if (!process_line(rbase2)) {
						clist_line[1] = trim_root_word(child, xfr);
						clist_line[2] = " ";
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst),
							row, 0, docpix,docmask);
						g_free(clist_line[1]);
						vline++;
					}
					else
					{
						gchar *rbase3 = extract_root_word(xfr, gnochivemain->active_page->ad->depth);
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(rbase3, dirs[k]))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(xfr, gnochivemain->active_page->ad->depth);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = trim_root_word(child, rbase3);
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
							g_free(clist_line[1]);
							vline++;
						}
						g_free(rbase3);
					}
					g_free(rbase2);
				}
			}
			else if (e>0  && strcmp(xfr, child) != 0)
			{
				if (is_dr && strcmp(xfr, child) != 0)
				{
					tdir = extract_root_word(xfr,
						gnochivemain->active_page->ad->depth);
					is_dr = FALSE;
					dirprs = FALSE;
					for (k=0;k<e;k++)
					{
						if (!cmpare(tdir, dirs[k]))
							dirprs = TRUE;
					}
					if (!dirprs)
					{
						dirs[e] = extract_root_word(xfr,
							gnochivemain->active_page->ad->depth);
						e++;
						/* found a new dir and append it in the clist */
						(gchar *)clist_line[1] = trim_root_word(child, tdir);
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
						g_free(clist_line[1]);
						vline++;
					}
					g_free(tdir);
				}
				else
				{
					gchar *rbase2 = trim_root_word(child, xfr);
					if (!process_line(rbase2)) {
						clist_line[1] = trim_root_word(child, xfr);
						clist_line[2] = " ";
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst),
							row, 0, docpix,docmask);
						g_free(clist_line[1]);
						vline++;
					}
					else
					{
						tdir = extract_root_word(xfr,
							gnochivemain->active_page->ad->depth);
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(dirs[k], tdir))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(xfr,
								gnochivemain->active_page->ad->depth);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = trim_root_word(child, tdir);
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
							g_free(clist_line[1]);
							vline++;
						}
						g_free(tdir);
					}
					g_free(rbase2);
				}
			}
		}
		g_free(clist_line[6]);g_free(clist_line[3]);g_free(clist_line[5]);
	}
	for (i=0;i<=9;i++)
		clist_line[i] = "\0";
	gtk_clist_set_auto_sort(GTK_CLIST(clst), 0);	
	clist_line[1] = g_strdup(directory_up);
	gtk_clist_prepend(GTK_CLIST(clst), clist_line);
	gtk_clist_set_pixmap(GTK_CLIST(clst), 0, 0, dotpix, dotmask);
	g_free(clist_line[1]);
	clist_line[1] = g_strdup(directory_read);
	gtk_clist_prepend(GTK_CLIST(clst), clist_line);
	g_free(clist_line[1]);
	gnochivemain->active_page->current_lines = vline;
	gnochivemain->active_page->selected_rows = 
						g_malloc0(gnochivemain->active_page->current_lines);
	temp = g_strdup(child);
	if (temp[0] == '/')
	{
		g_free(gnochivemain->active_page->ad->root);
		gtk_entry_set_text(GTK_ENTRY(arcmenu1), "\0");
		gnochivemain->active_page->ad->root = g_strdup("\0");
	}
	else
	{
		g_free(gnochivemain->active_page->ad->root);
		gtk_entry_set_text(GTK_ENTRY(arcmenu1), temp);
		gnochivemain->active_page->ad->root = g_strdup(child);
	}
	g_free(temp);
	gtk_clist_thaw(GTK_CLIST(clst));
	g_free(tword);
}


void
bzip_sel(gboolean have_context)
{
	gchar *text = NULL;
	gchar *ttext = NULL;
	pid_t pid;
	int error;
	gboolean status;
		
	while (gtk_events_pending())
		gtk_main_iteration();
	gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),last_select_row,1,&text);
	pid = gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path,
			"bzip2", "-f", text, NULL);
	while (!waitpid(pid,&status,WNOHANG))
	{
		error=(WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	ttext=g_strdup_printf(N_("%s passed through bzip2 successfuly!"),text);
	gnome_app_flash(GNOME_APP(gnochivemain->mainwin),ttext);
	g_free(ttext);
	gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),last_select_row,1);
	/* Useful when we're called from contextual menu */
	if (have_context)
	{
		gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
		read_pwd_files(".");
		g_free(gnochivemain->active_page->selected_rows);
		gnochivemain->active_page->selected_rows=g_malloc0(gnochivemain->active_page->current_lines);
	}
}

/* FIXME! */
void
bzip_stat(gpointer user_data)
{
	gchar *text='\0';
	int i=0, q=0, j=0, tss;
	gchar	name[100], size[100], pck[100],
			ratio[100], temp[100];
	gchar *clist1_line[2][9];
	FILE *tempfile;
	guint totalsize, arcsize, totalpck;
	
memset(name,0,strlen(name));
memset(size,0,strlen(size));
memset(pck,0,strlen(pck));
memset(ratio,0,strlen(ratio));
gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
strcpy(name,text);
(gchar *)clist1_line[0][0]=name;
tss=open("/tmp/tes1",O_RDWR);
if (fork()==0) {
    dup2(tss,0);
    dup2(tss,1);
    dup2(tss,2);
    gnochive_exec_arch("/tmp/statbz", "statbz", 
		gnochivemain->gc_cfg->archs.bz2_path, "-cvv", name, NULL);
    }
wait(0);
tempfile=fopen("/tmp/tes1","r");
while(gtk_events_pending())
gtk_main_iteration();
gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,3,&text);
strcpy(size,text);
totalsize=totalsize+atoi(size);
(gchar *)clist1_line[0][1]=size;
gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);

fseek(tempfile,0L,SEEK_END);
arcsize=ftell(tempfile);
fseek(tempfile,0L,SEEK_SET);
fread(temp,arcsize,1,tempfile);
g_print("%s",temp);
fclose(tempfile);
q=0x41;
while (temp[q]==0x20) q++;
j=0;
while (temp[q]!=0x20) {
			pck[j]=temp[q];
			j++;q++;
			}
(gchar *)clist1_line[0][2]=pck;
totalpck=totalpck+atoi(pck);
while (temp[q]==0x20) q++;
j=0;
while (temp[q]!=0x20) q++;
while (temp[q]==0x20) q++;
while (temp[q]!=0x20) {
			ratio[j]=temp[q];
			j++;q++;
			}
			ratio[j++]='\0';
(gchar *)clist1_line[0][3]=ratio;
gtk_clist_append(GTK_CLIST(user_data),clist1_line[0]);
}

void
tbzpack(void)
{
	GtkWidget *dialog;
	
	//strcpy(q_ext,".tar.bz2");
	//strcpy(comp_command,"-9");
	//sel_filter=2;
	(gchar *)arc_type="tar";
	dialog=create_quick_pack();
	gtk_widget_show(dialog);
}

static 
guint mytime(gpointer data)
{
	GDK_THREADS_ENTER();
	progr=progr+step;
	if (progr<=1)
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),progr);
	else gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
	GDK_THREADS_LEAVE();
	return TRUE;
}
