/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Lucian Langa
 *
 *  gnochive components: a set of components, in the future will become
 *  usable by bonobo support
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdarg.h"
#include "support.h"
#include "gnomefilelist.h"
#include "gnomedirlist.h"
#include "toolbar.h"
#include "components.h"
#include "gnochive.h"
#include "interface.h"
#include "gc_bk_end.h"
#include "conf.h"
#include "dnd.h"


#include <pwd.h>
#include <math.h>

extern void gzip_sel(gboolean have_context);
extern void bzip_sel(gboolean have_context);

extern guint srw;

GnomeApp *myapp;

extern gchivemain *gnochivemain;
extern archivedesc *my_ad;
extern int last_select_row;

qpack *qck_pack;

GtkWidget *dock1;

GtkWidget *entry3;
GtkWidget *entrygzip;
GtkWidget *entrytar;
GtkWidget *entryzip;
GtkWidget *entryunzip;
GtkWidget *entrybzip2;
GtkWidget *entryZ;
GtkWidget *entryarj;
GtkWidget *entryace;
GtkWidget *entrylha;
GtkWidget *entryzoo;
GtkWidget *entrycpio;
GtkWidget *entryrpm2cpio;
GtkWidget *entryrpm;

enum {
	DND_TYPE_STRING = 0,
	DND_TYPE_TEXT_PLAIN,
	DND_TYPE_NETSCAPE_URL,
	DND_TYPE_TEXT_URI_LIST,
	DND_TYPE_APPLICATION_X_ROOTWIN_DROP
};

static GtkTargetEntry possible_drag_types[] = {
	    {"STRING",						0,	DND_TYPE_STRING},
	    {"text/plain",					0,	DND_TYPE_TEXT_PLAIN},
		{"_NETSCAPE_URL_",				0,	DND_TYPE_NETSCAPE_URL},
	    {"text/uri-list",				0,	DND_TYPE_TEXT_URI_LIST},
	    {"application/x-rootwin-drop",	0,	DND_TYPE_APPLICATION_X_ROOTWIN_DROP}
	};
		
static guint n_possible_drag_types = sizeof(possible_drag_types)/sizeof(possible_drag_types[0]);

static gboolean main_arch_menu(GtkWidget *item, gpointer user_data);
static gboolean subarch_menu(GtkWidget *item, gpointer user_data);
static gboolean on_appear_menu_selected(GtkWidget *item, gpointer user_data);
static gboolean on_tabs_menu_selected(GtkWidget *item, gpointer user_data);

static gboolean
on_fileselection_delete_event (GtkWidget * w, GdkEvent * event, gpointer data)
{
	gtk_widget_hide (w);
	return TRUE;
}

static void
on_file_selection_ok_clicked (GtkButton * button, gpointer data)
{
	gchar *filename;
	FileSelData *fd = data;

	filename = gnome_filelist_get_filename (GNOME_FILELIST(fd->filesel));
	if (!filename)
		return;
	if (is_dir (filename))
	{
		gnome_filelist_set_dir (GNOME_FILELIST(fd->filesel), filename);
		g_free (filename);
		return;
	}
	gtk_widget_hide (fd->filesel);
	if (fd->click_ok_callback)
		fd->click_ok_callback (button, fd->data);
	
	g_free (filename);
	return;
}

static gboolean
on_dirsel_delete_event (GtkWidget * w, GdkEvent * event, gpointer data)
{
	gtk_widget_hide (w);
	return TRUE;
}

GtkWidget*
create_myfile_selection(FileSelData *myfs)
{
	GtkWidget *fileselection_gui;
	GtkWidget *fileselection_ok;
	GtkWidget *fileselection_cancel;

	fileselection_gui = gnome_filelist_new ();
	gnome_filelist_set_title (GNOME_FILELIST(fileselection_gui), myfs->title);
	gtk_container_set_border_width (GTK_CONTAINER (fileselection_gui), 10);
	gtk_window_set_position (GTK_WINDOW (fileselection_gui), GTK_WIN_POS_CENTER);
	gtk_window_set_wmclass (GTK_WINDOW (fileselection_gui), "filesel", "gnochive");
	
	
	fileselection_ok = GNOME_FILELIST (fileselection_gui)->ok_button;
	gtk_widget_show (fileselection_ok);
	GTK_WIDGET_SET_FLAGS (fileselection_ok, GTK_CAN_DEFAULT);

	fileselection_cancel = GNOME_FILELIST (fileselection_gui)->cancel_button;
	gtk_widget_show (fileselection_cancel);
	GTK_WIDGET_SET_FLAGS (fileselection_cancel, GTK_CAN_DEFAULT);
	
	//gtk_accel_group_attach (myapp->accel_group,
				//GTK_OBJECT (fileselection_gui));

	gtk_signal_connect (GTK_OBJECT (fileselection_gui), "delete_event",
			    GTK_SIGNAL_FUNC (on_fileselection_delete_event),
			    myfs);
	gtk_signal_connect (GTK_OBJECT (fileselection_ok), "clicked",
			    GTK_SIGNAL_FUNC (on_file_selection_ok_clicked),
			    myfs);
	gtk_signal_connect (GTK_OBJECT (fileselection_cancel), "clicked",
			    GTK_SIGNAL_FUNC (myfs->click_cancel_callback),
			    myfs->data);
	gtk_signal_connect (GTK_OBJECT (fileselection_gui), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_unref), NULL);

	myfs->filesel = fileselection_gui;
	gtk_widget_ref (fileselection_gui);
	return fileselection_gui;
}

GtkWidget*
create_dir_selection(DirSelData *dirsel)
{
	GtkWidget *dir_dialog;
	GtkWidget *dir_ok;
	GtkWidget *dir_cancel;
	
	dir_dialog = gnome_dirlist_new ();
	gtk_container_set_border_width (GTK_CONTAINER (dir_dialog), 10);
	gtk_window_set_position (GTK_WINDOW (dir_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_wmclass (GTK_WINDOW (dir_dialog), "dirsel", "gnochive");
	
	
	dir_ok = GNOME_DIRLIST (dir_dialog)->ok_button;
	gtk_widget_show (dir_ok);
	GTK_WIDGET_SET_FLAGS (dir_ok, GTK_CAN_DEFAULT);
	
	dir_cancel = GNOME_DIRLIST (dir_dialog)->cancel_button;
	gtk_widget_show (dir_cancel);
	GTK_WIDGET_SET_FLAGS (dir_cancel, GTK_CAN_DEFAULT);
	
	gtk_signal_connect (GTK_OBJECT (dir_dialog), "delete_event",
			    GTK_SIGNAL_FUNC (on_dirsel_delete_event),
			    dirsel);
	gtk_signal_connect (GTK_OBJECT (dir_ok), "clicked",
			    GTK_SIGNAL_FUNC (dirsel->click_ok_callback),
			    dirsel->data);
	gtk_signal_connect (GTK_OBJECT (dir_cancel), "clicked",
			    GTK_SIGNAL_FUNC (dirsel->click_cancel_callback),
			    dirsel->data);
	gtk_signal_connect (GTK_OBJECT (dir_dialog), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_unref), NULL);
	
	dirsel->dirsel = dir_dialog;
	gtk_widget_ref (dir_dialog);
	return dir_dialog;
}

/* draws gnochive buttons
 * style:
 * 0 for original gnochive buttons
 * 1 for GNOME buttons
 * crt
 * 0 - update existing toolbar
 * 1 - create a new toolbar
 * ***Add the skins stuff***
 */
 
gboolean
draw_toolbar_buttons			(guint type,guint style,gboolean crt)
{
	GtkWidget *tmp_but;
	GtkWidget *box;
	GtkWidget *toolbar;
	GtkWidget *ext_butt;
	//gchar *ac_type;
	//gchar *text;
	
	int i;
	
	static gchar *b_but[] = {"button7", "button8", "button9", "button10",
							"button83", "button11", "button12", "button17",
							"button77","button18","button13"};
	static char *stocks[]= {GNOME_STOCK_PIXMAP_ADD,GNOME_STOCK_PIXMAP_OPEN,
			GNOME_STOCK_PIXMAP_CONVERT,GNOME_STOCK_PIXMAP_INDEX,
			GNOME_STOCK_PIXMAP_REFRESH,GNOME_STOCK_PIXMAP_SEARCH,
			GNOME_STOCK_PIXMAP_TRASH,GNOME_STOCK_PIXMAP_PROPERTIES,
			/*GNOME_STOCK_PIXMAP_EXEC,*/ GNOME_STOCK_PIXMAP_HELP,
			GNOME_STOCK_PIXMAP_QUIT};
	static char *orig_24[] = {"gnochive/add-24.xpm","gnochive/open-24.xpm","gnochive/extr-24.xpm",
			"gnochive/tools-24.xpm","gnochive/refresh-24.xpm","gnochive/myview-24.xpm",
			"gnochive/delete-24.xpm","gnochive/tag-24.xpm",/*"gnochive/binst-24.xpm",*/
			"gnochive/ghelp-24.xpm","gnochive/sd-24.xpm"};
	static char *orig[] = {"gnochive/add.xpm","gnochive/open.xpm","gnochive/extr.xpm",
			"gnochive/tools.xpm","gnochive/refresh.xpm","gnochive/myview.xpm",
			"gnochive/delete.xpm","gnochive/tag-next.xpm",/*"gnochive/binst.xpm",*/
			"gnochive/ghelp.xpm","gnochive/sd.xpm"};
	gchar *texts[] = {_("Add"),_("Open"),_("Extract"),_("Test"),_("Refresh"),_("View"),
			_("Delete"),_("Options"), /*_("Build"),*/ _("Help"), _("Quit")};
	gchar *tips[] = {_("Create archive with selected files"),_("Open archive"),
		_("Extract selected files from archive to current path"),
		_("Test selected archive(s)"),_("Refresh current directory/archive"),
		_("View selected file(s)"),_("Delete selected file(s)"),
		_("gnochive customization"), /*_("Build archive installer"),*/
		_("gnome Help"),_("Out'n'away")};
	gpointer *cdata[] = {NULL, (gpointer *)gnochivemain, NULL, NULL, NULL, NULL,
						NULL, NULL, /*NULL,*/ NULL, NULL};
	static GtkSignalFunc func[] = {on_button7_clicked,gnochive_open_dialog,on_button9_clicked,
				on_button10_clicked,on_button83_clicked,on_button11_clicked,
				on_button12_clicked,on_button17_clicked,/*on_button77_clicked,*/
				on_button18_clicked,on_button13_clicked};
	static GtkSignalFunc func1[] = {on_button7_enter,on_button8_enter,on_button9_enter,
				on_button10_enter,on_button83_enter,on_button11_enter,
				on_button12_enter,on_button17_enter, /*on_button77_enter,*/
				on_button18_enter,on_button13_enter};
	static GtkSignalFunc func2[] = {on_button7_leave,on_button8_leave,on_button9_leave,
				on_button10_leave,on_button83_leave,on_button11_leave,
				on_button12_leave,on_button17_leave, /*on_button77_leave,*/
				on_button18_leave,on_button13_leave};
				
	toolbar = lookup_widget(GTK_WIDGET(gnochivemain->mainwin), "toolbar1");
				
	for (i=0;i<=9;i++) {
		if (!crt) {
    		tmp_but = lookup_widget(GTK_WIDGET(gnochivemain->mainwin), b_but[i]);
    		gtk_widget_hide(tmp_but);
		}
		switch (type) {
    		case 2: box=gnome_stock_pixmap_widget(gnochivemain->mainwin,stocks[i]);break;
    		case 1: box=create_pixmap(gnochivemain->mainwin,orig[i],TRUE);break;
			case 0: box=create_pixmap(gnochivemain->mainwin,orig_24[i],TRUE);break;
			default: box=create_pixmap(gnochivemain->mainwin,orig_24[i],TRUE); 
		}
		
		tmp_but=gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),GTK_TOOLBAR_CHILD_BUTTON,
				NULL,texts[i],tips[i],NULL,box,
				NULL,NULL);
    	gtk_widget_set_name(tmp_but,b_but[i]);
    	gtk_widget_ref(tmp_but);
    	gtk_object_set_data_full(GTK_OBJECT(gnochivemain->mainwin),b_but[i],tmp_but,
				(GtkDestroyNotify) gtk_widget_unref);
    	gtk_widget_show(tmp_but);
    	/* restore original signals */
    	gtk_signal_connect(GTK_OBJECT(tmp_but),"clicked",
				func[i],cdata[i]);
    	gtk_signal_connect(GTK_OBJECT(tmp_but),"enter",
				func1[i],NULL);
    	gtk_signal_connect(GTK_OBJECT(tmp_but),"leave",
				func2[i],NULL);
    	//if (i==3 || i==6 || i==8) gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar),2);
}
gtk_widget_hide(dock1);
gtk_widget_show(dock1);
switch (style) {
	case 0	:	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);break;
	case 1	:	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);break;
	case 2	:	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);break;
	case 3	:	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);break;
	case 0x1FF	:	break;
	default	:	break;
}
//gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), last_select_row, 1, &text);
ext_butt = lookup_widget(GTK_WIDGET(toolbar), "button9");
//ac_type = gnochive_get_archive_type(text);
//if (ac_type)
	//gtk_widget_set_sensitive(GTK_WIDGET(ext_butt), TRUE);
//else
	//gtk_widget_set_sensitive(GTK_WIDGET(ext_butt), FALSE);
gtk_widget_set_sensitive(GTK_WIDGET(ext_butt), FALSE);
ext_butt = lookup_widget(GTK_WIDGET(toolbar), "button10");
gtk_widget_set_sensitive(GTK_WIDGET(ext_butt), FALSE);
return style;
}

void cancel_fs_func(GtkButton *button, gpointer data)
{
	gtk_widget_hide(gnochivemain->op_filesel);
}

void cancel_dir_func(GtkButton *button, gpointer data)
{
	gtk_widget_hide(gnochivemain->dirsel);
}

void myfn(GtkButton *button, gpointer data)
{
	gtk_widget_hide(gnochivemain->op_filesel);
}


void
gnochive_open_dialog                   (GtkButton       *button,
                                        gpointer         user_data)
{
	gchivemain *gc = user_data;
	
	gtk_widget_show(gc->op_filesel);
}

void
gnochive_error							(gint errnum,gchar *err_msg, ...)
{
	gchar *mymsg;
	gchar *msg;
	va_list arg;
	va_start(arg,err_msg);
	mymsg=g_strdup_vprintf(err_msg,arg);
	msg=g_strconcat(mymsg,_("ERROR: "),g_strerror(errnum),NULL);
	va_end(arg);
	gnome_app_error(GNOME_APP(gnochivemain->app),msg);
	g_free(msg);
	g_free(mymsg);
}

void
gnochive_gerror							(gint errnum,gchar *err_msg, ...)
{
	gchar *mymsg;
	gchar *msg=NULL;
	va_list arg;
	va_start(arg,err_msg);
	mymsg=g_strdup_vprintf(err_msg,arg);
	//msg=g_strconcat(mymsg,_("ERROR: "),g_file_error_from_errno(errnum),NULL);
	va_end(arg);
	gnome_app_error(GNOME_APP(gnochivemain->app),msg);
	g_free(msg);
	g_free(mymsg);
}

GtkWidget*
create_pobox3 (void)
{
	GtkWidget *myprops;
	myprops = create_propertybox3();
	gnochivemain->gc_cfg->widget = myprops;
	
	entry3=lookup_widget(GTK_WIDGET(myprops),"entry32");
	entrygzip=lookup_widget(GTK_WIDGET(myprops),"entry33");
	entrytar=lookup_widget(GTK_WIDGET(myprops),"entry35");
	entryzip=lookup_widget(GTK_WIDGET(myprops),"entry34");
	entryunzip = lookup_widget(GTK_WIDGET(myprops),"entry63");
	entrybzip2 = lookup_widget(GTK_WIDGET(myprops),"entry36");
	entryZ = lookup_widget(GTK_WIDGET(myprops),"entry45");
	entryarj = lookup_widget(GTK_WIDGET(myprops),"entry48");
	entryace = lookup_widget(GTK_WIDGET(myprops),"entry49");
	entrylha = lookup_widget(GTK_WIDGET(myprops),"entry50");
	entryzoo = lookup_widget(GTK_WIDGET(myprops),"entry51");
	entryrpm = lookup_widget(GTK_WIDGET(myprops),"entry62");
	entrycpio = lookup_widget(GTK_WIDGET(myprops),"entry65");
	entryrpm2cpio = lookup_widget(GTK_WIDGET(myprops),"entry66");
	draw_toolbar_buttons(gnochivemain->gc_cfg->gbstat,
						gnochivemain->gc_cfg->tstyle, 1);
	update_propertybox3(myprops);
	update_all(myprops);
	return myprops;
}

void
update_all(GtkWidget *propbox3)
{
	GtkWidget 	*checkbutton20, *checkbutton25, *checkbutton58,
				*cb65, *cb94;
	gchar *sts[] = {"label194", "label199", "label205", "label206",
					"label206", "label209",	"label220", "label240",
					"label239", "label244", "label243", "label286",
					"label337", "label340"};
	gchar *paths[] = {gnochivemain->gc_cfg->archs.rar_path,
						gnochivemain->gc_cfg->archs.gzip_path,
						gnochivemain->gc_cfg->archs.tar_path,
						gnochivemain->gc_cfg->archs.zip_path,
						gnochivemain->gc_cfg->archs.unzip_path, gnochivemain->gc_cfg->archs.bz2_path,
						gnochivemain->gc_cfg->archs.Z_path,	gnochivemain->gc_cfg->archs.unarj_path,
						gnochivemain->gc_cfg->archs.unace_path,	gnochivemain->gc_cfg->archs.lha_path,
						gnochivemain->gc_cfg->archs.zoo_path, gnochivemain->gc_cfg->archs.rpm_path,
						gnochivemain->gc_cfg->archs.cpio_path, gnochivemain->gc_cfg->archs.rpm2cpio_path};
	GtkWidget *ents[] = {entry3, entrygzip, entrytar, entryzip, entryunzip, entrybzip2,
						entryZ, entryarj, entryace, entrylha, entryzoo, entryrpm, entrycpio,
						entryrpm2cpio};
	GtkWidget *st[11];
	guint i;
	checkbutton20 = lookup_widget(GTK_WIDGET(propbox3),"checkbutton61");
	checkbutton25 = lookup_widget(GTK_WIDGET(propbox3),"checkbutton60");
	checkbutton58 = lookup_widget(GTK_WIDGET(propbox3),"checkbutton58");
	cb65 = lookup_widget(GTK_WIDGET(propbox3),"checkbutton65");
	cb94 = lookup_widget(GTK_WIDGET(propbox3),"checkbutton94");
	gnochivemain->gc_cfg->complete = FALSE;
	for (i=0;i<=13;i++)
	{
		st[i] = lookup_widget(GTK_WIDGET(propbox3), sts[i]);
		display_arc_status(paths[i], GTK_ENTRY(ents[i]), GTK_LABEL(st[i]));
	}
	/* zip/unzip display*/
	display_dual_status(paths[3], paths[4], (GtkEntry *)entryzip,
			(GtkEntry *)entryunzip, GTK_LABEL(st[4]));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb94), 
			gnochivemain->gc_cfg->tit);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton20),
			gnochivemain->gc_cfg->res_syms);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton25),
			gnochivemain->gc_cfg->popn);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton58),
			gnochivemain->gc_cfg->bz2flag);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb65),
			gnochivemain->gc_cfg->internal);
	display_cclist(propbox3,gnochivemain->gc_cfg->cclist_val, 0);
	draw_toolbar_buttons(gnochivemain->gc_cfg->gbstat,
				gnochivemain->gc_cfg->tstyle, 0);
	gtk_notebook_set_tab_pos((GtkNotebook *)gnochivemain->noter, 
				gnochivemain->gc_cfg->tbstyle);
	gnochivemain->gc_cfg->complete = TRUE;
}

void
update_propertybox3(GtkWidget *propbox3)
{
	GtkRadioButton *radio;
	radio = (GtkRadioButton *)lookup_widget(propbox3, "radiobutton29");
	gtk_radio_button_set(radio, gnochivemain->gc_cfg->gbstat);
}

void
save_changes(void)
{
	struct passwd *pw;
	gchar *path = NULL;
	FILE *cfgfile;

	g_free(gnochivemain->gc_cfg->archs.rar_path);
	gnochivemain->gc_cfg->archs.rar_path = parse_arc_path(entry3);
	g_free(gnochivemain->gc_cfg->archs.gzip_path);
	gnochivemain->gc_cfg->archs.gzip_path = parse_arc_path(entrygzip);
	g_free(gnochivemain->gc_cfg->archs.tar_path);
	gnochivemain->gc_cfg->archs.tar_path = parse_arc_path(entrytar);
	g_free(gnochivemain->gc_cfg->archs.zip_path);
	gnochivemain->gc_cfg->archs.zip_path = parse_arc_path(entryzip);
	g_free(gnochivemain->gc_cfg->archs.unzip_path);
	gnochivemain->gc_cfg->archs.unzip_path = parse_arc_path(entryunzip);
	
	/*obsoleted due to the fact that there where freebsd distribution
	 *we had zip & unzip in differrent paths there
	 */
	//if	(!cmpare(gnochivemain->gc_cfg->archs.unzip_path,"zip")) //if no zip like text was parsed we do not interfer
	//{
		//ugly patch for "un" (zip) part wouldn`t make much sense to add another input just for unzip
		//strcpy(temp,"un");
		//strcat(temp,strstr((gnochivemain->gc_cfg->archs.unzip_path),"zip"));
		//strcpy(strstr((gnochivemain->gc_cfg->archs.unzip_path),"zip"),temp);
	//}
	g_free(gnochivemain->gc_cfg->archs.bz2_path);
	gnochivemain->gc_cfg->archs.bz2_path = parse_arc_path(entrybzip2);
	g_free(gnochivemain->gc_cfg->archs.Z_path);
	gnochivemain->gc_cfg->archs.Z_path = parse_arc_path(entryZ);
	g_free(gnochivemain->gc_cfg->archs.unarj_path);
	gnochivemain->gc_cfg->archs.unarj_path = parse_arc_path(entryarj);
	g_free(gnochivemain->gc_cfg->archs.unace_path);
	gnochivemain->gc_cfg->archs.unace_path = parse_arc_path(entryace);
	g_free(gnochivemain->gc_cfg->archs.lha_path);
	gnochivemain->gc_cfg->archs.lha_path=parse_arc_path(entrylha);
	g_free(gnochivemain->gc_cfg->archs.zoo_path);
	gnochivemain->gc_cfg->archs.zoo_path=parse_arc_path(entryzoo);
	g_free(gnochivemain->gc_cfg->archs.rpm_path);
	gnochivemain->gc_cfg->archs.rpm_path = parse_arc_path(entryrpm);
	g_free(gnochivemain->gc_cfg->archs.cpio_path);
	gnochivemain->gc_cfg->archs.cpio_path = parse_arc_path(entrycpio);
	g_free(gnochivemain->gc_cfg->archs.rpm2cpio_path);
	gnochivemain->gc_cfg->archs.rpm2cpio_path = parse_arc_path(entryrpm2cpio);
	remove(gnochivemain->gc_cfg->file_name);
	cfgfile=fopen(gnochivemain->gc_cfg->file_name,"w");
	if (cfgfile)
	{
		fprintf(cfgfile, "[%s]\n",GNOCHIVE_VER);
		fprintf(cfgfile, "#archivers path\n");
		
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.rar_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.gzip_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.tar_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.zip_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.unzip_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.bz2_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.Z_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.unarj_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.unace_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.lha_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.zoo_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.rpm_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.cpio_path);
		write_arc(cfgfile, gnochivemain->gc_cfg->archs.rpm2cpio_path);
		
		fprintf(cfgfile, "#interface options\n");
		if (gnochivemain->gc_cfg->tit) fprintf(cfgfile,"show-titles=yes\n");
		else fprintf(cfgfile,"show-titles=no\n");
		if (gnochivemain->gc_cfg->res_syms) fprintf(cfgfile,"follow-syms=yes\n");
		else fprintf(cfgfile,"follow-syms=no\n");
		if (gnochivemain->gc_cfg->popn) fprintf(cfgfile,"post-open=yes\n");
		else fprintf(cfgfile,"post-open=no\n");
		fprintf(cfgfile, "toolbar-style=%d,%d\n", gnochivemain->gc_cfg->gbstat,
						gnochivemain->gc_cfg->tstyle);
		fprintf(cfgfile,"tabs-style=%d\n", gnochivemain->gc_cfg->tbstyle);
		fprintf(cfgfile,"sel-type=single\n");
		if (gnochivemain->gc_cfg->internal) fprintf(cfgfile,"internal-viewer=yes\n");
		else fprintf(cfgfile,"internal-viewer=no\n");
		fprintf(cfgfile,"#working directory (used for overwriting the path parsed by nautilus)\n");
		fprintf(cfgfile,"#by default we use our $HOME dir\n");
		
		pw=getpwuid(getuid());
		path=g_strdup(pw->pw_dir);
		fprintf(cfgfile,"own-working-dir=%s\n",path);
		fprintf(cfgfile,"#compat flags\n");
		if (gnochivemain->gc_cfg->bz2flag)
			fprintf(cfgfile, "bz2_j_flag=yes\n");
		else
			fprintf(cfgfile, "bz2_j_flag=no\n");
		if (gnochivemain->gc_cfg->bz2flag)
			gnochivemain->gc_cfg->tbz2parm = g_strdup("-j");
		else
			gnochivemain->gc_cfg->tbz2parm = g_strdup("-I");
		if (gnochivemain->gc_cfg->rpm_handle)
			fprintf(cfgfile, "rpm_handle=1\n");
		else
			fprintf(cfgfile, "rpm_handle=0\n");
		gnochivemain->gc_cfg->cclist_val = apply_cclist();
		fprintf(cfgfile, "clist=%d\n", gnochivemain->gc_cfg->cclist_val);
		update_all(gnochivemain->gc_cfg->widget);
		fclose(cfgfile);
	}
	g_free(path);
}
	
void
display_arc_status			(gchar *path, GtkEntry *entry, GtkLabel *label)
{
	int stat = 0;
	
	gtk_entry_set_text(entry, path);
	if ((stat = check_arc(path)))
		gtk_label_set_text(label, _("OK"));
	else
		gtk_label_set_text(label, _("NOT FOUND!"));
}

void
display_dual_status			(gchar *path, gchar *path2, GtkEntry *entry, GtkEntry *entry2, GtkLabel *label)
{
	int stat = 0;
	int stat2 = 0;
	gtk_entry_set_text(entry, path);
	gtk_entry_set_text(entry2, path2);
	if ((stat = check_arc(path)) && (stat2 = check_arc(path2)))
		gtk_label_set_text(label, _("OK"));
	else
		gtk_label_set_text(label, _("NOT FOUND!"));
}

void
recheck_status			(gchar *path, GtkLabel *label)
{
	int stat = 0;
	if ((stat = check_arc(path)))
		gtk_label_set_text(label, _("OK"));
	else
		gtk_label_set_text(label, _("NOT FOUND!"));
}

/* !!! before calling this function g_free the result that'll return */
gchar*
parse_arc_path	(GtkWidget *entry)
{
	return g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
write_arc	(FILE *cfgfile,gchar *path)
{
	fwrite(path,strlen(path),1,cfgfile);
	fprintf(cfgfile,"\n");
}

/* 
 * Checks for a valid arch file
 */
 
int
check_arc			(gchar *path)
{
int arc_ok=0;
FILE *myfile;
gchar *buff;
buff=g_strdup(path);
if ((myfile=fopen(buff,"r"))!=NULL)
 {
    arc_ok=1;
    fclose(myfile);
 }
g_free(buff);
return arc_ok;
}

void
display_cclist			(GtkWidget *props,guint cclist_val,guint stat)
{
	guint myres;
	guint i;
	gint j=6;
	gchar *btstr[] = {"checkbutton86","checkbutton87","checkbutton88",
		    "checkbutton89","checkbutton90","checkbutton91","checkbutton93"};
			
if (!stat)
{
    i=0;
    while (j>=0)
    {
		myres=pow(2,j);
		myres=(cclist_val & myres) >> j;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(props,btstr[i])),myres);
		j--;i++;
    }
}
}

void
show_cclist			(GtkWidget *clist1,guint stat)
{
	guint myres;
	gint j=6, q=2;
	
	while (j>=0)
	{
		myres = pow(2, j);
		myres = (stat & myres) >> j;
		gtk_clist_set_column_visibility(GTK_CLIST(clist1), q, myres);
		j--;q++;
	}
}

gint
apply_cclist			(void)
{
int st,j=6,i;
GtkWidget *w;
guint val=0,val1;
gchar *btstr[] = {"checkbutton86","checkbutton87","checkbutton88",
		    "checkbutton89","checkbutton90","checkbutton91","checkbutton93"};
for (i=0;i<=6;i++) {
 	w=lookup_widget(gnochivemain->gc_cfg->widget,btstr[i]);
 	st=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) ? TRUE : FALSE;
	if (st) {
		val1=pow(2,j);
 		val=val ^ val1;
	}
 	j--;
}
return val;
}

GtkWidget*
create_clist(GtkWidget *parent)
{
	GtkWidget *scrolledwindow;
	GtkWidget *clist;
	GtkWidget *pixmap;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *label4;
	GtkWidget *label5;
	GtkWidget *label6;
	GtkWidget *label7;
	GtkWidget *label8;
	GtkAccelGroup *accel_group;
	
	accel_group = gtk_accel_group_new ();
	
	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  	gtk_widget_set_name (scrolledwindow, "scrolledwindow");
  	gtk_widget_ref (scrolledwindow);
	gtk_object_set_data_full (GTK_OBJECT (parent), "scrolledwindow", scrolledwindow,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow);
	gtk_container_add (GTK_CONTAINER (parent), scrolledwindow);
	//gnome_app_set_contents (GNOME_APP (parent), scrolledwindow);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  	clist = gtk_clist_new (9);
	gtk_widget_set_name (clist, "clist");
	gtk_widget_ref (clist);
  	gtk_object_set_data_full (GTK_OBJECT (parent), "clist", clist,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_set_events (clist, GDK_BUTTON_MOTION_MASK | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
	gtk_widget_set_extension_events (clist, GDK_EXTENSION_EVENTS_ALL);
  	gtk_widget_show (clist);
  	gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
	
  	
	//gtk_widget_add_accelerator (clist, "select_all", accel_group,
                              //GDK_A, GDK_CONTROL_MASK,
                              //GTK_ACCEL_VISIBLE);
	gtk_clist_set_column_width (GTK_CLIST (clist), 0, 17);
	gtk_clist_set_column_width (GTK_CLIST (clist), 1, 167);
	gtk_clist_set_column_width (GTK_CLIST (clist), 2, 29);
	gtk_clist_set_column_width (GTK_CLIST (clist), 3, 80);
	gtk_clist_set_column_width (GTK_CLIST (clist), 4, 83);
	gtk_clist_set_column_width (GTK_CLIST (clist), 5, 162);
	gtk_clist_set_column_width (GTK_CLIST (clist), 6, 94);
	gtk_clist_set_column_width (GTK_CLIST (clist), 7, 83);
	gtk_clist_set_column_width (GTK_CLIST (clist), 8, 32);
	
	gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_EXTENDED);
	gtk_clist_column_titles_show (GTK_CLIST (clist));
	gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_ETCHED_OUT);

  	pixmap = create_pixmap (parent, "gnochive/down.xpm", FALSE);
  	gtk_widget_set_name (pixmap, "pixmap1");
  	gtk_widget_ref (pixmap);
  	gtk_object_set_data_full (GTK_OBJECT (parent), "pixmap", pixmap,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (pixmap);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 0, pixmap);
	gtk_pixmap_set_build_insensitive (GTK_PIXMAP (pixmap), FALSE);

	label1 = gtk_label_new (_("Name"));
	gtk_widget_set_name (label1, "label1");
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label1);

  	label2 = gtk_label_new (_("Type"));
	gtk_widget_set_name (label2, "label2");
	gtk_widget_ref (label2);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label2);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label2);

	label3 = gtk_label_new (_("Size"));
	gtk_widget_set_name (label3, "label3");
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label3", label3,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 3, label3);
	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

	label4 = gtk_label_new (_("Packed"));
	gtk_widget_set_name (label4, "label4");
	gtk_widget_ref (label4);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label4);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 4, label4);

	label5 = gtk_label_new (_("Date"));
	gtk_widget_set_name (label5, "label5");
	gtk_widget_ref (label5);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label5", label5,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label5);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 5, label5);

	label6 = gtk_label_new (_("Attr"));
	gtk_widget_set_name (label6, "label6");
	gtk_widget_ref (label6);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label6", label6,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label6);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 6, label6);

	label7 = gtk_label_new (_("CRC"));
	gtk_widget_set_name (label7, "label7");
	gtk_widget_ref (label7);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label7", label7,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label7);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 7, label7);

	label8 = gtk_label_new (_("Ratio"));
	gtk_widget_set_name (label8, "label8");
	gtk_widget_ref (label8);
	gtk_object_set_data_full (GTK_OBJECT (parent), "label8", label8,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label8);
	gtk_clist_set_column_widget (GTK_CLIST (clist), 8, label8);
	
	gtk_signal_connect (GTK_OBJECT (clist), "select_all",
                      GTK_SIGNAL_FUNC (on_clist1_select_all),
                      NULL);
	srw = gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                      GTK_SIGNAL_FUNC (on_clist1_select_row),
                      NULL);
  	gtk_signal_connect (GTK_OBJECT (clist), "unselect_all",
                      GTK_SIGNAL_FUNC (on_clist1_unselect_all),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "unselect_row",
                      GTK_SIGNAL_FUNC (on_clist1_unselect_row),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "button-press-event",
                      GTK_SIGNAL_FUNC (on_clist1_button_press_event),//gnochive_clist_button_press),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "add",
                      GTK_SIGNAL_FUNC (on_clist1_add),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "key_release_event",
                      GTK_SIGNAL_FUNC (on_clist1_key_release_event),
                      NULL);
	gtk_widget_grab_focus (clist);
	show_cclist(clist, gnochivemain->gc_cfg->cclist_val);
	return clist;
}

void
clist_drag_link (GtkCList *clist, GtkSignalFunc recv_func)
{
	gtk_signal_connect (GTK_OBJECT (clist), "drag_data_received",
                      recv_func,
                      clist);
	gtk_signal_connect (GTK_OBJECT (clist), "drag_data_get",
                      GTK_SIGNAL_FUNC (on_clist1_drag_data_get),
                      clist);
	gtk_signal_connect (GTK_OBJECT (clist), "drag_begin",
                      GTK_SIGNAL_FUNC (gnochive_clist_drag_begin),
                      clist);
	gtk_signal_connect (GTK_OBJECT (clist), "drag_end",
                      GTK_SIGNAL_FUNC (gnochive_clist_drag_end),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (clist), "drag_motion",
                      GTK_SIGNAL_FUNC (gnochive_clist_drag_motion),
                      NULL);
	/*gtk_signal_connect (GTK_OBJECT (clist), "drag_drop",
                      GTK_SIGNAL_FUNC (gnochive_clist_drag_drop),
                      NULL);*/
}

GtkWidget*
create_note(GtkWidget *parent)
{
	GtkWidget *notebook;
	
	notebook = gtk_notebook_new ();
	gtk_widget_set_name (notebook, "notebook");
	gtk_widget_ref (notebook);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook),TRUE);
	gtk_notebook_popup_enable(GTK_NOTEBOOK(notebook));
	gtk_object_set_data_full (GTK_OBJECT (parent), "notebook", notebook,
							(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (notebook);
	gnome_app_set_contents (GNOME_APP (parent), notebook);
	gtk_signal_connect (GTK_OBJECT (notebook), "switch_page",
                      GTK_SIGNAL_FUNC (on_notebook_switch_page),
                      NULL);
	
	return notebook;
}

GtkWidget*
add_page(pager *page, GtkWidget *parent, GtkWidget *notebook, gchar *lbl)
{
	GtkWidget *empty_notebook_page = NULL;
	GtkWidget *label, *event;
	GtkWidget *pb, *pb1;
	GtkWidget *button;
	GtkRequisition r;
	GtkStyle *style;
	GdkColor color;
	
	pb = create_pixmap(notebook, "gnochive/close_arch.xpm", FALSE);
	gtk_widget_show (pb);
	button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(button), pb);
	
	pb1 = create_pixmap(notebook, "gnochive/close_arch.xpm", FALSE);
	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
	gtk_widget_size_request(button, &r);
	gtk_widget_set_usize(pb1, r.width, r.height);
	gtk_widget_set_sensitive(pb1, FALSE);
	label = gtk_label_new(lbl);
	gtk_widget_show(label);
	color.red = 0;
	color.green = 0;
	color.blue = 0;
	style = gtk_widget_get_style(button);
	style->fg[GTK_STATE_NORMAL] = color;
	gtk_widget_set_style(button, style);
	
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
						GTK_SIGNAL_FUNC(on_notebook_close_page),
						empty_notebook_page);
	
	empty_notebook_page = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(empty_notebook_page), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(empty_notebook_page), button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(empty_notebook_page), pb1, FALSE, FALSE, 0);
	gtk_widget_show (empty_notebook_page);
	
	event = gtk_event_box_new();
	gtk_widget_show(event);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), event, empty_notebook_page);
	
	page->bt = button;
	page->bt_gr = pb1;
	return event;
}

/* get total number of pages */

guint
gnochive_notebook_get_pages (pager *my_page)
{
	guint pages = 0;
	pager *page;
	for (page = my_page ; page->next != NULL ; page = (pager *)page->next)
		pages++;
	return pages++;
}

void
on_notebook_close_page (GtkNotebook *notebook,
						GtkNotebookPage *page,
						gint page_num, gpointer user_data)
{
	gnochive_close_archive(gnochivemain->active_page->ad);
}

pager*
gnochive_pager_init(void)
{
	pager *gc_pager = (pager *)g_malloc0(sizeof(pager));
	
	if (gc_pager)
	{
		gc_pager->page_no = 0;
		gc_pager->path = g_get_current_dir();
		gc_pager->opd_archive = FALSE;
		gc_pager->next = NULL;
		gnochivemain->gc_pager = gnochivemain->active_page = gc_pager;
		gnochive_draw_pager(gc_pager);
		create_clist_pixmaps(gc_pager->clist);
		gtk_clist_set_reorderable(GTK_CLIST(gc_pager->clist), TRUE);
		gtk_clist_set_use_drag_icons(GTK_CLIST(gc_pager->clist), TRUE);
		//setting the DND
		gtk_drag_source_set(GTK_WIDGET(gc_pager->clist), GDK_BUTTON1_MASK | 
							   GDK_BUTTON2_MASK |
							   GDK_BUTTON3_MASK, 
							possible_drag_types, n_possible_drag_types,
							GDK_ACTION_COPY | GDK_ACTION_MOVE);
		gtk_drag_dest_unset(GTK_WIDGET(gc_pager->clist));
		gtk_drag_dest_set(gc_pager->clist,
								GTK_DEST_DEFAULT_ALL, possible_drag_types,
								n_possible_drag_types,
								GDK_ACTION_COPY |
								GDK_ACTION_MOVE |
								GDK_ACTION_ASK);
		show_cclist(gc_pager->clist,gnochivemain->gc_cfg->cclist_val);
		gc_pager->bt = NULL;
		gc_pager->bt_gr = NULL;
		return gc_pager;
	}
	return NULL;
}

void
gnochive_draw_pager(pager *gc_pager)
{
	GtkWidget *pixmap;
	pager *pg=gc_pager;

	gnochivemain->noter=create_note(gnochivemain->mainwin);
	pixmap = create_pixmap (gnochivemain->mainwin, "gnochive/dir.xpm", FALSE);
  	gtk_widget_set_name (pixmap, "pixmap1");
  	gtk_widget_ref (pixmap);
  	gtk_widget_show (pixmap);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(gnochivemain->noter),FALSE);
	pg->page = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (pg->page);
	gtk_container_add (GTK_CONTAINER (gnochivemain->noter), pg->page);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (gnochivemain->noter),
			gtk_notebook_get_nth_page(GTK_NOTEBOOK(gnochivemain->noter),0),
			pixmap);
	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (gnochivemain->noter),
			gtk_notebook_get_nth_page(GTK_NOTEBOOK(gnochivemain->noter),0),
			N_("Files"));
	pg->clist = create_clist(pg->page);
	clist_drag_link(GTK_CLIST(pg->clist),
			GTK_SIGNAL_FUNC(on_mclist_drag_data_received));
	gnochivemain->active_page = pg;
	gnochivemain->active_page->path = g_get_current_dir();
	
}

pager*
gnochive_pager_add(gchar *label)
{
	pager *my;
	
	pager *pg = (pager *)g_malloc0(sizeof(pager));
	
	if (pg) {
		
		my = gnochivemain->gc_pager;
		
		if (my->next==NULL)
			my->next = (gpointer *)pg;
		else {
			while (my->next!=NULL)
			{
				//if (!my->next)
					//my->next=(gpointer *)pg;
				//else
					my=(pager *)my->next;
			}
			my->next = (gpointer *)pg;
		}
		
		pg->page = add_page(pg, gnochivemain->mainwin, gnochivemain->noter, label);
		gtk_widget_show(pg->bt);
		pg->clist = create_clist(pg->page);
		clist_drag_link(GTK_CLIST(pg->clist),
				GTK_SIGNAL_FUNC(on_aclist_drag_data_received));
		gtk_drag_dest_set(pg->clist,
								GTK_DEST_DEFAULT_ALL, possible_drag_types,
								n_possible_drag_types,
								GDK_ACTION_COPY |
								GDK_ACTION_MOVE |
								GDK_ACTION_ASK);
		pg->next = NULL;
	}
	return pg;
}

/* finds the pager structure for a given page_no */

pager*
gnochive_dig_pager(gint page_no)
{
	pager *my=gnochivemain->gc_pager;
	
	while(my!=NULL)
	{
		if (my->page_no==page_no)
			return my;
		else
			my=(pager *)my->next;
	}
	return (pager *)NULL;
}

/* remove a page from the pager list
 * preserving the countinuity of the others
 */

gboolean
gnochive_page_destroy(pager *top_page,pager *del_page)
{
	pager *temp;
	pager *top=top_page;
	while (top->next!=(gpointer *)del_page)
		top=(pager *)top->next;
	if ((gpointer *)top->next!=(gpointer *)del_page)
		return FALSE;	// couldn't free page
	else {
		temp=(pager *)top->next;		//del page
		if (temp->next)
			top->next=temp->next;
		else top->next=NULL;
		g_free(del_page);
		return TRUE;
	}
	return TRUE;
}

GdkPixmap 	*dirpix,*docpix,*xpmpix,*makpix;
GdkPixmap	*rpmpix,*comppix,*dotpix,*lockpix;
GdkPixmap *hdrpix,*sympix,*rdpix,*rpm1pix;

GdkBitmap 	*dirmask,*docmask,*xpmmask,*makmask;
GdkBitmap	*rpmmask,*compmask,*dotmask,*lockmask;
GdkBitmap *hdrmask,*symmask,*rdmask,*rpm1mask;

void
create_clist_pixmaps		(GtkWidget *clist1)
{
	open_xpm("dir.xpm",clist1,&dirpix,&dirmask,1);
	open_xpm("file.xpm",clist1,&docpix,&docpix,1);
	open_xpm("dotdot.xpm",clist1,&dotpix,&dotmask,1);
	open_xpm("rpm1.xpm",clist1,&rpmpix,&rpmmask,1);
	open_xpm("xpm.xpm",clist1,&xpmpix,&xpmmask,1);
	open_xpm("sym.xpm",clist1,&sympix,&symmask,1); // FIX-ME!!! directclass
	open_xpm("reload_small.xpm",clist1,&rdpix,&rdmask,1);
	open_xpm("rpm1.xpm",clist1,&rpm1pix,&rpm1mask,1);

	//open_xpm("locked.xpm",clist1,&lockpix,&lockmask,1);
	//open_xpm("make.xpm",clist1,&makpix,&makmask,1);
	// FIX-ME!!! directclass bug in open_xpm
	//open_xpm("hdr.xpm",clist1,&hdrpix,&hdrmask,1);

}

static GnomeUIInfo pack_sfile_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("tar.gz"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(0), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("gzip"),
    N_("filter selected file through gzip"),
    (gpointer) on_mpack_activate, GINT_TO_POINTER(1), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("tar.bz2"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(2), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("bzip2"),
    N_("filter selected file through bzip2"),
    (gpointer) on_mpack_activate, GINT_TO_POINTER(3), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("RAR"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(4), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("zip"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(5), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("lha"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(6), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("zoo"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(7), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("cpio"),
    NULL,
    (gpointer) on_mpack_activate, GINT_TO_POINTER(8), NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};


static GnomeUIInfo menu1_uiinfo[] =
{
  {
    GNOME_APP_UI_SUBTREE, N_("Pack file(s) as"),
    NULL,
    pack_sfile_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "gnochive/add-18.xpm",
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Extract file (Remove filter)"),
    NULL,
    (gpointer) extract_single, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    GDK_a, (GdkModifierType) GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Extract file(s) to ..."),
    NULL,
    (gpointer) extract_dialog, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "gnochive/extr-18.xpm",
    GDK_a, (GdkModifierType) GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("View file(s)"),
    NULL,
    (gpointer) on_properties2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Add comment"),
    NULL,
    (gpointer) on_properties2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Test Archive"),
    NULL,
    (gpointer) quick_test, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Select All"),
    NULL,
    (gpointer) on_item8_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    GDK_a, (GdkModifierType) GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Un-select all"),
    NULL,
    (gpointer) on_un_select_all1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Cut"),
    NULL,
    (gpointer) on_copy2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Copy"),
    NULL,
    (gpointer) on_copy2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Paste"),
    NULL,
    (gpointer) on_copy2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE,
    0, (GdkModifierType) 0, NULL
  },
    {
    GNOME_APP_UI_ITEM, N_("Paste as new archive..."),
    NULL,
    (gpointer) on_copy2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Properties"),
    NULL,
    (gpointer) on_properties2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

GtkWidget*
create_folder_pack_menu(void)
{
	GtkWidget *menu1 = gtk_menu_new ();
	gchar *names[]={"separator9", "menu_it_2", "menu_it_3", "separator10",
					"menu_it_4", "menu_it_5", "menu_it_6", "separator11",
					"menu_it_7", "menu_it_8", "separator12", "menu_it_9",
					"menu_it_10", "menu_it_11", "menu_it_12", "separator13", "menu_it_13"};
	gchar *subnames[]={"item4", "item5", "item6", "item7", "item8", "item9", "lha1", "zoo1", "cpio1"};
	guint i,j=0;
	
	gtk_widget_set_name (menu1, "menu1");
	gtk_object_set_data (GTK_OBJECT (menu1), "menu1", menu1);
	gnome_app_fill_menu (GTK_MENU_SHELL (menu1), menu1_uiinfo,
                       NULL, FALSE, 0);

	gtk_widget_set_name (menu1_uiinfo[0].widget, "menu_it_1");
	gtk_widget_ref (menu1_uiinfo[0].widget);
	gtk_object_set_data_full (GTK_OBJECT (menu1), "menu_it_1",
                            menu1_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
	for (i=0;i<=8;i++) {
		gtk_widget_set_name (pack_sfile_menu_uiinfo[i].widget, subnames[j]);
		gtk_widget_ref (pack_sfile_menu_uiinfo[i].widget);
		gtk_object_set_data_full (GTK_OBJECT (menu1), subnames[j],
                            pack_sfile_menu_uiinfo[i].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
		j++;
	}
	j=0;
	for (i=1;i<=17;i++) {
		gtk_widget_set_name (menu1_uiinfo[i].widget, names[j]);
  		gtk_widget_ref (menu1_uiinfo[i].widget);
		gtk_object_set_data_full (GTK_OBJECT (menu1), names[j],
                            menu1_uiinfo[i].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
		j++;
	}
	return menu1;
}

/*
 * state: 1 extract disabled , gzip & bzip pack disabled
 */

void
menu_set_state(guint32 state)
{
	gchar *items[]={"menu_it_1", "item4", "item5", "item6", "item7",
					"item8", "item9", "lha1", "zoo1", "menu_it_2",
					"menu_it_3", "menu_it_4", "menu_it_5", "menu_it_6",
					"menu_it_7", "menu_it_8", "menu_it_9", "menu_it_10",
					"menu_it_11", "menu_it_12", "menu_it_13"};
	GtkWidget *myitem;
	gint i=0;
	gint j=20;
	guint32 myres;

    while (j>=0)
    {
		myres=pow(2, j);
		myres=(state & myres) >> j;
		myitem=lookup_widget(GTK_WIDGET(gnochivemain->mainmenu),items[i]);
		gtk_widget_set_sensitive(GTK_WIDGET(myitem),myres);
		j--;i++;
    }
}

GtkWidget*
create_single_pack_menu(void)
{
	g_print("no menu");
	return NULL;
}

void
on_mpack_activate(GtkMenuItem     *menuitem, gpointer         user_data)
{
	gchar *it_names[] = {"item4", "item5", "item6", "item7", 
						"item8", "item9", "lha1", "zoo1", "cpio1"};
	gchar *ext[] = {".tar.gz", "", ".tar.bz2", "", ".rar",
					".zip", ".lha", ".zoo", ".cpio"};
	gchar *comp[] = {"-9", "", "-9", "", "-9", "-9", "o6", "h", ""};
	gchar *types[] =   {"tar", "", "tar", "", "RAR",
						"zip", "lha", "zoo", "cpio"};
	guint filters[] = {1, 0, 2, 0, 0, 0, 0, 0, 0};
	gchar *wid_name = gtk_widget_get_name(GTK_WIDGET(menuitem));
	guint i,time_id;
	GtkWidget *dialog;
	
	while (gtk_events_pending())
		gtk_main_iteration();
	
	if (GPOINTER_TO_INT(user_data) == 1)
	{
		time_id = set_activity();
		gzip_sel(TRUE);
		remove_activity(time_id);
	}
	else {
		if (GPOINTER_TO_INT(user_data) == 3)
		{
			time_id = set_activity();
			bzip_sel(TRUE);
			remove_activity(time_id);
		}
		else {
			
			qck_pack = (qpack *)g_malloc(sizeof(qpack));
			
			if (qck_pack)
			{
				for (i=0;i<=8;i++)
				{
					if (!cmpare(it_names[i], wid_name))
					{
						qck_pack->q_ext = g_strdup(ext[i]);
						qck_pack->comp_command = g_strdup(comp[i]);
						qck_pack->arc_type = g_strdup(types[i]);
						if (i<3) qck_pack->filter = filters[i];
						else qck_pack->filter = 0;
						break;
					}
				}
				dialog = create_quick_pack();
				qck_pack->dialog = dialog;
				gnochivemain->qdialog = qck_pack;
				gtk_widget_show(dialog);
				
			}
		}
	}
}

GtkWidget*
create_arch_menu(void)
{
	gchar *archivers[] = {"tar", "RAR", "zip", "lha",
						"zoo", "gzip", "bzip2", "compress", "cpio"};
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	guint i;
	for (i=0;i<=8;i++)
	{
		item = gtk_menu_item_new_with_label(_(archivers[i]));
		gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(main_arch_menu),
							GINT_TO_POINTER(i));
		gtk_widget_show(item);
		gtk_menu_append(GTK_MENU(menu), item);
	}
	return menu;
}

GtkWidget*
create_subarch_menu(void)
{
	gchar *filters[] = {"gzip","bzip2","compress"};
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	guint i;
	for (i=0;i<=2;i++)
	{
		item=gtk_menu_item_new_with_label(_(filters[i]));
		gtk_signal_connect(GTK_OBJECT(item),"activate",
							GTK_SIGNAL_FUNC(subarch_menu),
							GINT_TO_POINTER(i));
		gtk_widget_show(item);
		gtk_menu_append(GTK_MENU(menu),item);
	}
	return menu;
}

GtkWidget*
create_appearance_menu(void)
{
	gchar *filters[]={"Text under icons", "Text only", "Icons only"};
	GtkWidget *menu=gtk_menu_new();
	GtkWidget *item;
	guint i;
	for (i = 0;i<=2;i++)
	{
		item = gtk_menu_item_new_with_label(N_(filters[i]));
		gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(on_appear_menu_selected),
							GINT_TO_POINTER(i));
		gtk_widget_show(item);
		gtk_menu_append(GTK_MENU(menu), item);
	}
	return menu;
}

GtkWidget*
create_tabs_menu(void)
{
	gchar *filters[] = {"Left", "Right", "Top", "Bottom"};
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	guint i;
	for (i=0;i<=3;i++)
	{
		item=gtk_menu_item_new_with_label(N_(filters[i]));
		gtk_signal_connect(GTK_OBJECT(item),"activate",
							GTK_SIGNAL_FUNC(on_tabs_menu_selected),
							GINT_TO_POINTER(i));
		gtk_widget_show(item);
		gtk_menu_append(GTK_MENU(menu),item);
	}
	return menu;
}

static gboolean
main_arch_menu(GtkWidget *item, gpointer user_data)
{
	gchar *archivers[] = {"tar", "RAR", "zip", "lha", "zoo", "gzip", "bzip2", "compress", "cpio"};
	GtkWidget *menu2 = lookup_widget(GTK_WIDGET(item), "optionmenu2");
	GtkWidget *rar_opt = lookup_widget(GTK_WIDGET(item), "rar_options");
	GtkWidget *pass_opt = lookup_widget(GTK_WIDGET(item), "password_option");
	if (!user_data)
		gtk_widget_show(menu2);
	else gtk_widget_hide(menu2);
	if (GPOINTER_TO_INT(user_data) == 1) {
		gtk_widget_show(rar_opt);
		gtk_widget_set_sensitive(rar_opt, TRUE);
		gtk_widget_show(pass_opt);
		//gtk_widget_set_sensitive(pass_opt, TRUE);
	}
	else {
		if (GPOINTER_TO_INT(user_data) == 2)
		{
			gtk_widget_show(pass_opt);
			//gtk_widget_set_sensitive(pass_opt, TRUE);
			gtk_widget_hide(rar_opt);
			gtk_widget_set_sensitive(rar_opt,FALSE);
		}
		else {
			gtk_widget_hide(rar_opt);
			gtk_widget_set_sensitive(rar_opt,FALSE);
			gtk_widget_hide(pass_opt);
			//gtk_widget_set_sensitive(pass_opt,FALSE);
		}
	}
	my_ad->type = g_strdup(archivers[GPOINTER_TO_INT(user_data)]);
	return TRUE;
}

static gboolean
subarch_menu(GtkWidget *item, gpointer user_data)
{
	/* used only if type is 0 */
	my_ad->sub_type = GPOINTER_TO_INT(user_data);
	return TRUE;
}

static gboolean
on_appear_menu_selected(GtkWidget *item, gpointer user_data)
{
	gnome_property_box_changed((GnomePropertyBox *)gnochivemain->gc_cfg->widget);
	switch (GPOINTER_TO_INT(user_data))
	{
		case 0:	gnochivemain->gc_cfg->tstyle = 0;break;
		case 1:	gnochivemain->gc_cfg->tstyle = 1;break;
		case 2:	gnochivemain->gc_cfg->tstyle = 2;break;
		case 3:	gnochivemain->gc_cfg->tstyle = 3;break;
	}
	return TRUE;
}

static gboolean
on_tabs_menu_selected(GtkWidget *item, gpointer user_data)
{
	gnome_property_box_changed((GnomePropertyBox *)gnochivemain->gc_cfg->widget);
	switch (GPOINTER_TO_INT(user_data))
	{
		case 0:	gnochivemain->gc_cfg->tbstyle = 0;break;
		case 1:	gnochivemain->gc_cfg->tbstyle = 1;break;
		case 2:	gnochivemain->gc_cfg->tbstyle = 2;break;
		case 3:	gnochivemain->gc_cfg->tbstyle = 3;break;
	}
	return TRUE;
}

/*FIXME! Clean up the parameters, used for backwd comp only now*/

void
extract_to(GtkMenuItem     *menuitem, gpointer         user_data)
{
	GtkWidget *extr = create_extract_dialog();
	GtkWidget *entry = lookup_widget(GTK_WIDGET(extr), "combo_entry24");
	GtkWidget *arclbl = lookup_widget(GTK_WIDGET(extr), "label292");
	gchar *path = g_get_current_dir();
	/*FIXME! default: overwrite any files*/
	gnochivemain->gc_cfg->overwrite = TRUE;
	gnochivemain->gc_cfg->all = FALSE;
	gtk_entry_set_text(GTK_ENTRY(entry), path);
	gtk_label_set_text(GTK_LABEL(arclbl), user_data);
	gtk_widget_show(extr);
	g_free(path);		
}

void
extract_dialog(GtkMenuItem     *menuitem, gpointer         user_data)
{
	gchar *path = g_get_current_dir();
	gchar *text = NULL;
	gchar *full_path = NULL;
	gchar *ac_type = NULL;
	
	if (!gnochivemain->active_page->opd_archive) 
	{
		gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),
				last_select_row, 1, &text);
		if (text != NULL)
		{
			full_path = g_concat_dir_and_file(path, text);
			ac_type = gnochive_get_archive_type(full_path);
			if (ac_type)
			{
				extract_to(NULL, text);
				g_free(ac_type);
			}
			g_free(full_path);
		}
	}
	else
		extract_to(NULL, gnochivemain->active_page->ad->full_name);
	g_free(path);
}

static GnomeUIInfo file1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_Add to Archive"), NULL, on_new_file1_activate, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (NULL, NULL),
  GNOMEUIINFO_MENU_CLOSE_ITEM (on_close1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (on_exit1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo edit1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM (on_select_all1_activate, NULL),
  GNOMEUIINFO_MENU_CLEAR_ITEM (on_clear1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CUT_ITEM (on_cut1_activate, NULL),
  GNOMEUIINFO_MENU_COPY_ITEM (on_copy1_activate, NULL),
  GNOMEUIINFO_MENU_PASTE_ITEM (on_paste1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo view1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Main toolbar"),
    NULL,
    (gpointer) on_main_toolbar1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Location Toolbar"),
    NULL,
    (gpointer) on_location_toolbar1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Archive Toolbar"),
    NULL,
    (gpointer) on_extended_toolbar1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Selected file"),
    NULL,
    (gpointer) on_selected_file1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Log"),
    NULL,
    (gpointer) on_log1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo convert_archive1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("tar.gz"),
    NULL,
    (gpointer) on_targz1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("tar.bz2"),
    NULL,
    (gpointer) on_tarbz1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("tar.Z"),
    NULL,
    (gpointer) on_tarz1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo tools1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_SUBTREE, N_("Convert Archive"),
    NULL,
    convert_archive1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Estimate/Repack"),
    NULL,
    (gpointer) on_estimate_repack1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Test archives"),
    NULL,
    (gpointer) on_test_archive1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Extract file(s)"),
    NULL,
	(gpointer) extract_dialog, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Delete file(s)"),
    NULL,
    (gpointer) on_button12_clicked, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Add comment"),
    NULL,
    (gpointer) on_add_comment1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
	{
    GNOME_APP_UI_ITEM, N_("Paths"),
    NULL,
    (gpointer) on_preferences1_activate, (gpointer) 1, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("User Interface"),
    NULL,
    (gpointer) on_preferences1_activate, (gpointer) 2, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Advanced"),
    NULL,
    (gpointer) on_preferences1_activate, (gpointer) 3, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Plugins"),
    NULL,
    (gpointer) on_preferences1_activate, (gpointer) 4, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("RPM"),
    NULL,
    (gpointer) on_preferences1_activate, (gpointer) 5, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_preferences1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo windows1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Close current window"),
    NULL,
    (gpointer) on_close_this_window1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Close All"),
    NULL,
    (gpointer) on_close_all_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
  GNOMEUIINFO_HELP ("gnochive"),
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("gnochive Home Page"),
    NULL,
    (gpointer) on_homepage_activate,
	"http://gnochive.sourceforge.net", NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Report a bug"),
    NULL,
    (gpointer) on_report_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about2_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_EDIT_TREE (edit1_menu_uiinfo),
  GNOMEUIINFO_MENU_VIEW_TREE (view1_menu_uiinfo),
  {
    GNOME_APP_UI_SUBTREE, N_("_Tools"),
    NULL,
    tools1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_WINDOWS_TREE (windows1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};

void
create_main_menu(GnomeApp *app, MainMenu *menu)
{
	guint i;
	
	gnome_app_create_menus (GNOME_APP (app), menubar1_uiinfo);

	menu->file.new_file = file1_menu_uiinfo[0].widget;
	menu->file.open_file = file1_menu_uiinfo[1].widget;
	menu->file.close_file = file1_menu_uiinfo[2].widget;
	menu->file.exit = file1_menu_uiinfo[3].widget;
	
	for (i=0;i<4;i++)
	{
		gtk_widget_ref (file1_menu_uiinfo[i].widget);
		gtk_accel_group_attach (GNOME_APP (app)->accel_group,
							GTK_OBJECT(file1_menu_uiinfo[i].widget));
	}
	
	menu->view.main_toolbar = view1_menu_uiinfo[0].widget;
	menu->view.location_toolbar = view1_menu_uiinfo[2].widget;
	menu->view.archive_toolbar = view1_menu_uiinfo[4].widget;
	menu->view.selected_file = view1_menu_uiinfo[6].widget;

	menu->tools.convert_archive = tools1_menu_uiinfo[0].widget;
	menu->tools.estimate = tools1_menu_uiinfo[1].widget;
	menu->tools.test_archive = tools1_menu_uiinfo[2].widget;
	menu->tools.extract_file = tools1_menu_uiinfo[4].widget;
	menu->tools.delete_file = tools1_menu_uiinfo[5].widget;
	menu->tools.add_comment = tools1_menu_uiinfo[7].widget;
	menu->tools.tgz_conv = convert_archive1_menu_uiinfo[0].widget;
	menu->tools.tbz2_conv = convert_archive1_menu_uiinfo[1].widget;
	menu->tools.tz_conv = convert_archive1_menu_uiinfo[2].widget;
	
	menu->windows.close_window = windows1_menu_uiinfo[0].widget;
	menu->windows.close_all = windows1_menu_uiinfo[1].widget;
	
  /*gtk_widget_set_name (menubar1_uiinfo[0].widget, "file1");
  gtk_widget_ref (menubar1_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "file1",
                            menubar1_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (file1_menu_uiinfo[0].widget, "new_file1");
  gtk_widget_ref (file1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "new_file1",
                            file1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (file1_menu_uiinfo[1].widget, "open1");
  gtk_widget_ref (file1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "open1",
                            file1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (file1_menu_uiinfo[2].widget, "close1");
  gtk_widget_ref (file1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "close1",
                            file1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (file1_menu_uiinfo[3].widget, "separator2");
  gtk_widget_ref (file1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "separator2",
                            file1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (file1_menu_uiinfo[4].widget, "exit1");
  gtk_widget_ref (file1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "exit1",
                            file1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (menubar1_uiinfo[1].widget, "edit1");
  gtk_widget_ref (menubar1_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "edit1",
                            menubar1_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (edit1_menu_uiinfo[0].widget, "select_all1");
  gtk_widget_ref (edit1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "select_all1",
                            edit1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (edit1_menu_uiinfo[1].widget, "cut1");
  gtk_widget_ref (edit1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "cut1",
                            edit1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (edit1_menu_uiinfo[2].widget, "copy1");
  gtk_widget_ref (edit1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "copy1",
                            edit1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (edit1_menu_uiinfo[3].widget, "paste1");
  gtk_widget_ref (edit1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "paste1",
                            edit1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (edit1_menu_uiinfo[4].widget, "clear1");
  gtk_widget_ref (edit1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "clear1",
                            edit1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (menubar1_uiinfo[2].widget, "view1");
  gtk_widget_ref (menubar1_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "view1",
                            menubar1_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (view1_menu_uiinfo[0].widget, "item4");
  gtk_widget_ref (view1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "item4",
                            view1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (as_files1_uiinfo[0].widget, "as_files1");
  gtk_widget_ref (as_files1_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "as_files1",
                            as_files1_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (as_files1_uiinfo[1].widget, "as_sorted_dirs1");
  gtk_widget_ref (as_files1_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "as_sorted_dirs1",
                            as_files1_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (view1_menu_uiinfo[1].widget, "toolbars1");
  gtk_widget_ref (view1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "toolbars1",
                            view1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (toolbars1_menu_uiinfo[0].widget, "main_toolbar1");
  gtk_widget_ref (toolbars1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "main_toolbar1",
                            toolbars1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (toolbars1_menu_uiinfo[1].widget, "extended_toolbar1");
  gtk_widget_ref (toolbars1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "extended_toolbar1",
                            toolbars1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (toolbars1_menu_uiinfo[2].widget, "location_toolbar1");
  gtk_widget_ref (toolbars1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "location_toolbar1",
                            toolbars1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (toolbars1_menu_uiinfo[3].widget, "hide_open_archive_list1");
  gtk_widget_ref (toolbars1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "hide_open_archive_list1",
                            toolbars1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (toolbars1_menu_uiinfo[3].widget), TRUE);

  gtk_widget_set_name (toolbars1_menu_uiinfo[4].widget, "hide_clist1");
  gtk_widget_ref (toolbars1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "hide_clist1",
                            toolbars1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (toolbars1_menu_uiinfo[4].widget), TRUE);

  gtk_widget_set_name (view1_menu_uiinfo[2].widget, "separator7");
  gtk_widget_ref (view1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "separator7",
                            view1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (view1_menu_uiinfo[3].widget, "item1");
  gtk_widget_ref (view1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "item1",
                            view1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (item1_menu_uiinfo[0].widget, "item2");
  gtk_widget_ref (item1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "item2",
                            item1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (item1_menu_uiinfo[1].widget, "same_window1");
  gtk_widget_ref (item1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "same_window1",
                            item1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (item1_menu_uiinfo[2].widget, "separator8");
  gtk_widget_ref (item1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "separator8",
                            item1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (item1_menu_uiinfo[3].widget, "selected_file1");
  gtk_widget_ref (item1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "selected_file1",
                            item1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (item1_menu_uiinfo[4].widget, "log1");
  gtk_widget_ref (item1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "log1",
                            item1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_sensitive (item1_menu_uiinfo[4].widget, FALSE);

  gtk_widget_set_name (menubar1_uiinfo[3].widget, "tools1");
  gtk_widget_ref (menubar1_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "tools1",
                            menubar1_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (tools1_menu_uiinfo[0].widget, "convert_archive1");
  gtk_widget_ref (tools1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "convert_archive1",
                            tools1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (convert_archive1_menu_uiinfo[0].widget, "targz1");
  gtk_widget_ref (convert_archive1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "targz1",
                            convert_archive1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (convert_archive1_menu_uiinfo[1].widget, "tarbz1");
  gtk_widget_ref (convert_archive1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "tarbz1",
                            convert_archive1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (convert_archive1_menu_uiinfo[2].widget, "tarz1");
  gtk_widget_ref (convert_archive1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "tarz1",
                            convert_archive1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (tools1_menu_uiinfo[1].widget, "estimate_repack1");
  gtk_widget_ref (tools1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "estimate_repack1",
                            tools1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (tools1_menu_uiinfo[2].widget, "test_archive1");
  gtk_widget_ref (tools1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "test_archive1",
                            tools1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (tools1_menu_uiinfo[3].widget, "add_comment1");
  gtk_widget_ref (tools1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "add_comment1",
                            tools1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (menubar1_uiinfo[4].widget, "settings1");
  gtk_widget_ref (menubar1_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "settings1",
                            menubar1_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (settings1_menu_uiinfo[0].widget, "preferences1");
  gtk_widget_ref (settings1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "preferences1",
                            settings1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (menubar1_uiinfo[5].widget, "windows1");
  gtk_widget_ref (menubar1_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "windows1",
                            menubar1_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (windows1_menu_uiinfo[0].widget, "create_new_window1");
  gtk_widget_ref (windows1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "create_new_window1",
                            windows1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (windows1_menu_uiinfo[1].widget, "close_this_window1");
  gtk_widget_ref (windows1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "close_this_window1",
                            windows1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (windows1_menu_uiinfo[2].widget, "separator4");
  gtk_widget_ref (windows1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "separator4",
                            windows1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (windows1_menu_uiinfo[3].widget, "close_all1");
  gtk_widget_ref (windows1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "close_all1",
                            windows1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (menubar1_uiinfo[6].widget, "help1");
  gtk_widget_ref (menubar1_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "help1",
                            menubar1_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (help1_menu_uiinfo[1].widget, "separator5");
  gtk_widget_ref (help1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "separator5",
                            help1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_set_name (help1_menu_uiinfo[2].widget, "about2");
  gtk_widget_ref (help1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (gnochive), "about2",
                            help1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);*/
}

void
update_main_menu(MainMenu *menu)
{
	if (!gnochivemain->active_page->opd_archive)
	{
		
		gtk_widget_set_sensitive(menu->file.close_file, FALSE);
		gtk_widget_set_sensitive(menu->view.selected_file, FALSE);
		gtk_widget_set_sensitive(menu->tools.convert_archive, FALSE);
		gtk_widget_set_sensitive(menu->tools.test_archive, FALSE);
		gtk_widget_set_sensitive(menu->tools.extract_file, FALSE);
		gtk_widget_set_sensitive(menu->tools.delete_file, FALSE);
		gtk_widget_set_sensitive(menu->tools.estimate, FALSE);
		gtk_widget_set_sensitive(menu->tools.add_comment, FALSE);
		gtk_widget_set_sensitive(menu->windows.close_window, FALSE);
		gtk_widget_set_sensitive(menu->windows.close_all, FALSE);
	}
	else
	{
		gtk_widget_set_sensitive(menu->file.close_file, TRUE);
		gtk_widget_set_sensitive(menu->view.selected_file, TRUE);
		gtk_widget_set_sensitive(menu->tools.convert_archive, TRUE);
		gtk_widget_set_sensitive(menu->tools.test_archive, TRUE);
		gtk_widget_set_sensitive(menu->tools.extract_file, TRUE);
		gtk_widget_set_sensitive(menu->tools.delete_file, TRUE);
		gtk_widget_set_sensitive(menu->tools.estimate, TRUE);
		gtk_widget_set_sensitive(menu->tools.add_comment, TRUE);
		gtk_widget_set_sensitive(menu->windows.close_window, TRUE);
		gtk_widget_set_sensitive(menu->windows.close_all, TRUE);
		if (!cmpare(gnochivemain->active_page->ad->type, "tgz"))
		{
			gtk_widget_set_sensitive(menu->tools.tgz_conv, FALSE);
			gtk_widget_set_sensitive(menu->tools.tbz2_conv, TRUE);
			gtk_widget_set_sensitive(menu->tools.tz_conv, TRUE);
		}
		if (!cmpare(gnochivemain->active_page->ad->type, "tbz2"))
		{
			gtk_widget_set_sensitive(menu->tools.tgz_conv, TRUE);
			gtk_widget_set_sensitive(menu->tools.tbz2_conv, FALSE);
			gtk_widget_set_sensitive(menu->tools.tz_conv, TRUE);
		}
		if (!cmpare(gnochivemain->active_page->ad->type, "comp"))
		{
			gtk_widget_set_sensitive(menu->tools.tgz_conv, TRUE);
			gtk_widget_set_sensitive(menu->tools.tbz2_conv, TRUE);
			gtk_widget_set_sensitive(menu->tools.tz_conv, FALSE);
		}
		gtk_widget_set_sensitive(menu->file.close_file, TRUE);
	}
}

void
gnochive_attach_keys(GtkObject *obj)
{
	gtk_signal_connect (obj, "key-press-event",
                      GTK_SIGNAL_FUNC (gnochive_key_press_event),
                      NULL);
	gtk_signal_connect (obj, "key-release-event",
                      GTK_SIGNAL_FUNC (gnochive_key_release_event),
                      NULL);
}
