/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Lucian Langa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _COMPONENTS_H_
#define _COMPONENTS_H_

#include "gnomefilelist.h"
#include "gc_bk_end.h"

#define MAX_NAME_LENGTH 64

typedef struct _FileSelData FileSelData;
typedef struct _DirSelData DirSelData;

struct _FileSelData
{
  char title[MAX_NAME_LENGTH];
  gpointer data;
  void (*click_ok_callback)(GtkButton*, gpointer);
  void (*click_cancel_callback)(GtkButton*, gpointer);
  GtkWidget *filesel;
  
};

struct _DirSelData
{
  char title[MAX_NAME_LENGTH];
  gpointer data;
  void (*click_ok_callback)(GtkButton*, gpointer);
  void (*click_cancel_callback)(GtkButton*, gpointer);
  GtkWidget *dirsel;
  GtkWidget *parent;
  
};

typedef struct _pager pager;

struct _pager
{
	GtkWidget 		*clist;
	gboolean		opd_archive;
	gint			page_no;
	GtkWidget		*page;
	gchar 			*selected_rows;
	gchar 			*clist_line[3][8];
	guint			current_lines;
	guint			files_selected;
	archivedesc		*ad;
	gpointer		*next;
	gchar			*path;
	GtkWidget		*bt;
	GtkWidget		*bt_gr;
};

typedef struct _qpack qpack;
	
struct _qpack
{
	GtkWidget		*dialog;
	gchar			*q_ext;
	gchar			*q_name;
	gchar			*comp_command;
	guint			filter;
	gchar			*arc_type;
	gchar			*full_name;
	gchar			*name;
	gchar			*path;
};

typedef struct _FM FM;
typedef struct _EM EM;
typedef struct _VM VM;
typedef struct _TM TM;
typedef struct _SM SM;
typedef struct _WM WM;
typedef struct _HM HM;	
	
typedef struct _MainMenu MainMenu;
	
struct _FM
{
	GtkWidget *new_file;
	GtkWidget *open_file;
	GtkWidget *close_file;
	GtkWidget *exit;
};

struct _EM
{
	GtkWidget *select_all;
	GtkWidget *cut;
	GtkWidget *copy;
	GtkWidget *paste;
	GtkWidget *clear;
};

struct _VM
{
	GtkWidget *main_toolbar;
	GtkWidget *location_toolbar;
	GtkWidget *archive_toolbar;
	GtkWidget *selected_file;
	GtkWidget *log;
};

struct _TM
{
	GtkWidget *convert_archive;
	GtkWidget *estimate;
	GtkWidget *test_archive;
	GtkWidget *add_comment;
	GtkWidget *extract_file;
	GtkWidget *delete_file;
	
	GtkWidget *tgz_conv;
	GtkWidget *tbz2_conv;
	GtkWidget *tz_conv;
};

struct _SM
{
	GtkWidget *preferences;
};

struct _WM
{
	GtkWidget *close_window;
	GtkWidget *close_all;
};

struct _HM
{
	GtkWidget *about;
};

struct _MainMenu
{
	FM	file;
	EM	edit;
	VM	view;
	TM	tools;
	SM	settings;
	WM	windows;
	HM	help;
};


void myfn(GtkButton *button, gpointer data);

GtkWidget* create_myfile_selection(FileSelData *myfs);
GtkWidget* create_dir_selection(DirSelData *dirsel);

void
gnochive_open_dialog                   (GtkButton       *button,
                                        gpointer         user_data);

gboolean draw_toolbar_buttons (guint type,guint style,gboolean crt);
void gnochive_error	(gint errnum,gchar *err_msg, ...);
void cancel_fs_func(GtkButton *button, gpointer data);
void cancel_dir_func(GtkButton *button, gpointer data);
void myfn(GtkButton *button, gpointer data);
void update_all(GtkWidget *propbox3);
void update_propertybox3(GtkWidget *propbox3);
void display_arc_status	(gchar *path,GtkEntry *entry,GtkLabel *label);
void display_dual_status (gchar *path, gchar *path2, GtkEntry *entry, GtkEntry *entry2, GtkLabel *label);
GtkWidget* create_pobox3 (void);
void write_arc (FILE *cfgfile,gchar *path);
gchar* parse_arc_path (GtkWidget *entry);
void recheck_status (gchar *path,GtkLabel *label);
int check_arc (gchar *path);
void save_changes(void);
void display_cclist	(GtkWidget *prop,guint cclist_val,guint stat);
gint apply_cclist (void);
void show_cclist (GtkWidget *clist,guint stat);

GtkWidget* create_clist(GtkWidget *parent);
GtkWidget* create_note(GtkWidget *parent);
GtkWidget* add_page(pager *page, GtkWidget *parent,GtkWidget *notebook,gchar *lbl);
void clist_drag_link (GtkCList *clist, GtkSignalFunc recv_func);
void gnochive_draw_pager(pager *gc_pager);
pager* gnochive_pager_add(gchar *label);
pager* gnochive_pager_init(void);
pager* gnochive_dig_pager(gint page_no);
gboolean gnochive_page_destroy(pager *top_page,pager *del_page);

/* gnochive right click menus */
GtkWidget* create_folder_pack_menu(void);
void menu_set_state(guint32 state);
GtkWidget* create_single_pack_menu(void);
void on_mpack_activate(GtkMenuItem     *menuitem,gpointer         user_data);

guint
gnochive_notebook_get_pages (pager *my_page);

void
on_notebook_close_page (GtkNotebook *notebook,
						GtkNotebookPage *page,
						gint page_num, gpointer user_data);

void extract_to(GtkMenuItem     *menuitem,gpointer         user_data);
void extract_dialog(GtkMenuItem     *menuitem, gpointer         user_data);

void create_clist_pixmaps(GtkWidget *clist1);
GtkWidget* create_arch_menu(void);
GtkWidget* create_subarch_menu(void);
GtkWidget* create_appearance_menu(void);
GtkWidget* create_tabs_menu(void);
void update_main_menu(MainMenu *menu);
void create_main_menu(GnomeApp *app, MainMenu *menu);
void gnochive_attach_keys(GtkObject *obj);

#endif
