/*  GUI frontend for common compresion programs
 *  Copyright 2000-2002  Lucian Langa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "dnd.h"
#include <gnome.h>
#include "gnochive.h"

extern gchivemain *gnochivemain;

extern int last_select_row;
extern int last_select_column;

mydnd*
gnochive_drag_init(void)
{
	mydnd *dnd = NULL;
	dnd = (mydnd *) g_malloc(sizeof(mydnd));
	return dnd;
}

void     
gnochive_clist_drag_begin (GtkWidget *widget, GdkDragContext *context,
			       gpointer user_data)
{
	GtkCList *clist;

	clist = GTK_CLIST (user_data);
	//dnd = tree_view->details->dnd;

	/* The drag is started. reinit the drag data. */
	gnochivemain->dnd->drag_pending = FALSE;
	
	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
				      "drag_begin");

	//gnochive_set_dnd_icon (clist, context);
}

void
gnochive_clist_drag_end (GtkWidget *widget, GdkDragContext *context,
			     gpointer user_data)
{
	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
				      "drag_end");

}

gboolean 
gnochive_clist_drag_motion (GtkWidget *widget, GdkDragContext *context,
				int x, int y, guint time, gpointer user_data)
{
	/* Update draggin cursor */
	gdk_drag_status (context, GDK_ACTION_COPY, time);
	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
			      "drag_motion");
	return TRUE;
}

gboolean 
gnochive_clist_drag_drop (GtkWidget *widget,
			      GdkDragContext *context,
			      int x, int y, guint time,
			      gpointer user_data)
{
	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
				      "drag_drop");
	return TRUE;
}

int 
gnochive_clist_button_press (GtkWidget *widget, GdkEventButton *event)
{
	int retval;
	GtkCList *clist;
		
	int press_row, press_column, on_row;

	clist = GTK_CLIST (widget);
	retval = FALSE;

	if (event->window != clist->clist_window)
		return retval;

	on_row = gtk_clist_get_selection_info (clist, 
						    event->x, 
						    event->y, 
						    &press_row, &press_column);
	if (on_row == 0) {
		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button-press-event");
		return FALSE;
	}
	
	//gtk_clist_select_row(clist, press_row, press_column);
	//on_clist1_button_press_event(clist, event, NULL);

		/* Clicking in the expander should not start a drag */
		/*tree_view->details->dnd->drag_pending = FALSE;
	
		ctree_row = ROW_ELEMENT (clist, press_row)->data;
		if (ctree_row != NULL) {
			ctree_row->mouse_down = TRUE;
			ctree_row->in_hotspot = TRUE;

			node = eel_ctree_find_node_ptr (EEL_CTREE (widget), ctree_row);
			if (node != NULL) {
				eel_ctree_draw_node (EEL_CTREE (widget), node);
			}
		}
	} else {*/
		switch (event->type) {
		case GDK_BUTTON_PRESS:
			gnochivemain->dnd->drag_pending = (gboolean *)TRUE;
			gnochivemain->dnd->press_x = event->x;
			gnochivemain->dnd->press_y = event->y;
			gnochivemain->dnd->pressed_button = (GdkEventButton *)event->button;
			break;
		case GDK_2BUTTON_PRESS:
		default:
			break;
		}

	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button-press-event");
	
	return TRUE;
}


void 
gnochive_set_dnd_icon (GtkCList *clist, GdkDragContext *context)
{
	GdkPixmap 	*pixmap;
	GdkBitmap 	*mask;
		
	gtk_clist_get_pixmap (GTK_CLIST (clist),
						last_select_row,
						0,
						&pixmap, &mask);

	gtk_drag_set_icon_pixmap (context, gdk_colormap_get_system (), pixmap, mask, 12, 12);
}

/* added uri support when parsing files from nautilus */

void
on_mclist_drag_data_received           (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
	GList *mynames, *node;
	char *nparsed;
	gchar *file = NULL;
	
	mynames = gnome_uri_list_extract_uris((char *)data->data);
	
	for (node = mynames ; node != NULL ; node=node->next)
	{
		nparsed = (char*)node->data;
		if (strncmp(nparsed, "file:", 5)==0)
			file = g_strndup(nparsed+5, strlen(nparsed)-4);
		else
			file = g_strdup(nparsed);
		open_real_archive(file);
		g_free(file);
	}
	
	gnome_uri_list_free_strings(mynames);
	gtk_drag_finish(drag_context, TRUE, FALSE, time);
}

void
on_aclist_drag_data_received           (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
	GList *mynames, *node;
	char *nparsed;
	gchar *file = NULL;
	gchar *temp_name = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	int tf;
	FILE *tmpf;
	gchar zoo_cmd[20];
	
	tf = mkstemp(temp_name);
	close(tf);
	tmpf = fopen(temp_name, "w+");
	mynames = gnome_uri_list_extract_uris((char *)data->data);
	
	if (strcmp(gnochivemain->active_page->ad->type, "zoo") == 0)
	{	// workaround ZOO bug when parsing directories from stdin
			fprintf(tmpf, "#!/bin/sh\n");
			strcpy(zoo_cmd, "un");
			fprintf(tmpf, "%s %s%sqqq $1 ", gnochivemain->gc_cfg->archs.zoo_path,
						zoo_cmd, "h");
	}
	
	for (node = mynames ; node != NULL ; node=node->next)
	{
		nparsed = (char*)node->data;
		if (strncmp(nparsed, "file:", 5)==0)
			file = g_strndup(nparsed+5, strlen(nparsed)-4);
		else
			file = g_strdup(nparsed);
		if (strcmp(gnochivemain->active_page->ad->type, "zoo") == 0)
			fprintf(tmpf,"%s ", file);
		else
			fprintf(tmpf,"%s\n", file);
		g_free(file);
	}
	fclose(tmpf);
	gnochive_add_archive_files(gnochivemain->active_page->ad, temp_name);
	gnome_uri_list_free_strings(mynames);
	gtk_drag_finish(drag_context, TRUE, FALSE, time);
	remove(temp_name);
	g_free(temp_name);
}
