/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Lucian Langa
 *
 *	gnochive backend to supported archive formats
 *
 *  This program is free csoftware; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <unistd.h>
#include <errno.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <dirent.h>
#include <fcntl.h>

#include "archs.h"
#include "conf.h"
#include "misc.h"
#include "mime.h"
#include "components.h"
#include "gnochive.h"
#include "interface.h"
#include "support.h"
#include "gc_bk_end.h"

#include "bzip2.h"
#include "gzip.h"
#include "zip.h"
#include "zoo.h"
#include "lha.h"
#include "rar.h"
#include "tar.h"
#include "arj.h"
#include "ace.h"
#include "rpm.h"
#include "cpio.h"

#define TIMEOUT_GTK 5

extern gchivemain *gnochivemain;

extern int endpos;

extern GdkPixmap 	*dirpix,*docpix,*xpmpix,*makpix;
extern GdkPixmap	*rpmpix,*comppix,*dotpix,*lockpix;
extern GdkPixmap *hdrpix,*sympix,*rdpix,*rpm1pix;

extern GdkBitmap 	*dirmask,*docmask,*xpmmask,*makmask;
extern GdkBitmap	*rpmmask,*compmask,*dotmask,*lockmask;
extern GdkBitmap *hdrmask,*symmask,*rdmask,*rpm1mask;

GtkWidget *text1;

extern int sel_filter;	// tar filter 0 - none 1 - gzip 2 - bzip2 3 - compress 4 - custom

extern char buff[0x100];

guint i;

gchar *arc_type;
gchar custom_filter[0x100];

gchar sel_comp[0x20];			// selected value of the compression level
gchar compr;
gchar *arch_pass;
gchar pass_command[0x30];
extern gchar no_pass_command[0x30];

extern int opd_archive;
extern char current_opd_archive[0x200];

gchar *sel_files;
extern int select_all;

extern int last_select_row;
extern int total_sel_size;

extern gchar *name[0x100];

extern int stepping;
extern gfloat step;

extern float progr;

guint time_id;

static guint op_time(gpointer data);

/* creates an arc_type archive
    if it's a tar like archive (tar.X, rar , zip) gets the file from the clist
    writes the temporary file and finally creates archive returning 1 on succes
    RETURNS: a pointer to an archivedescriptor structure
    result should be freed and no longer used.
    FIXME: add single bz2 pack
*/

gboolean
create_archive				(archivedesc *ad)
{
	gchar *myfile = get_arch_name_from_type(ad->type, ad->name, ad->sub_type);
	ad->full_name = myfile;
	while (gtk_events_pending())
		gtk_main_iteration();
	switch (gnochive_check_arch_update(myfile)) {
		case 0:/*update*/
		{
			if (!gnochive_check_file(myfile, 2))	// if r,w are ok continue FIXME!
			{
				gnochive_error(errno,
						_("Could not create archive %s \n"), myfile);
				g_free(myfile);
				return FALSE;
			}
			switch (make_archive(ad, 1))
			{
				case -1:
					gnochive_error(0,
						_("Could not create archive %s \n"),myfile);
				case 0: return FALSE;
				case 1: break;
			}
			break;
		}
		case 1:/*overwrite*/
		{
			if (!gnochive_check_file(myfile,2))	// if r,w are ok continue FIXME!
			{
				gnochive_error(errno,_("Could not create archive %s \n"),myfile);
				g_free(myfile);
				return FALSE;
			}
			unlink(myfile);
			while (gtk_events_pending())
			gtk_main_iteration();
			switch (make_archive(ad, 0))
			{
				case -1:
					gnochive_error(0,_("Could not create archive %s \n"),myfile);
				case 0: return FALSE;
				case 1: break;
			}
			break;
		}
		case 2:/*cancel*/
		{
			g_free(myfile);
			return FALSE;
		}
		case 3:/*archive does not exists*/
		{
			while (gtk_events_pending())
			gtk_main_iteration();
			if (!gnochive_can_create(myfile))
			{
				gnochive_error(errno,_("Could not create archive %s \n"),myfile);
				g_free(myfile);
				return FALSE;
			}
			switch (make_archive(ad, 0))
			{
				case -1:
					gnochive_error(0,_("Could not create archive %s \n"),myfile);
				case 0: return FALSE;
				case 1: break;
			}
			break;
		}
		default:/*nop*/
		{
			g_free(myfile);
			break;
		}
}
return TRUE;
}

/* internal function to create/update an archive
 * tar.xx archives are generated/updated in two phases since tar does
 * no support compression pass to filters
 * return 0 on success
 */

/* FIXME update all gnochive_exec_arch to return pid ,
 * update all wait functions to use pid returned 
 */

gint
make_archive (archivedesc *ad, gint update)
{
	gchar *text;
	gchar *tmp;
	gchar tempfn[0x100];
	gchar zoo_cmd[3];
	gchar *pass_command = NULL;
	gchar vol_command[0x10];
	gchar *temp_name = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	int tf;
	FILE *tmpf;
	pid_t pid = 0;
	int status;
	gboolean error, exec_status = TRUE;
	gchar *name;
	gchar *path;
	gchar *tar_levels[] = {"-1", "-2", "-4", "-5", "-7", "-9"};
	gchar *rar_levels[] = {"-m0", "-m1", "-m2", "-m3", "-m4", "-m5"};
	gchar *level;
	guint j;

	while (gtk_events_pending())
		gtk_main_iteration();
	
	if (strcmp(ad->type, "gz") != 0)
	{
		tf = mkstemp(temp_name);
		tmpf = fopen(temp_name, "w");
		
#ifdef _GLIB_2_0_
		name = g_path_get_basename(temp_name);
		path = g_path_get_dirname(temp_name);
#else	
		name = g_strdup(g_basename(temp_name));
		path = g_strdup(g_dirname(temp_name));
#endif
				
	if (tf && tmpf) {
		if (strcmp(ad->type, "zoo") == 0)
		{	// workaround ZOO bug when parsing directories from stdin
			fprintf(tmpf, "#!/bin/sh\n");
			if (update)
				strcpy(zoo_cmd, "un");
			else
				strcpy(zoo_cmd, "a");
			if (!cmpare(ad->compress_level, "-9"))
				fprintf(tmpf, "%s %s%sqqq $1 ", gnochivemain->gc_cfg->archs.zoo_path,
						zoo_cmd, "h");
			else
				fprintf(tmpf, "%s %sqqq $1 ", gnochivemain->gc_cfg->archs.zoo_path,
						zoo_cmd);
			for (i=0;i<gnochivemain->active_page->current_lines;i++) {
				if (gnochivemain->active_page->selected_rows[i] == 1) {
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 6, &tmp);
						if (tmp[0] == 'd')
							fprintf(tmpf, "%s/* ", text);
						else
							fprintf(tmpf, "%s ", text);
				}
			}
		}
		else {
			if (strcmp(ad->type, "cpio") == 0)
			{
				gchar *shell;
				gchar *cmd;
				gchar *attr;
				
				for (i=0;i<gnochivemain->active_page->current_lines;i++)
				{
					if (gnochivemain->active_page->selected_rows[i] == 1)
					{
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 6, &attr);
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
						if (attr[0] == 'd')
						{
							shell = gnome_util_user_shell();
							cmd = g_strconcat("find ", text, ">>", temp_name, NULL);
							if ((pid = fork()) == 0)
								execl(shell, shell, "-c", cmd, NULL);
							waitpid(pid, NULL, 0);
							g_free(cmd);
						}
						else
							fprintf(tmpf, "%s\n", text);
					}
				}
			}
			else {
				for (i=0;i<gnochivemain->active_page->current_lines;i++)
				{
					if (gnochivemain->active_page->selected_rows[i] == 1)
					{
					gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
					gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
					fprintf(tmpf, "%s\n", text);
					}
				}
			}
		}
		fclose(tmpf);
		close(tf);
		strcpy(tempfn, ad->name);
		
		if (strcmp(ad->type,"RAR")==0)
		{
			if (cmpare(ad->name,".rar") !=0 )
				strcat(tempfn,".rar");
			if (!ad->locked)
			{
				pass_command = g_strdup(" ");//-p-");
			}
			else
			{
				pass_command = g_strdup_printf("-p%s", ad->passwd);
			}
			if (ad->volume == 0)
			{
				gchar *rtemp = g_strconcat("@", temp_name, NULL);
				if (update)
				{
					if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.rar_path,
							"rar", "u", "-r", "-inul", ad->full_name, rtemp, NULL))
						exec_status = FALSE;
				}
				else {
					j = 0;
					while (cmpare(tar_levels[j],ad->compress_level) != 0)
						j++;
					level = g_strdup (rar_levels[j]);
					if (!gnochive_exec_arch("/tmp/grarcrt", "grarcrt",
							gnochivemain->gc_cfg->archs.rar_path, "-v",
							pass_command, level, ad->dict_size, "-r", tempfn, rtemp, NULL))
						exec_status = FALSE;
					g_free(level);
				}
				while (!waitpid(pid, &status, WNOHANG))
				{
					error = (WEXITSTATUS(status)<0);
					while (gtk_events_pending())
						gtk_main_iteration();
				}
				g_free(rtemp);
			}
			else {
				gchar *rtemp = g_strconcat("@", temp_name, NULL);
				g_snprintf(vol_command, 10, "-v%db", ad->vol_size);
				if (!gnochive_exec_arch("/tmp/grarcrt", "grarcrt",
					gnochivemain->gc_cfg->archs.rar_path,
					pass_command, vol_command, tempfn, rtemp, NULL))	// "-inul",
					exec_status = FALSE;
				while (!waitpid(pid, &status, WNOHANG))
				{
					error = (WEXITSTATUS(status)<0);
					while (gtk_events_pending())
						gtk_main_iteration();
				}
				g_free(rtemp);
			}
		}
		if (strcmp(ad->type, "tar") == 0) {
			
			gchar *rtemp = g_strconcat("--files-from=", temp_name, NULL);
			
				if (cmpare(ad->name, ".tar") != 0)  strcat(tempfn, ".tar");
				if (update)
				{
					switch (ad->sub_type) {
						case 0:	break; //plain tar
						case 1: // tar.gz
						{
							if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path,
								"gzip", "-d", "--force", ad->full_name, NULL))
								exec_status = FALSE;
							break;
						}
						case 2: // tar.bz2
						{
							if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path,
								"bzip2", "-d", "--force", ad->full_name, NULL))
								exec_status = FALSE;
							break;
						}
						case 3:// tar.Z
						{
							if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path,
								"compress", "-d", "-f", ad->full_name, NULL))
								exec_status = FALSE;
							break; 
						}
						case 4:
						{
							//FIXME	
							//if (fork()==0) execl(custom_filter,custom_filter,tempfn,NULL);break;
							// tar.custom
						}
						default: break;
					}
					while (!waitpid(pid, &status, WNOHANG))
					{
						error = (WEXITSTATUS(status)<0);
						while (gtk_events_pending())
							gtk_main_iteration();
					}
					if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
						"tar", "-uf", tempfn, rtemp, NULL))
						exec_status = FALSE;
					while (!waitpid(pid,&status,WNOHANG))
					{
						error = (WEXITSTATUS(status)<0);
						while (gtk_events_pending())
							gtk_main_iteration();
					}
				}
				else
				{
					if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
						"tar", "-cf", tempfn, rtemp, NULL))
						exec_status = FALSE;
					while (!waitpid(pid, &status, WNOHANG))
					{
						error = (WEXITSTATUS(status)<0);
						while (gtk_events_pending())
							gtk_main_iteration();
					}
				}
				switch (ad->sub_type) {
					case 0:	break; //plain tar
					case 1:  // tar.gz
							if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path, "gzip",
									"-f", ad->compress_level, tempfn, NULL))
								exec_status = FALSE;
							break;
					case 2:  // tar.bz2
							if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path, "bzip2",
									"-f", ad->compress_level, tempfn, NULL))
								exec_status = FALSE;
							break;
					case 3:  // tar.Z
								if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path, "compress",
										"-f", "-r", tempfn, NULL))
									exec_status = FALSE;
								break;
					case 4:  // tar.custom
								if (!gnochive_exec_arch(custom_filter,custom_filter,tempfn,NULL))
									exec_status = FALSE;
								break;
					default: break;
				}
				while (!waitpid(pid,&status,WNOHANG))
				{
					error = (WEXITSTATUS(status)<0);
					while (gtk_events_pending())
						gtk_main_iteration();
				}
				g_free(rtemp);
		}
		if (strcmp(ad->type, "zip") == 0) {
			if (cmpare(ad->name, ".zip") != 0)
				strcat(tempfn, ".zip");
			if (!ad->locked)
			{
				if (update)
				{
					if (!gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
						gnochivemain->gc_cfg->archs.zip_path,
						tempfn, "-ur", "-xi", temp_name, NULL))
						exec_status = FALSE;
				}
				else
				{
					if (!gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
						gnochivemain->gc_cfg->archs.zip_path,
						tempfn, "-r", "-xi", temp_name, NULL))
						exec_status = FALSE;
				}
			}
			else
			{
				gchar *pass = g_strdup_printf("-P %s",ad->passwd);
				if (update)
				{
						if (!gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
							gnochivemain->gc_cfg->archs.zip_path, pass,
							tempfn, "-urxi", temp_name, NULL))
							exec_status = FALSE;
				}
				else
				{
						if (!gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
							gnochivemain->gc_cfg->archs.zip_path, pass,
							tempfn, "-rxi", temp_name, NULL))
							exec_status = FALSE;
				}
				g_free(pass);
			}
			while (!waitpid(pid,&status,WNOHANG))
			{
				error = (WEXITSTATUS(status)<0);
				while (gtk_events_pending())
					gtk_main_iteration();
			}
		}
		if (strcmp(ad->type, "lha") == 0) {
						if (cmpare(ad->name, ".lha") != 0) strcat(tempfn, ".lha");
						if (update) {
								if (!gnochive_exec_arch("/tmp/zpps", "zpps", ".",
									gnochivemain->gc_cfg->archs.lha_path,
									"uq", ad->full_name, temp_name, NULL))
									exec_status = FALSE;
						}
						else {
							gchar *lha_cmd = NULL;
							if (!cmpare(ad->compress_level, "-9"))
								lha_cmd = g_strdup("aqo6");
							if (!cmpare(ad->compress_level, "-5"))
								lha_cmd = g_strdup("aqo5");
							if (!cmpare(ad->compress_level, "-1"))
								lha_cmd = g_strdup("aq");
							if (!gnochive_exec_arch("/tmp/zpps", "zpps", ".",
								gnochivemain->gc_cfg->archs.lha_path,
								lha_cmd, tempfn, temp_name, NULL))
								exec_status = FALSE;
							g_free(lha_cmd);
						}
						while (!waitpid(pid,&status,WNOHANG))
						{
							error=(WEXITSTATUS(status)<0);
							while (gtk_events_pending())
								gtk_main_iteration();
						}
		}
		if (strcmp(ad->type, "zoo") == 0) {
			chmod(temp_name, S_IRUSR | S_IWUSR | S_IXUSR);
			if (cmpare(ad->name, ".zoo") != 0)
				strcat(tempfn, ".zoo");
			if ((pid = fork()) == 0)
				execl(temp_name, name, tempfn, NULL);
			while (!waitpid(pid, &status, WNOHANG))
			{
				error = (WEXITSTATUS(status)<0);
				while (gtk_events_pending())
					gtk_main_iteration();
			}
		}
		if (strcmp(ad->type, "cpio") == 0) {
			create_cpio_archive(ad->full_name, temp_name);
		}
		g_free(name);
		g_free(path);
		g_free(temp_name);
	}
	else {
		remove(temp_name);
		g_free(name);
		g_free(path);
		g_free(temp_name);
		return -1;	//critical error
	}
	}
	else {
		for (i=0;i<gnochivemain->active_page->current_lines;i++)
		{
			if (gnochivemain->active_page->selected_rows[i] == 1) {
				gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
				if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path, "gzip", "-f", text, NULL))
					exec_status = FALSE;
				while (!waitpid(pid,&status,WNOHANG))
				{
					error=(WEXITSTATUS(status)<0);
					while (gtk_events_pending())
						gtk_main_iteration();
				}
			}
		}
	}
	remove(temp_name);
	if (exec_status)
		return 1;		//all ok
	else
		return 0;		// errors but keep-going
}

archivedesc*
open_real_archive(gchar *file_name)
{
	FILE *tmpfile = fopen(file_name, "r");
	int tmpsize;
	gchar *fname = NULL;
	gchar *ac_type = NULL;
	gchar *vfs_type;
	pager *new_page;

#ifdef _GLIB_2_0_
	/*works only against glib 2.0*/
	gchar *name = g_path_get_basename(file_name);
	gchar *path = g_path_get_dirname(file_name);
#else	
	/*compatibility agains glib 1.2*/
	gchar *name = g_strdup(g_basename(file_name));
	gchar *path = g_strdup(g_dirname(file_name));
#endif
	
	fseek(tmpfile, 0L, SEEK_END);
    tmpsize = ftell(tmpfile);
	if (!cmpare(path, ".")) {
		g_free(path);
		path = g_get_current_dir();
	}
	fname = g_concat_dir_and_file(path, name);
	ac_type = gnochive_get_archive_type(fname);
	/* if magic detection fails here force with vfs detection */
	vfs_type = gnochive_vfs_get_archive_type(fname);
	if (!ac_type && !vfs_type) tmpsize = 0;
	if (is_regular(file_name) && tmpfile && tmpsize>0)
	{
		archivedesc *ad = (archivedesc *)g_malloc(sizeof(archivedesc));
		if (ad)
		{
			fclose(tmpfile);
			if (!gnochivemain->gc_pager->next)
				gtk_notebook_set_show_tabs(GTK_NOTEBOOK(gnochivemain->noter), TRUE);
			ad->full_name = fname;
			ad->name=name;
			update_main(ad->full_name, 3);
			new_page = gnochive_pager_add(name);
			new_page->path = path;
			new_page->page_no =
					gnochive_notebook_get_pages(gnochivemain->gc_pager);
			new_page->ad = ad;
			ad->locked = FALSE;
			ad->volume = 0;
			ad->dict_size = NULL;
			ad->compress_level = NULL;
			ad->dict_size = NULL;
			
			if (select_open_function(new_page, ad))
			{
				new_page->opd_archive = TRUE;
				gnochivemain->active_page = new_page;
				gtk_notebook_set_page(GTK_NOTEBOOK(gnochivemain->noter), new_page->page_no);
				return ad;
			}
		}
	}
	if (!errno)
	{
		gchar *msg = g_strdup_printf(_("Could not open archive %s \nUnknown archive or file corrupted!"), file_name);
		GtkWidget *dialog = gnome_message_box_new(msg,GNOME_MESSAGE_BOX_ERROR, _("OK"), NULL);
		//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
		gtk_widget_show(GTK_WIDGET(dialog));
		g_free(msg);
	}
	else
		gnochive_error(errno, _("Could not open archive %s \n"), file_name);
	return NULL;
}

/* opens an archive based on file_name extension
    (eg. file.tar.bz2)
 */
 
gboolean
select_open_function			(pger *page, archivedesc *ad)
{
	gchar fbuffer[20];
	FILE *tempf = fopen(ad->full_name,"r");
	ad->type = NULL;
	gnochivemain->active_page->files_selected = 0;

	if(detect_lha(ad->full_name)!=0)
	{
		lha_open(page, ad->full_name);
		ad->type = g_strdup("lha");
		return TRUE;
	}
	if(detect_zoo(ad->full_name))
	{
		zoo_open(page, ad->full_name);
		ad->type = g_strdup("zoo");
		return TRUE;
	}
	if(detect_ace(ad->full_name))
	{
		ace_open(page, ad->full_name);
		ad->type = g_strdup("ace");
		return TRUE;
	}
	if(detect_rpm(ad->full_name))
	{
		//if (gnochivemain->gc_cfg->rpm_handle)
			//rpm_open_filesystem(page, ad->full_name);
		//else
			rpm_open(page, ad->full_name);
		ad->type = g_strdup("rpm");
		return TRUE;
	}
	if ((!cmpare(ad->full_name,"tar.bz2")) || (!cmpare(ad->full_name,"tbz2")))
	{
		tbz2_open(page, ad->full_name);
		ad->type = g_strdup("tbz2");
	}
	else
	{
		fread(fbuffer,4,1,tempf);
		if (tempf != 0) { // files that might have a magic number
			if(strncmp(fbuffer,RAR_MAGIC,4)==0)
			{
				rar_open(page, ad->full_name);
				ad->type=g_strdup("RAR");
			}
			if(strncmp(fbuffer, GZIP_MAGIC, 2)==0)
			{
				if (gzip_open(page, ad->full_name)==0)
					ad->type = g_strdup("gz");
				else
					ad->type = g_strdup("tgz");
			}
			if(strncmp(fbuffer, BZIP2_MAGIC, 2)==0)
			{
				if (bzip2_open(page, ad->full_name)==0)
					ad->type = g_strdup("bz2");
				ad->type = g_strdup("bz2");
			}
			if(strncmp(fbuffer, ZIP_MAGIC, 2)==0)
			{
				zip_open(page, ad->full_name);
				ad->type = g_strdup("zip");
			}
			if(strncmp(fbuffer, Z_MAGIC, 2)==0)
			{
				Z_open(page, ad->full_name);
				ad->type = g_strdup("comp");
			}
			if(strncmp(fbuffer, ARJ_MAGIC, 2)==0)
			{
				arj_open(page, ad->full_name);
				ad->type = g_strdup("arj");
			}
			if (!ad->type)
			{
				if (gnochive_istr_has_suffix(ad->full_name, ".tar"))
				{
					tar_open(page, ad->full_name);
					ad->type = g_strdup("tar");
				}
				if (gnochive_istr_has_suffix(ad->full_name, ".cpio"))
				{
					cpio_open(page, ad->full_name);
					ad->type = g_strdup("cpio");
				}
			}
		}
	}
	fclose(tempf);
	return TRUE;
}

/*
 * This function is bulit agains gnome_vfs stuff
 * returns the type of archive
 * this function will always return the first level of archive
 * (eg. won't return tar in file abc.tar.gz just the gz filter)
 */

gchar*
gnochive_get_archive_type(gchar* filename)
{
	gchar fbuffer[20];
	FILE *tempf;

	if (is_readable(filename))
	{
		if(detect_lha(filename)!=0)
			return g_strdup("lha");
		if(detect_zoo(filename))
			return g_strdup("zoo");
		if(detect_ace(filename))
			return g_strdup("ace");
		if(detect_rpm(filename))
			return g_strdup("rpm");
		tempf = fopen(filename,"r");
		fread(fbuffer,4,1,tempf);
		fclose(tempf);
		if (tempf!=0) { // files that might have a magic number
			if(strncmp(fbuffer,RAR_MAGIC,4)==0)
				return g_strdup("RAR");
			if(strncmp(fbuffer,GZIP_MAGIC,2)==0)
			{
				/* Avoid these extension as gnome_vfs said */
				if (gnochive_istr_has_suffix(filename,".gnumeric")
					|| gnochive_istr_has_suffix(filename,".abw")
					|| gnochive_istr_has_suffix(filename,".dia")
					|| gnochive_istr_has_suffix(filename,".pdf")) {
					return NULL;
					}
				return g_strdup("gz");
			}
			if(strncmp(fbuffer,BZIP2_MAGIC,2)==0)
			{
				return g_strdup("bz2");
			}
			if(strncmp(fbuffer,ZIP_MAGIC,2)==0)
				return g_strdup("zip");
			if(strncmp(fbuffer,Z_MAGIC,2)==0)
				return g_strdup("comp");
			if(strncmp(fbuffer,ARJ_MAGIC,2)==0)
				return g_strdup("arj");
			if (gnochive_istr_has_suffix(filename, ".cpio"))
				return g_strdup("cpio");
			if (gnochive_istr_has_suffix(filename, ".tar"))
				return g_strdup("tar");
		}
		return NULL;
	}
	else 
		return NULL;
}

void
refresh_arch		(archivedesc* ad)
{
	//memset(clist_line,0,sizeof(clist_line));
	select_open_function(gnochivemain->active_page, ad); // we just re-read archive
}

gchar*
get_arch_name_from_type	(gchar *arc_type, gchar *file_name, gint sub_type)
{
	if (!cmpare(arc_type,"RAR"))
		return g_strconcat(file_name,".","rar",NULL);
	if (!cmpare(arc_type,"gz"))
		return g_strconcat(file_name,".","gz",NULL);
	if (!cmpare(arc_type,"bz"))
		return g_strconcat(file_name,".","bz",NULL);
	if (!cmpare(arc_type,"tar"))
	{
		switch (sub_type) {
				case 0:	return g_strconcat(file_name,".","tar",NULL);
				case 1: return g_strconcat(file_name,".","tar",".gz",NULL);
				case 2:	return g_strconcat(file_name,".","tar",".bz2",NULL);
				case 3:	return g_strconcat(file_name,".","tar",".Z",NULL);
				}
			}
	if (!cmpare(arc_type,"zip"))
		return g_strconcat(file_name,".","zip",NULL);
	if (!cmpare(arc_type,"lha"))
		return g_strconcat(file_name,".","lha",NULL);
	if (!cmpare(arc_type,"zoo"))
		return g_strconcat(file_name,".","zoo",NULL);
	if (!cmpare(arc_type, "cpio"))
		return g_strconcat(file_name, ".", "cpio", NULL);
	/*we hopefuly won't get here*/
	return 0;
}

/*
 * removes selected files from archive
 */
 
void
delete_arc_file		(archivedesc *ad)
{
	gchar *text=NULL;
	gchar *tmp=NULL;
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	gchar *tfile2 = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	FILE *tmpf = fopen(tfile,"w");
	int i;
	int tf;
	tf = mkstemp(tfile);
	close (tf);
	tmpf = fopen(tfile,"w");

	mkstemp(tfile2);
	
if (tmpf) {
	if (strcmp(ad->type, "zoo")==0)
	{
	 fprintf(tmpf,"#!/bin/sh\n");
	 fprintf(tmpf,"%s Dqqq %s ",gnochivemain->gc_cfg->archs.zoo_path, ad->full_name);
	 for (i=0;i<gnochivemain->active_page->current_lines;i++)
		{
			if (gnochivemain->active_page->selected_rows[i] == 1)
			{
				gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
				gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
				fprintf(tmpf, "%s ", text);
			}
		}
	}
	else {
		for (i = 0;i<gnochivemain->active_page->current_lines;i++)
		{
			if (gnochivemain->active_page->selected_rows[i] == 1)
			{
				gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
				gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
				tmp = g_strconcat(ad->root, text, NULL);
				fprintf(tmpf, "%s\n", tmp);
				g_free(tmp);
			}
		}
	}
	fclose(tmpf);
	
	if (strcmp(ad->type, "RAR")==0)
	{
		tmp = g_strconcat("@", tfile, NULL);
		gnochive_exec_arch("/tmp/grardel", "grardel",
			gnochivemain->gc_cfg->archs.rar_path, "-inul",
			ad->full_name, tmp, NULL);
		wait(0);
	}
	tmp = g_strconcat("--files-from=", tfile, NULL);
	if (strcmp(ad->type, "tar")==0) {
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
			"tar", "--delete", "-f", ad->full_name, tmp, NULL);
		wait(0);
	}
	if (strcmp(ad->type, "tbz2")==0)
	{
		gnochive_exec_arch("/tmp/remflt", "remflt",
			gnochivemain->gc_cfg->archs.bz2_path, "-d", "-k",
			ad->full_name, "-c", tfile2, NULL);
		wait(0);
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
			"tar", "--delete", "-f", tfile2, tmp, NULL);
		wait(0);
		gnochive_exec_arch("/tmp/remflt", "remflt",
			gnochivemain->gc_cfg->archs.bz2_path,
			"-z", "-f", tfile2, "-c", ad->full_name, NULL);
		wait(0);
	}
	if (strcmp(ad->type,"tgz")==0)
	 {
		gnochive_exec_arch("/tmp/remflt", "remflt",
			gnochivemain->gc_cfg->archs.gzip_path,
			"-d", "-f", ad->full_name, "-c", tfile2, NULL);
		wait(0);
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
			"tar", "--delete", "-f", tfile2, tmp, NULL);
		wait(0);
		gnochive_exec_arch("/tmp/remflt", "remflt",
			gnochivemain->gc_cfg->archs.gzip_path,
			"-q", "-f", tfile2, "-c", ad->full_name, NULL);
		wait(0);
	 }
	if (strcmp(ad->type, "comp")==0)
	 {
		gnochive_exec_arch("/tmp/remflt", "remflt",
			gnochivemain->gc_cfg->archs.Z_path,"-d","-f",
			ad->full_name, "-c", tfile2, NULL);
		wait(0);
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
			"tar", "--delete", "-f", tfile2, tmp, NULL);
		wait(0);
		gnochive_exec_arch("/tmp/remflt", "remflt", 
			gnochivemain->gc_cfg->archs.Z_path, "-f", "-r",
			tfile2, "-c", ad->full_name, NULL);
		wait(0);
	 }
	if (strcmp(ad->type, "zip")==0)
	 {
		gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
			gnochivemain->gc_cfg->archs.zip_path, "-d", ad->full_name, "-xi", tfile, NULL);
		wait(0);
	 }
	if (strcmp(ad->type,"lha")==0)
	 {
		gnochive_exec_arch("/tmp/zpps", "zpps", " ",
			gnochivemain->gc_cfg->archs.lha_path, "dq", ad->full_name, tfile, NULL);
		wait(0);
	 }
	if (strcmp(ad->type,"zoo")==0)
	 {
		chmod(tfile,S_IWUSR | S_IXUSR | S_IRUSR);
		if (fork()==0) execl(tfile,tfile,NULL);
		wait(0);
	 } 
	remove(tfile);
	remove(tfile2);
 }
 g_free(tmp);
 g_free(tfile);
 g_free(tfile2);
}

/* obsolete */

void
extract_files				(gchar *file_name)
{
	FILE *tmpf;
	gchar *text;
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	int tf=mkstemp(tfile);
	
	if (strncmp(arc_type,"gz",2)==0)
	{
		gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),
				last_select_row,1,&text);
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path,
				"gzip", "-d", "-c", file_name,
				">", text, NULL);
	}
 	else {
		tmpf=fopen(tfile,"w+");
	    if (tf && tmpf) {
			/* We handle zip&zoo archives differently since it does not support listfile
	 		 *it's a dirty patch but it works
  	 		 */
			if (strcmp(arc_type,"zip")==0 || strcmp(arc_type,"zoo")==0) {
				fprintf(tmpf,"#!/bin/sh\n");
				fprintf(tmpf,"$1 $2 $3 ");
				for (i=0;i<gnochivemain->active_page->current_lines;i++) {
					if (gnochivemain->active_page->selected_rows[i]==1) {
						/* Maybe a fix for zoo arcs when deleting dirs */
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
						fprintf(tmpf,"%s ",text);
					}
				}
			fclose(tmpf);
			chmod(tfile,S_IRUSR | S_IXUSR);
			}
			else {
				for (i=0;i<gnochivemain->active_page->current_lines;i++) {
					if (gnochivemain->active_page->selected_rows[i]==1) {
						gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
						fprintf(tmpf,"%s\n",text);
					}
				}
				fclose(tmpf);
			}
	 if (cmpare(arc_type,"tar")==0)
	 {
	 if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"",file_name,tfile,NULL);
		wait(0);
	 }
	 if (cmpare(arc_type,"tgz")==0)
	 {
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0.3);
		stepping=1;
		//time_id=gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)mytime,(gpointer)stepping);
		if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"-z",file_name,tfile,NULL);
		wait(0);
	 }
	 if (cmpare(arc_type,"tbz2")==0)
	 {
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0.3);
		if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,gnochivemain->gc_cfg->tbz2parm,file_name,tfile,NULL);
		wait(0);
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
	 }
	 if (cmpare(arc_type,"comp")==0)
	 {
	 if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"-Z",file_name,tfile,NULL);
		wait(0);
	 }
	 if (cmpare(arc_type,"lha")==0)
	 {
	 if (fork()==0) execl("/tmp/zpps","zpps", " ", gnochivemain->gc_cfg->archs.lha_path,"xq",file_name, tfile,NULL);
		wait(0);
	 }
	 if (cmpare(arc_type,"zoo")==0)
	 {
		if (fork()==0) execl(tfile,tfile,gnochivemain->gc_cfg->archs.zoo_path,"xqqqOO",file_name,NULL);
		wait(0);
		unlink(tfile);
	 }
	 if (cmpare(arc_type,"zip")==0)
	 {
		if (fork()==0) execl(tfile,tfile,gnochivemain->gc_cfg->archs.unzip_path,"-qq",file_name,NULL);
		wait(0);
		remove(tfile);
	 }
	 if (cmpare(arc_type,"RAR")==0) {
				if (select_all==1) {
						if (fork()==0) execl(gnochivemain->gc_cfg->archs.rar_path,"rar","x","-o+","-y",pass_command,name,NULL);
						wait(0);
						}
				else {
					gchar *tmp=g_strconcat("@",tfile,NULL);
					if (fork()==0) execl("/tmp/grarextr","grarextr",gnochivemain->gc_cfg->archs.rar_path,pass_command,file_name,tmp,NULL);
					wait(0);
					gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
					total_sel_size=0;		// reset selected files size
					gnome_appbar_pop(GNOME_APPBAR(gnochivemain->mainbar));
					g_free(tmp);
				}
	 }
		/* Unfortunately unarj will extract whole archive contents :( */
		if (cmpare(arc_type,"arj")==0) {
			if (fork()==0) execl(gnochivemain->gc_cfg->archs.unarj_path,"unarj","x",file_name,NULL);
			wait(0);
		}
	}
}
g_free(tfile);
}

/* internal
 * returns the name of extracted file as a temporary file
 * output should be freed and no longer used,
 * and file erased
 */
gchar*
extract_file_from_archive		(gchar *arc_name, gchar *file)
{
	gchar *gc_type = gnochive_get_archive_type(arc_name);
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchive.ext.XXXXXX");
	int tf;
	
	tf = mkstemp(tfile);
	
	if (tf) {
		close(tf);
		//test_encrypted_archive(arc_name);
		if (!cmpare(gc_type,"zip"))
			gnochive_exec_arch("/tmp/remflt","remflt",
				gnochivemain->gc_cfg->archs.unzip_path,
				"-q", "-p", arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "RAR"))
			gnochive_exec_arch("/tmp/remflt", "remflt",
				gnochivemain->gc_cfg->archs.rar_path,
				"p", "-y", arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "lha"))
			gnochive_exec_arch("/tmp/zpps", "zpps", "nice", 
				gnochivemain->gc_cfg->archs.lha_path,
				"p", arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "zoo"))
			gnochive_exec_arch("/tmp/remflt", "remflt", "nice",
				gnochivemain->gc_cfg->archs.zoo_path,
				"xpq", arc_name, file, tfile, NULL);
		/*if (!cmpare(gc_type, "ace"))
			gnochive_exec_arch("/tmp/remflt", "remflt",
				gnochivemain->gc_cfg->archs.unace_path,
				"-xoy", arc_name, tfile, file, NULL);*/
		/* we don't need to check deeper into this types
		 * since single bz2,gz,comp archives does not 
		 * support multiple files
		 */
		if (!cmpare(gc_type, "gz"))
			gnochive_exec_arch("/tmp/remflt", "remflt",
				gnochivemain->gc_cfg->archs.tar_path,
				"-xzO", "-f", arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "bz2"))
			gnochive_exec_arch("/tmp/remflt", "remflt",
				gnochivemain->gc_cfg->archs.tar_path,
				gnochivemain->gc_cfg->tbz2parm, "-xOf",
				arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "comp"))
			gnochive_exec_arch("/tmp/remflt", "remflt",
				gnochivemain->gc_cfg->archs.tar_path,
				"-xZO", "-f", arc_name, file, tfile, NULL);
		if (!cmpare(gc_type, "cpio"))
		{
			remove(tfile);
			g_free(tfile);
			tfile = cpio_extract_file_from_archive(arc_name, file);
		}
		if (!cmpare(gc_type, "rpm"))
		{
			remove(tfile);
			g_free(tfile);
			tfile = rpm_extract_with_cpio(arc_name, file);
		}
		/* FIXME add plain tar */
			wait(0);
		g_free(gc_type);
		return tfile;
	}
	return NULL;
}

/*returns a pointer to the list of extracted files*/
gchar*
gnochive_extract_files_from_archive (gchar *arc_name)
{
	gchar *list;
	gchar *lfile;
	int lf;
	lfile = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	lf = mkstemp(lfile);
	close(lf);
	list = internal_create_list_file(lfile);
	if (list)
	{
		internal_extract(arc_name, list);
	}
	else {
	}
	remove(lfile);
	return NULL;
}

gchar*
internal_create_list_file	(gchar *list)
{
	FILE *tmpf = fopen(list,"w+");
	gchar *text;
	
	if (tmpf) {
		/* We handle zip&zoo archives differently since it does not support listfile
		 *it's a dirty patch but it works
		 */
		if (strcmp(arc_type,"zip")==0 || strcmp(arc_type,"zoo")==0) {
			fprintf(tmpf,"#!/bin/sh\n");
			fprintf(tmpf,"$1 $2 $3 ");
			for (i=0;i<gnochivemain->active_page->current_lines;i++) {
				if (gnochivemain->active_page->selected_rows[i]==1) {
					/* Maybe a fix for zoo arcs when deleting dirs */
					gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
					gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
					fprintf(tmpf,"%s ",text);
				}
			}
		fclose(tmpf);
		chmod(list,S_IRUSR | S_IXUSR);
		}
		else {
			for (i=0;i<gnochivemain->active_page->current_lines;i++) {
				if (gnochivemain->active_page->selected_rows[i]==1) {
					gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
					gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
					fprintf(tmpf,"%s\n",text);
				}
			}
			fclose(tmpf);
		}
		return list;
	}
	return NULL;
}

gboolean
internal_extract	(gchar *file_name,gchar *list)
{
	FILE *tmpf = fopen(list,"r");
	
	if (tmpf)
	{
		if (cmpare(arc_type,"tar")==0)
		{
			if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"",file_name,list,NULL);
			wait(0);
		}
		if (cmpare(arc_type,"tgz")==0)
		{
			gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0.3);
			stepping=1;
			//time_id=gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)mytime,(gpointer)stepping);
			if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"-z",file_name,list,NULL);
			wait(0);
		}
		if (cmpare(arc_type,"tbz2")==0)
		{
			gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0.3);
			if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,gnochivemain->gc_cfg->tbz2parm,file_name,list,NULL);
			wait(0);
			gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
		}
		if (cmpare(arc_type,"comp")==0)
		{
			if (fork()==0) execl("/tmp/tgzextr","tgzextr",gnochivemain->gc_cfg->archs.tar_path,"-Z",file_name,list,NULL);
			wait(0);
		}
		if (cmpare(arc_type,"lha")==0)
		{
			if (fork()==0) execl("/tmp/zpps","zpps", " ", gnochivemain->gc_cfg->archs.lha_path,"xq",file_name, list,NULL);
			wait(0);
		}
		if (cmpare(arc_type,"zoo")==0)
		{
			if (fork()==0) execl("/tmp/gcparse","gcparse",gnochivemain->gc_cfg->archs.zoo_path,"xqqqOO",file_name,NULL);
			wait(0);
			remove("/tmp/gcparse");
		}
		if (cmpare(arc_type,"zip")==0)
		{
			if (fork()==0) execl("/tmp/gcparse","gcparse",gnochivemain->gc_cfg->archs.unzip_path,"-qq",file_name,NULL);
			wait(0);
			remove("/tmp/gcparse");
		}
		if (cmpare(arc_type,"RAR")==0) {
			if (select_all==1) {
				if (fork()==0) execl(gnochivemain->gc_cfg->archs.rar_path,"rar","x","-o+","-y",pass_command,name,NULL);
				wait(0);
			}
			else {
				gchar *rp=g_strconcat("@",list,NULL);
				if (fork()==0) execl("/tmp/grarextr","grarextr",gnochivemain->gc_cfg->archs.rar_path,pass_command,file_name,rp,NULL);
				wait(0);
				gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
				total_sel_size=0;		// reset selected files size
				gnome_appbar_pop(GNOME_APPBAR(gnochivemain->mainbar));
				g_free(rp);
			}
		}
		/* Unfortunately unarj will extract whole archive contents :( */
		if (cmpare(arc_type,"arj")==0) {
			if (fork()==0) execl(gnochivemain->gc_cfg->archs.unarj_path,"unarj","x",file_name,NULL);
		wait(0);
		}
		return TRUE;
	}
	return FALSE;
}

/* Returns the temporary path where the file was extracted
 *
 */

gchar*
gnochive_extract_file	(gchar *file_name)
{
	gchar *name = gnochivemain->active_page->ad->full_name;
	gchar *path = gnochivemain->active_page->path;
	gchar *archive = g_strconcat(path,G_DIR_SEPARATOR_S,name,NULL);
	g_print("Archive: %s\n",archive);
	g_print("File: %s\n",file_name);
	
	
	
	g_free(archive);
	return NULL;
}

/*extracts a file gzipped/bzipped2 previously*/

void
extract_single				(void)
{
	gchar *text=NULL;
	gchar *temp=NULL;
	guint time_id;
	pid_t pid;
	int status;
	gboolean error;
	gchar *tfile=g_strdup("gchiveXXXXXX");
	/*check currentdir if it's writable*/
	mkstemp(tfile);
	
	if (gnochive_can_create(tfile)) {
		time_id=set_activity();
		for (i=0;i<gnochivemain->active_page->current_lines;i++) {
			if (gnochivemain->active_page->selected_rows[i] == 1 && i != 0 && i != 1) {
				gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
				if (!cmpare(text,".gz")) {
						if ((pid = fork()) == 0) execl(gnochivemain->gc_cfg->archs.gzip_path,"gzip","-d",text,NULL);
						while (!waitpid(pid,&status,WNOHANG))
						{
							error=(WEXITSTATUS(status)<0);
							while (gtk_events_pending())
								gtk_main_iteration();
						}
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
						temp=g_strdup_printf(N_("Extracted gzipped file: %s !"),text);
						gnome_app_flash(GNOME_APP(gnochivemain->mainwin),temp);
						gnochivemain->active_page->files_selected=0;
						g_free(temp);
						}
				if (!cmpare(text,".bz2")) {
						if ((pid = fork())==0) execl(gnochivemain->gc_cfg->archs.bz2_path,"bzip2","-d",text,NULL);
						while (!waitpid(pid,&status,WNOHANG))
						{
							error=(WEXITSTATUS(status)<0);
							while (gtk_events_pending())
								gtk_main_iteration();
						}
						gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
						temp=g_strdup_printf(N_("Extracted bzip2-ed file: %s !"),text);
						gnome_app_flash(GNOME_APP(gnochivemain->mainwin),temp);
						gnochivemain->active_page->files_selected=0;
						g_free(temp);
						}
			}
		}
	gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
	read_pwd_files(".");
	last_select_row=0;
	gnochivemain->active_page->files_selected=0;
	total_sel_size=0;
	g_free(gnochivemain->active_page->selected_rows);
	gnochivemain->active_page->selected_rows=g_malloc0(gnochivemain->active_page->current_lines);
	remove_activity(time_id);
	}
	else gnochive_error(errno,_("Cannot extract!\n"));
	g_free(tfile);
}

/* gc_config->overwrite */

gboolean
extract_to_path				(gchar *file_name, gchar *path, gchar *files)
{
	pid_t pid = 0;
	int status;
	gboolean error;
	gchar fbuffer[20];
	gchar *temp = NULL;
	FILE *tempf = fopen(file_name, "r");
	gchar *tdir;
	
	if (is_regular(file_name) && tmpfile)
	{
		archivedesc *ad = (archivedesc *)g_malloc(sizeof(archivedesc));
		
		if (ad)
		{
			ad->full_name = g_strdup(file_name);
			ad->locked = FALSE;
			ad->volume = 0;
			ad->dict_size = NULL;
			ad->compress_level = NULL;
			ad->dict_size = NULL;
			ad->type = NULL;
			
	if (tempf)
	{
		if(detect_lha(ad->full_name) != 0)
		{
			lha_open(gnochivemain->active_page, ad->full_name);
			ad->type = g_strdup("lha");
			return TRUE;
		}
		if(detect_zoo(ad->full_name))
		{
			tdir = g_get_current_dir();
			chdir(path);
			if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
			{
				temp = g_strdup(g_basename(files));
				chmod(files, S_IRUSR | S_IWUSR | S_IXUSR);
				if (gnochivemain->gc_cfg->overwrite)
					gnochive_exec_arch(files, temp, gnochivemain->gc_cfg->archs.zoo_path,
						"-xOO", file_name, NULL);
				else
					gnochive_exec_arch(files, temp, gnochivemain->gc_cfg->archs.zoo_path,
						"-xqqq", file_name, NULL);
				g_free(temp);
			}
			else
			{
				if (gnochivemain->gc_cfg->overwrite)
					gnochive_exec_arch(gnochivemain->gc_cfg->archs.zoo_path, "zoo", "-x:qqqOO",
						file_name, NULL);
				else
					gnochive_exec_arch(gnochivemain->gc_cfg->archs.unzip_path, "zoo", "-x:qqq",
						file_name, NULL);
			}
			chdir(tdir);
			g_free(tdir);
			ad->type = g_strdup("zoo");
			return TRUE;
		}
		if ((!cmpare(ad->full_name, "tar.bz2")) 
			|| (!cmpare(ad->full_name, "tbz2")))
		{
			if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
			{
				temp = g_strconcat("--files-from=", files, NULL);
				gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
					"tar", "-C", path, "-x", gnochivemain->gc_cfg->tbz2parm,
					"-f", ad->full_name, temp, NULL);
				g_free(temp);
			}
			else
				gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
					"tar", "-C", path, "-x", gnochivemain->gc_cfg->tbz2parm,
					"-f", ad->full_name, NULL);
			ad->type = g_strdup("tbz2");
		}
		else
		{
			tempf = fopen(ad->full_name, "r");
			fread(fbuffer, 4, 1, tempf);
			fclose(tempf);
			if (tempf != 0) { // files that might have a magic number
				if(strncmp(fbuffer, RAR_MAGIC, 4) == 0)
				{
					if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
					{
						temp = g_strconcat("@", files, NULL);
						if (gnochivemain->gc_cfg->overwrite)
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.rar_path,
								"rar", "x", "-o+", "-inul",
								"-y", file_name, temp, path, NULL);
						else
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.rar_path,
							"rar", "x", "-o-", "-inul", "-y", file_name, temp, path, NULL);
						g_free(temp);
					}
					else
					{
						if (gnochivemain->gc_cfg->overwrite)
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
								"rar","x", "-o+", //"-inul",
									"-y", file_name, path, NULL);
						else
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
								"rar","x", "-o-", //"-inul",
									"-y", file_name, path, NULL);
					}
					ad->type = g_strdup("RAR");
				}
				if(strncmp(fbuffer, GZIP_MAGIC, 2) == 0)
				{
					if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
					{
						temp = g_strconcat("--files-from=",files,NULL);
						gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path, "tar", "-C", path,
								"-xzf", file_name, temp, NULL);
						g_free(temp);
					}
					else
						gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path, "tar", "-C", path,
								"-xzf", file_name, NULL);
					//if (gzip_open(ad->full_name)==0) ad->type=g_strdup("gz");
					//else 
					ad->type = g_strdup("tgz");
				}
				if(strncmp(fbuffer, BZIP2_MAGIC, 2) == 0)
				{
					/*FIXME bzip2_open not needed here*/
					if (bzip2_open(gnochivemain->active_page, ad->full_name) == 0) ad->type = g_strdup("bz2");
					ad->type = g_strdup("bz2");
				}
				if(strncmp(fbuffer, ZIP_MAGIC, 2) == 0)
				{
					if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
					{
						temp = g_strdup(g_basename(files));
						chmod(files, S_IRUSR | S_IWUSR | S_IXUSR);
						if (gnochivemain->gc_cfg->overwrite)
							gnochive_exec_arch(files, temp, gnochivemain->gc_cfg->archs.unzip_path,
									"-oqq", file_name, "-d", path, NULL);
						else
							gnochive_exec_arch(files, temp, gnochivemain->gc_cfg->archs.unzip_path,
									"-qq", file_name, "-d", path, NULL);
						g_free(temp);
					}
					else
					{
						if (gnochivemain->gc_cfg->overwrite)
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.unzip_path, "unzip", "-oqq",
								file_name, "-d", path, NULL);
						else
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.unzip_path, "unzip", "-qq",
								file_name, "-d", path, NULL);
					}
					ad->type = g_strdup("zip");
				}
				if(strncmp(fbuffer, Z_MAGIC, 2) == 0)
				{
					if ((gnochivemain->gc_cfg->all) || (files != NULL))
					{
						temp = g_strconcat("--files-from=",files,NULL);
						gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,"tar", "-C", path,
								"-xZvf", file_name, files, NULL);
						g_free(temp);
					}
					else
						gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path, "tar", "-C", path,
								"-xZvf", file_name, NULL);
					ad->type = g_strdup("comp");
				}
				if(strncmp(fbuffer, ARJ_MAGIC, 2) == 0)
				{
					gnochive_exec_arch(gnochivemain->gc_cfg->archs.unarj_path,
						"unarj", "x", file_name, NULL);
					ad->type = g_strdup("arj");
				}
				if(strncmp(fbuffer, RPM_MAGIC, 4) == 0)
				{
					gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchive.ext.XXXXXX");
					gchar *tfile2 = g_concat_dir_and_file(TMPDIR, "gchive.ext.XXXXXX");
					int tf, tf2;
					FILE *tmp;
					
					tf = mkstemp(tfile);
					tf2 = mkstemp(tfile2);
						
					if (tf && tf2)
					{
						close(tf);
						close(tf2);
						tmp = fopen(tfile2, "w");
						fprintf(tmp, "#!/bin/sh\n");
						fprintf(tmp, "%s \"%s\" >%s \n",
						gnochivemain->gc_cfg->archs.rpm2cpio_path,
						ad->full_name, tfile);
						fclose(tmp);
						chmod(tfile2, S_IRUSR | S_IWUSR | S_IXUSR);
						if (gnochive_exec_arch(tfile2, tfile2, NULL))
							wait(0);
						if ((gnochivemain->gc_cfg->all) || (files != NULL))
						{
							temp = g_strconcat("--file=", tfile, NULL);
							chdir(path);
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.cpio_path,
								"cpio",  "-icumd", "-E", files, temp, NULL);
							wait(0);
							g_free(temp);
						}
						else
						{
							temp = g_strconcat("--file=", tfile, NULL);
							chdir(path);
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.cpio_path,
								"cpio", "-icumd", temp, NULL);
							wait(0);
							g_free(temp);
						}
					}
					remove(tfile2);
					remove(tfile);
					g_free(tfile2);
					g_free(tfile);
					ad->type = g_strdup("rpm");
				}
				if (!ad->type)
				{
					if (gnochive_istr_has_suffix(ad->full_name, ".tar"))
					{
						if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
						{
							temp = g_strconcat("--files-from=", files, NULL);
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
								"tar", "-C", path, "-xvf", ad->full_name, temp, NULL);
							g_free(temp);
						}
						else
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
								"tar", "-xvf", ad->full_name, "-C", path, NULL);
						ad->type = g_strdup("tar");
					}
					if (gnochive_istr_has_suffix(ad->full_name, ".cpio"))
					{
						if ((gnochivemain->gc_cfg->all != TRUE) && (files != NULL))
						{
							temp = g_strconcat("--file=", ad->full_name, NULL);
							chdir(path);
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.cpio_path,
								"cpio",  "-icumd", "-E", files, temp, NULL);
							g_free(temp);
						}
						else
						{
							temp = g_strconcat("--file=", ad->full_name, NULL);
							chdir(path);
							gnochive_exec_arch(gnochivemain->gc_cfg->archs.cpio_path,
								"cpio", "-icumd", temp, NULL);
							g_free(temp);
						}
						ad->type = g_strdup("cpio");
					}
				}
			}
		}
	}}

/*	if (!cmpare(arc_type,"lha"))
		if ((pid=fork())==0)
			execl("/tmp/zpps","zpps",gnochivemain->gc_cfg->archs.lha_path,"xq",file_name,"","/tmp/gcparse",NULL);*/
	/*if (!cmpare(arc_type,"zoo"))
		if (fork()==0)
			execl("/tmp/gcparse","gcparse",gnochivemain->gc_cfg->archs.zoo_path,"xqqqOO",file_name,NULL);
	if (cmpare(arc_type,"RAR")==0) {
		    if (select_all==1) {
				    if (fork()==0) execl(gnochivemain->gc_cfg->archs.rar_path,"rar","x","-o+","-y",pass_command,name,NULL);
				    wait(0);
				    }
		    else {
			    if (fork()==0) execl("/tmp/grarextr","grarextr",gnochivemain->gc_cfg->archs.rar_path,pass_command,file_name,"@/tmp/gcparse",NULL);
			wait(0);
			gnome_appbar_set_progress(GNOME_APPBAR(statusbar1),1);
			total_sel_size=0;		// reset selected files size
			gnome_appbar_pop(GNOME_APPBAR(statusbar1));
		    }
 }*/

	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	return TRUE;
	}
	gnochive_error(errno,_("Could not open archive %s \n"),file_name);
	return FALSE;
}

/* 
 *  cannot test plain tar (tar does not support it)
 */
 
gboolean
test_archive				(gchar *file_name)
{
	FILE *tmpf;
	gchar *tmpfile=NULL;
	gchar *buff=NULL;
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	gchar *ac_type = NULL;
	GtkWidget *msg;
	int tmpsize;
	
	mkstemp(tfile);
	if (gnochivemain->active_page->opd_archive) {
		if (cmpare(gnochivemain->active_page->ad->type, "RAR") == 0)
		if (fork() == 0)
			execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.rar_path, 
					"t", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "bz2") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.bz2_path,
					"-vt", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "gz") == 0)
		if (fork() == 0)
			execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.gzip_path,
					"-vt", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "zip") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.zip_path,
					"-T", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "arj") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.unarj_path,
					"t", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "ace") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.unace_path,
					"t", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "lha") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.lha_path,
					"t", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "zoo") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.zoo_path,
					"xNd", file_name, tfile, NULL);
		if (cmpare(gnochivemain->active_page->ad->type, "rpm") == 0)
		{
		}
	}
	else {
		ac_type = gnochive_get_archive_type(file_name);
		if (!ac_type)
			return FALSE;
		g_free(ac_type);
		if (gnochive_check_file(file_name,0))
		{
			if (cmpare(file_name, ".rar") == 0)
				if (fork() == 0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.rar_path,
							"t", file_name, tfile, NULL);
			if (cmpare(file_name, ".bz2") == 0
				|| cmpare(file_name, ".tbz2") == 0
				|| cmpare(file_name, ".tbz") == 0)
				if (fork() == 0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.bz2_path,
							"-vt", file_name, tfile, NULL);
			if (cmpare(file_name, ".gz") == 0
				|| cmpare(file_name, ".tgz") == 0)
				if (fork() == 0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.gzip_path,
							"-vt", file_name, tfile, NULL);
			if (cmpare(file_name, ".zip") == 0)
				if (fork() == 0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.zip_path,
							"-T", file_name, tfile, NULL);
			if (cmpare(file_name, ".arj") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.unarj_path,
						"t", file_name, tfile, NULL);
			if (cmpare(file_name, ".ace") == 0)
			if (fork() == 0)
				execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.unace_path,
						"t", file_name, tfile, NULL);
			if (cmpare(file_name, ".lha") == 0)
				if (fork()==0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.lha_path,
							"t", file_name, tfile, NULL);
			if (cmpare(file_name, ".zoo") == 0)
				if (fork()==0)
					execl("/tmp/gtest", "gtest", gnochivemain->gc_cfg->archs.zoo_path,
							"xNd", file_name, tfile, NULL);	
		}
		else {
			gnochive_error(errno, _("Could not test archive %s \n"), file_name);
			return FALSE;
		}
	}
	wait(0);
	tmpf = fopen(tfile,"r");
	if (tmpf) {
		fseek(tmpf,0L,SEEK_END);
		tmpsize = ftell(tmpf);
		tmpfile = g_malloc0(tmpsize);
		fseek(tmpf, 0L, SEEK_SET);
		fread(tmpfile, tmpsize, 1, tmpf);
		fclose(tmpf);
		if (	(cmpare(tmpfile, "Error:") == 0)
			|| (cmpare(tmpfile, "error:") == 0)
			|| (cmpare(tmpfile, "FATAL:") == 0)
			|| (!cmpare(tmpfile, "CRC-errors"))
			)
		{
			buff = g_strdup_printf(_("%s: contains errors!"), file_name);
			msg = gnome_message_box_new(buff,GNOME_MESSAGE_BOX_ERROR, _("OK"), NULL);
			gnome_dialog_run_and_close(GNOME_DIALOG(msg));
		}
		else if ((cmpare(tmpfile,"OK")==0)
				|| (cmpare(tmpfile,"ok")==0)
				|| (cmpare(tmpfile,"All OK")==0)
				|| (cmpare(tmpfile,":  o")==0)
				) {
				buff = g_strdup_printf(_("%s: check ok!"),file_name);
				msg = gnome_message_box_new(buff,GNOME_MESSAGE_BOX_INFO, _("OK"), NULL);
				/* FIXME! seems gnome_dialog is alwayz modal */
				gnome_dialog_run_and_close(GNOME_DIALOG(msg));
			}
		g_free(buff);
		g_free(tmpfile);
		remove(tfile);
		return TRUE;
	}
	g_free(tfile);
	return FALSE;
}

void
quick_test				(void)
{
	GtkWidget *terror;
	gchar *text=NULL;
	gchar *pwd=NULL;
	gchar *full_path=NULL;
	guint i;
	while (gtk_events_pending())
		gtk_main_iteration();
	if (!gnochivemain->active_page->opd_archive) {
		for (i=0;i<gnochivemain->active_page->current_lines;i++) {
			if (gnochivemain->active_page->selected_rows[i]==1 && i!=0 && i!=1) {
				gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist),i,1,&text);
				gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist),i,0);
				pwd = g_get_current_dir();
				full_path = g_concat_dir_and_file(pwd,text);
				if (!gnochive_istr_has_suffix(full_path,".tar") 
					&& !gnochive_istr_has_suffix(full_path,".tar.Z")
					&& !gnochive_istr_has_suffix(full_path,".cpio")
					&& !gnochive_istr_has_suffix(full_path,".rpm"))
					(void *)test_archive(full_path);
				else {
					terror = gnome_message_box_new(_("Plain tar, cpio, and tar.Z archives cannot be tested!"),
						GNOME_MESSAGE_BOX_INFO, _("OK"), NULL);
				    gnome_dialog_run_and_close(GNOME_DIALOG(terror));
				    }
			}
		}
	}
	else
	{
		if (!cmpare(gnochivemain->active_page->ad->type, "tar") 
			&& !cmpare(gnochivemain->active_page->ad->type, "cpio")
			&& !cmpare(gnochivemain->active_page->ad->type, "comp")
			&& !cmpare(gnochivemain->active_page->ad->type, "rpm")
			)
			test_archive(gnochivemain->active_page->ad->full_name);
		else {
			terror=gnome_message_box_new(_("Plain tar, cpio, and tar.Z archives cannot be tested!"),GNOME_MESSAGE_BOX_INFO,_("OK"),NULL);
			gnome_dialog_run_and_close(GNOME_DIALOG(terror));
			}
	}
	g_free(pwd);
	g_free(full_path);
}

void
display_comments		(GtkWidget *widget)
{
	FILE *commfile,*tmp;
	char *cmmfile;
	int commsize;
	gint q=0;
	guint textlen;
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	gchar *tfile2 = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	gchar *tfile3 = g_concat_dir_and_file(TMPDIR, "gchiveXXXXXX");
	int tf, tf2, tf3;
	GtkWidget *text1;
	
	if (gnochivemain->active_page->opd_archive)
	{
		tf = mkstemp(tfile);
		tf2 = mkstemp(tfile2);
		tf3 = mkstemp(tfile3);
		close(tf);
		close(tf2);
		close(tf3);
		
		if (cmpare(gnochivemain->active_page->ad->type, "RAR")==0)
		{
			gnochive_exec_arch(gnochivemain->gc_cfg->archs.rar_path,
				"rar", "-inul", "-o+", "-y", "cw",
				gnochivemain->active_page->ad->full_name, tfile3,0);
		}
		if (cmpare(gnochivemain->active_page->ad->type, "zip")==0)
		{
			gnochive_exec_arch("/tmp/zpvw", "zpvw", 
			gnochivemain->gc_cfg->archs.unzip_path, "-z",
			gnochivemain->active_page->ad->full_name, tfile3, NULL);
		}
		if (cmpare(gnochivemain->active_page->ad->type, "zoo")==0)
		{
			/* do some magic for grabing zoo comments */
			tmp=fopen(tfile, "w");
			fprintf(tmp, "#!/bin/sh\n");
			fprintf(tmp, "cat > %s <<EOF\n", tfile2);
			fprintf(tmp, "\nEOF\n");
			fprintf(tmp, "%s cA %s < %s >%s 2>/dev/null\n",
				gnochivemain->gc_cfg->archs.zoo_path,
				gnochivemain->active_page->ad->full_name, tfile2, tfile3);
			fclose(tmp);
			chmod(tfile, S_IRUSR | S_IWUSR | S_IXUSR);
			if (fork()==0) 
				execl(tfile, tfile, NULL);
		}
		wait(0);
		remove(tfile);
		remove(tfile2);
		g_free(tfile);
		g_free(tfile2);
		if ((commfile = fopen(tfile3, "r"))!=0)
		{
			fseek(commfile, 0L, SEEK_END);
			commsize = ftell(commfile);
			cmmfile=malloc(commsize);
			fseek(commfile,0L,SEEK_SET);
			fread(cmmfile,commsize,1,commfile);
			fclose(commfile);
			if (cmpare(gnochivemain->active_page->ad->type, "zip")==0
				|| cmpare(gnochivemain->active_page->ad->type, "zoo")==0)
			{
				while (cmmfile[q]!='\n') q++;
				q++;
			}
			text1 = lookup_widget(GTK_WIDGET(widget), "text4");
			textlen = gtk_text_get_length(GTK_TEXT(text1));
			gtk_text_set_point(GTK_TEXT(text1), 0);
			gtk_text_forward_delete(GTK_TEXT(text1), textlen);
			gtk_text_freeze(GTK_TEXT(text1));
			gtk_text_insert(GTK_TEXT(text1), NULL,
				&text1->style->black, NULL, cmmfile+q, commsize-q);
			gtk_text_thaw(GTK_TEXT(text1));
			textlen = gtk_text_get_length(GTK_TEXT(text1));
			remove(tfile3);
			g_free(cmmfile);
			g_free(tfile3);
		}
	}
}

void
convert_arch				(gchar *type)
{
	gchar myarc[0x100]="\0";
	gint ps = 0;
	gchar pat[0x100];
	pid_t pid = 0;
	int status;
	gboolean error;
	
	
	if (!cmpare(gnochivemain->active_page->ad->type, "tgz")) {
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path,
			"gzip", "-d", gnochivemain->active_page->ad->full_name, NULL);
		strcpy(pat, "bz2");
		step=0.005;progr=0.1;
		ps=2;
	}
	if (!cmpare(gnochivemain->active_page->ad->type, "tbz2")) {
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path,
			"bzip2", "-d", gnochivemain->active_page->ad->full_name, NULL);
		strcpy(pat, "gz");
		step=0.001;progr=0.1;
		ps=3;
	}
	if (!cmpare(gnochivemain->active_page->ad->type, "comp")) {
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path,
			"compress", "-d", gnochivemain->active_page->ad->full_name, NULL);
		strcpy(pat, "gz");
		ps=1;
	}
	//time_id=gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)mytime,NULL);
	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	//gtk_timeout_remove(time_id);
	gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0);
	strncpy(myarc, gnochivemain->active_page->ad->full_name,
		strlen(gnochivemain->active_page->ad->full_name)-ps-1);
	if (!cmpare(type, "tbz2"))
	{
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path,
			"bzip2", myarc, NULL);
		step=0.0007;progr=0.1;
	}
	if (!cmpare(type, "tgz"))
	{
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path,
			"gzip", myarc, NULL);
		step=0.001;progr=0.1;
	}
	if (!cmpare(type, "comp"))
		gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path,
			"compress", myarc, NULL);
	//time_id=gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)mytime,NULL);
	//while (!waitpid(0,NULL,WNOHANG))
	//{
	//gtk_main_iteration();
	//if (progr) progr=0;
	//}
	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	strcat(myarc, ".");
	strcat(myarc, pat);
	//gtk_timeout_remove(time_id);
	//gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
	g_snprintf(buff,90,_("Archive converted!"));
	gnome_app_flash(GNOME_APP(gnochivemain->mainwin), buff);
}

/* Checks if an archive exists if so
 * prompts the user for update or overwrite
 * file_name must be asbolute
 * return 2 if archive does not exist
 */

gint
gnochive_check_arch_update(gchar *file_name)
{
	GtkWidget *mesgbox;
	GtkWidget *dialog_vbox;
	/*if archive exists*/
	if (g_file_test(file_name, G_FILE_TEST_EXISTS))
	{
		mesgbox = gnome_message_box_new(_("Archive allready exists!\n What shall I do ?"),
			GNOME_MESSAGE_BOX_QUESTION, NULL);
		gtk_window_set_policy (GTK_WINDOW (mesgbox), FALSE, FALSE, FALSE);
		dialog_vbox = GNOME_DIALOG (mesgbox)->vbox;
		gtk_widget_show (dialog_vbox);
		gnome_dialog_append_button (GNOME_DIALOG (mesgbox),
				    "Update");
		gnome_dialog_append_button (GNOME_DIALOG (mesgbox),
				    "Overwrite");
		gnome_dialog_append_button (GNOME_DIALOG (mesgbox),
				    GNOME_STOCK_BUTTON_CANCEL);
		return gnome_dialog_run_and_close(GNOME_DIALOG(mesgbox));
	}
	return 3;
	
}

/* frees the archivedesc structure leaf by leaf returning true on success */
/* FIXME remove the temp files too */

gboolean
gnochive_close_archive(archivedesc *ad)
{
	pager *dest_pg = gnochivemain->active_page;
	guint page = gnochivemain->active_page->page_no;
	g_free(ad->full_name);
	g_free(ad->type);
	//g_free(ad->name);
	if (ad->compress_level) g_free(ad->compress_level);
	if (ad->locked) g_free(ad->passwd);
	if (ad->dict_size) g_free(ad->dict_size);
	g_free(ad);
	g_free(gnochivemain->active_page->selected_rows);
	gtk_widget_destroy(gnochivemain->active_page->clist);
	g_free(gnochivemain->active_page->path);
	while (dest_pg->next!=NULL) {
		dest_pg=(pager *)dest_pg->next;
		dest_pg->page_no=dest_pg->page_no-1;
	}
	if (!gnochive_page_destroy(gnochivemain->gc_pager,gnochivemain->active_page))
		g_error("Critical: Coudn't free page!");
	if (gnochivemain->gc_pager->next==NULL)
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(gnochivemain->noter),FALSE);
	gtk_notebook_remove_page(GTK_NOTEBOOK(gnochivemain->noter),page);
	return TRUE;
}

void
read_pwd_files(char *pwd)
{

	int line=0;
	gchar *clist_line[3][8];
	gchar *cur_dir;
	gchar buff[0x100];
	DIR *reading;
	struct dirent *next;
	struct stat statbuf;
	gchar attr[11];
	gint pixset=1;
	gchar *full=NULL;

	cur_dir = g_get_current_dir();
	if (is_readable(pwd))
	{
		gnochivemain->active_page->current_lines=0;
		
		gtk_entry_set_text(GTK_ENTRY(gnochivemain->location_entry),cur_dir);
		memset(buff,0,strlen(buff));
		full=g_strconcat(GNOCHIVE_VER,_(":"),cur_dir,NULL);
		gtk_window_set_title(GTK_WINDOW(gnochivemain->mainwin),full);
		g_free(full);
		reading=opendir(pwd);
		gtk_clist_freeze(GTK_CLIST(gnochivemain->active_page->clist));
		gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
		memset(clist_line,0,sizeof(clist_line));
		
		next=readdir(reading);
		next=readdir(reading);
		
		while((next=readdir(reading))!=NULL)
		{
			stat((char *)next->d_name,&statbuf); // do a normal stat for sorting sym from symdirectories
			xfer_mode(statbuf.st_mode,attr);
			if (attr[0]=='d') (gchar*)clist_line[0][2]="dir";
			else (gchar*)clist_line[0][2]=0;
			(gchar*)clist_line[0][1]=(char *)next->d_name;
			if (!gnochivemain->gc_cfg->res_syms) lstat((char *)clist_line[0][1],&statbuf);
			else stat((char *)clist_line[0][1],&statbuf);
			g_snprintf(buff,50,"%-10ld",statbuf.st_size);
			(gchar*)clist_line[0][3]=(gchar *)buff;
			(gchar*)clist_line[0][5]=ctime(&statbuf.st_atime);
			xfer_mode(statbuf.st_mode,attr);
			(gchar*)clist_line[0][6]=(gchar*)attr;
			gnochivemain->active_page->current_lines++;
			if (attr[0]=='d') {
					gtk_clist_insert(GTK_CLIST(gnochivemain->active_page->clist), 0, clist_line[0]);//prepend
					gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist), 0, 0, dirpix, dirmask);
					}
				else {
						gtk_clist_insert(GTK_CLIST(gnochivemain->active_page->clist), 0, clist_line[0]);//append
						if (cmpare(clist_line[0][1],".xpm")==0) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,xpmpix,xpmmask);pixset=0;}
						//*FIXME!*if (cmpare(clist_line[0][1],".rpm")==0) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,rpm1pix,rpm1mask);pixset=0;}
						//*FIXME!*if (clist_line[0][6][0]=='l') {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,sympix,symmask);pixset=0;}
						if (pixset==1) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,docpix,docmask);pixset=1;}
						pixset=1;
					}
			line++;
		}
		gtk_clist_set_sort_column(GTK_CLIST(gnochivemain->active_page->clist),1);
		gtk_clist_set_sort_type(GTK_CLIST(gnochivemain->active_page->clist),GTK_SORT_ASCENDING);
		gtk_clist_set_auto_sort(GTK_CLIST(gnochivemain->active_page->clist),1);
		gtk_clist_set_sort_column(GTK_CLIST(gnochivemain->active_page->clist),2);
		gtk_clist_set_sort_type(GTK_CLIST(gnochivemain->active_page->clist),GTK_SORT_DESCENDING);
		gtk_clist_sort(GTK_CLIST(gnochivemain->active_page->clist));
		gtk_clist_set_auto_sort(GTK_CLIST(gnochivemain->active_page->clist),0);
		memset(clist_line,0,sizeof(clist_line));
		clist_line[0][1]="..";
		gtk_clist_prepend(GTK_CLIST(gnochivemain->active_page->clist),clist_line[0]);
		gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,dotpix,dotmask);
		clist_line[0][1]=".";
		gtk_clist_prepend(GTK_CLIST(gnochivemain->active_page->clist),clist_line[0]);
		gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,rdpix,rdmask);
		gtk_clist_thaw(GTK_CLIST(gnochivemain->active_page->clist));
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0);
		gnochivemain->active_page->files_selected=0;
		gnochivemain->active_page->current_lines+=2;
	}
	g_free(cur_dir);
}

gboolean
is_archive(gchar *file_name)
{
	if (!cmpare(file_name,".gz"))
		return TRUE;
	if (!cmpare(file_name,".bz2"))
		return TRUE;
	/*if (!cmpare(text,".tgz"))
		return TRUE;
	if (!cmpare(text,".tbz2"))
		return TRUE;
	if (!cmpare(text,"tbz"))
		return TRUE;*/
	return FALSE;
}

static 
guint op_time(gpointer data)
{
	GDK_THREADS_ENTER();
	progr=progr+step;
	if (progr<=1)
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),progr);
	else gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
	GDK_THREADS_LEAVE();
	return TRUE;
}

guint
set_activity(void)
{
	gtk_progress_set_activity_mode(gnome_appbar_get_progress(GNOME_APPBAR(gnochivemain->mainbar)),TRUE);
	//gtk_progress_bar_set_activity_step(GTK_PROGRESS_BAR(statusbar1),10);
	while (gtk_events_pending())
		gtk_main_iteration();
	step=0.003;
	progr=0.1;
	return gtk_timeout_add(TIMEOUT_GTK,(GtkFunction)op_time,NULL);
	
}

void
remove_activity(guint time)
{
	gtk_timeout_remove(time);
	gtk_progress_set_activity_mode(gnome_appbar_get_progress(GNOME_APPBAR(gnochivemain->mainbar)),FALSE);
	gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),0);
}

guint
explode_archive(GtkCList *clist, guint vline,
				gchar *rbase, gchar *clist_line[])
{
	g_print("rbase:%s\n",rbase);
	g_print("cline:%s\n",clist_line[1]);
	if (sort_dirs(clist_line[1])) {
		if (!rbase) {
			rbase = extract_root_word(clist_line[1], 1);
			gtk_clist_append(GTK_CLIST(clist), clist_line);
			gtk_clist_set_pixmap(GTK_CLIST(clist), vline, 0, dirpix, dirmask);
			vline++;
		}
		else if (cmpare(clist_line[1], rbase) != 0) {
			g_free(rbase);
			rbase = extract_root_word(clist_line[1], 1);
			gtk_clist_append(GTK_CLIST(clist), clist_line);
			gtk_clist_set_pixmap(GTK_CLIST(clist), vline, 0, dirpix, dirmask);
			vline++;
		}
	}
	else if (!process_line(clist_line[1])) {
		gtk_clist_append(GTK_CLIST(clist), clist_line);
		gtk_clist_set_pixmap(GTK_CLIST(clist), vline, 0, docpix, docmask);
		vline++;
	}
	g_print("rbase2:%s\n",rbase);
	return vline;
}

void
test_encrypted_archive(gchar *arc_name)
{
	GtkWidget *archive_passwd;
  GtkWidget *dialog_vbox26;
  GtkWidget *vbox30;
  GtkWidget *label54;
  GtkWidget *hbox22;
  GtkWidget *label55;
  GtkWidget *entry5;
  GtkWidget *checkbutton95;
  GtkWidget *dialog_action_area26;
  GtkWidget *button168;
  GtkWidget *button170;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  archive_passwd = gnome_dialog_new (_("Archive password"), NULL);
  gtk_widget_set_name (archive_passwd, "archive_passwd");
  gtk_object_set_data (GTK_OBJECT (archive_passwd), "archive_passwd", archive_passwd);
  GTK_WINDOW (archive_passwd)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_policy (GTK_WINDOW (archive_passwd), FALSE, FALSE, FALSE);
  gnome_dialog_set_close (GNOME_DIALOG (archive_passwd), TRUE);
  gnome_dialog_close_hides (GNOME_DIALOG (archive_passwd), TRUE);

  dialog_vbox26 = GNOME_DIALOG (archive_passwd)->vbox;
  gtk_widget_set_name (dialog_vbox26, "dialog_vbox26");
  gtk_object_set_data (GTK_OBJECT (archive_passwd), "dialog_vbox26", dialog_vbox26);
  gtk_widget_show (dialog_vbox26);

  vbox30 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox30, "vbox30");
  gtk_widget_ref (vbox30);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "vbox30", vbox30,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox30);
  gtk_box_pack_start (GTK_BOX (dialog_vbox26), vbox30, TRUE, TRUE, 0);

  label54 = gtk_label_new (_("Archive locked with password !"));
  gtk_widget_set_name (label54, "label54");
  gtk_widget_ref (label54);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "label54", label54,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label54);
  gtk_box_pack_start (GTK_BOX (vbox30), label54, FALSE, FALSE, 0);
  gtk_widget_set_usize (label54, 248, 48);

  hbox22 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox22, "hbox22");
  gtk_widget_ref (hbox22);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "hbox22", hbox22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox22);
  gtk_box_pack_start (GTK_BOX (vbox30), hbox22, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox22), 10);

  label55 = gtk_label_new (_("Password:"));
  gtk_widget_set_name (label55, "label55");
  gtk_widget_ref (label55);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "label55", label55,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label55);
  gtk_box_pack_start (GTK_BOX (hbox22), label55, FALSE, FALSE, 0);
  gtk_widget_set_usize (label55, 88, 24);

  entry5 = gtk_entry_new ();
  gtk_widget_set_name (entry5, "entry5");
  gtk_widget_ref (entry5);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "entry5", entry5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry5);
  gtk_box_pack_start (GTK_BOX (hbox22), entry5, TRUE, TRUE, 0);
  gtk_widget_set_usize (entry5, 208, 22);
  gtk_widget_set_events (entry5, GDK_KEY_PRESS_MASK);
  gtk_widget_add_accelerator (entry5, "hide", accel_group,
                              GDK_Return, 0,
                              GTK_ACCEL_VISIBLE);
  gtk_entry_set_visibility (GTK_ENTRY (entry5), FALSE);

  checkbutton95 = gtk_check_button_new_with_label (_("Remember shit"));
  gtk_widget_set_name (checkbutton95, "checkbutton95");
  gtk_widget_ref (checkbutton95);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "checkbutton95", checkbutton95,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton95);
  gtk_box_pack_start (GTK_BOX (vbox30), checkbutton95, FALSE, FALSE, 0);

  dialog_action_area26 = GNOME_DIALOG (archive_passwd)->action_area;
  gtk_widget_set_name (dialog_action_area26, "dialog_action_area26");
  gtk_object_set_data (GTK_OBJECT (archive_passwd), "dialog_action_area26", dialog_action_area26);
  gtk_widget_show (dialog_action_area26);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area26), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area26), 8);

  gnome_dialog_append_button (GNOME_DIALOG (archive_passwd), GNOME_STOCK_BUTTON_OK);
  button168 = GTK_WIDGET (g_list_last (GNOME_DIALOG (archive_passwd)->buttons)->data);
  gtk_widget_set_name (button168, "button168");
  gtk_widget_ref (button168);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "button168", button168,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button168);

  gnome_dialog_append_button (GNOME_DIALOG (archive_passwd), GNOME_STOCK_BUTTON_CANCEL);
  button170 = GTK_WIDGET (g_list_last (GNOME_DIALOG (archive_passwd)->buttons)->data);
  gtk_widget_set_name (button170, "button170");
  gtk_widget_ref (button170);
  gtk_object_set_data_full (GTK_OBJECT (archive_passwd), "button170", button170,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button170);

  //gtk_signal_connect (GTK_OBJECT (entry5), "changed",
                      //GTK_SIGNAL_FUNC (on_entry5_changed),
                      //NULL);
	gtk_signal_connect (GTK_OBJECT (button168), "clicked",
                      GTK_SIGNAL_FUNC (on_button168_clicked),
                      NULL);
	gtk_signal_connect (GTK_OBJECT (button170), "clicked",
                      GTK_SIGNAL_FUNC (on_button170_clicked),
                      NULL);

  //gtk_widget_grab_focus (entry5);
  //gtk_window_add_accel_group (GTK_WINDOW (archive_passwd), accel_group);
	gtk_widget_show(archive_passwd);
}

void
archive_cd(GtkWidget *widget, gchar *path)
{
	if (!cmpare(gnochivemain->active_page->ad->type, "tgz"))
		gzip_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "tbz2"))
		bzip_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "zip"))
		zip_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "lha"))
		lha_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "RAR"))
		rar_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "tar"))
		tar_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "rpm"))
		rpm_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "comp"))
		Z_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "zoo"))
		zoo_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "arj"))
		arj_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "ace"))
		ace_cd(widget, path);
	if (!cmpare(gnochivemain->active_page->ad->type, "cpio"))
		cpio_cd(widget, path);
}

gint
gnochive_add_archive_files	(archivedesc *ad, gchar *file)
{
	gchar *pcmmd;
	gchar *myfile, *rtemp;
	gchar *dfile, *arc_path;
	gboolean *exec_status;
	gchar *name, *path;
	pid_t pid = 0;
	int status;
	gboolean error;

#ifdef _GLIB_2_0_
		name = g_path_get_basename(file);
		path = g_path_get_dirname(file);
#else	
		name = g_strdup(g_basename(file));
		path = g_strdup(g_dirname(file));
#endif
	
	if (!cmpare(ad->type, "RAR"))
	{
		rtemp = g_strconcat("@", file, NULL);
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.rar_path,
				"rar", "a", "-r-", "-inul", ad->full_name, rtemp, NULL))
		g_free(rtemp);
		exec_status = FALSE;
	}
	if (!cmpare(ad->type, "zip"))
	{
		if (!gnochive_exec_arch("/tmp/zpcrt", "zpcrt",
						gnochivemain->gc_cfg->archs.zip_path,
						ad->full_name, "-ur", "-xi", file, NULL))
		exec_status = FALSE;
	}
	if (!cmpare(ad->type, "lha"))
	{
		if (!gnochive_exec_arch("/tmp/zpps", "zpps", "nice",
									gnochivemain->gc_cfg->archs.lha_path,
									"uq", ad->full_name, file, NULL))
									exec_status = FALSE;
	}
	if (!cmpare(ad->type, "zoo"))
	{
		chmod(file, S_IRUSR | S_IWUSR | S_IXUSR);
		gnochive_exec_arch(file, name, ad->full_name, NULL);
	}
	pcmmd = g_strconcat("--files-from=", file, NULL);
	myfile = gnochive_remove_ext(ad);
	
#ifdef _GLIB_2_0_
		arc_path = g_path_get_dirname(ad->full_name);
#else	
		arc_path = g_strdup(g_dirname(ad->full_name));
#endif
	
	dfile = g_strconcat(arc_path, G_DIR_SEPARATOR_S, myfile, ".tar", NULL);
	if (!cmpare(ad->type, "tgz"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path,
			"gzip", "-d", "--force", ad->full_name, NULL))
			exec_status = FALSE;
	}
	if (!cmpare(ad->type, "tbz2"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path,
			"bzip2", "-d", "--force", ad->full_name, NULL))
			exec_status = FALSE;
	}
	if (!cmpare(ad->type, "comp"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path,
			"compress", "-d", "-f", ad->full_name, NULL))
			exec_status = FALSE;
	}
	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.tar_path,
		"tar", "-uf", dfile, pcmmd, NULL))
		exec_status = FALSE;
	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	/* FIXME! compression level is lost here */
	if (!cmpare(ad->type, "tgz"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.gzip_path, "gzip",
			"-f", dfile, NULL))
			exec_status = FALSE;
	}
	if (!cmpare(ad->type, "tbz2"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.bz2_path, "bzip2",
			"-f", dfile, NULL))
		exec_status = FALSE;
	}
	if (!cmpare(ad->type, "comp"))
	{
		if (!gnochive_exec_arch(gnochivemain->gc_cfg->archs.Z_path, "compress",
				"-f", "-r", dfile, NULL))
		exec_status = FALSE;
	}
	while (!waitpid(pid, &status, WNOHANG))
	{
		error = (WEXITSTATUS(status)<0);
		while (gtk_events_pending())
			gtk_main_iteration();
	}
	g_free(dfile);
	g_free(arc_path);
	g_free(myfile);
	g_free(pcmmd);
	g_free(name);
	g_free(path);
	refresh_arch(gnochivemain->active_page->ad);
	return 0;
}

void
gnochive_file_open_internal(gchar *file)
{
	GtkWidget *vd;
	GtkWidget *text;
	FILE *fl;
	guint flsize;
	gchar *content = NULL;
	gchar *title;
	
	if (is_readable(file))
	{
		fl = fopen(file, "r+");
		if (fl)
		{
			fseek(fl, 0L, SEEK_END);
			flsize = ftell(fl);
			content = g_malloc0(flsize);
			fseek(fl, 0L, SEEK_SET);
			fread(content, 1, flsize-1, fl);
			fclose(fl);
			vd = create_view_dialog();
			gtk_widget_show(vd);
			title = g_strconcat (gnochivemain->active_page->ad->full_name, " - ", file, NULL);
			gtk_window_set_title(GTK_WINDOW(vd), title);
			g_free(title);
			text = lookup_widget(GTK_WIDGET(vd), "text5");
			gtk_text_freeze(GTK_TEXT(text));
			gtk_text_insert(GTK_TEXT(text), NULL,
				NULL, NULL, content, -1);
			gtk_text_thaw(GTK_TEXT(text));
		}
	}
	g_free(content);
}

void
show_props (archivedesc *ad)
{
	GtkWidget *dialog4;
	GtkWidget *lname,*ltype,*lmet,*lsize;
	gchar atype[0x20];
	gchar ameth[0x20];
	gchar buff[20];
	guint asize;
	gint meth;
	FILE *tmp;
	gchar *lhameth[] = {"Unknown", "LZ Huffman 0",
			"LZ Huffman 1", "LZ Huffman 2",
		    "LZ Huffman 3", "LZ Huffman 4", "LZ Huffman 5",
		    "LZ Huffman 6", "L Arc", "L Arc 5", "L Arc 4","LZH Dirs"};
	
	dialog4 = create_archive_info();
	gtk_widget_show(dialog4);
	
	lname = lookup_widget(GTK_WIDGET(dialog4), "label229");
	ltype = lookup_widget(GTK_WIDGET(dialog4), "label232");
	lmet = lookup_widget(GTK_WIDGET(dialog4), "label233");
	lsize = lookup_widget(GTK_WIDGET(dialog4), "label248");
	gtk_label_set_text(GTK_LABEL(lname), ad->name);
	strcpy(atype,ad->type);
	strcpy(ameth, lhameth[0]);
	if (cmpare(ad->type, "tgz")==0)
		strcpy(atype, _("TAR - gzipped"));
	if (cmpare(ad->type, "tbz2")==0)
		strcpy(atype, _("TAR - bzip2"));
	if (cmpare(ad->type, "tar")==0)
		strcpy(atype, _("plain TAR"));
	if (cmpare(ad->type, "comp")==0)
		strcpy(atype, _("TAR - compressed(Z)"));
	if (cmpare(ad->type, "lha")==0)
	{
		strcpy(atype, _("LHA archive"));
		meth = detect_lha(ad->full_name);
		strcpy(ameth, lhameth[meth]);
	}
	gtk_label_set_text(GTK_LABEL(ltype), atype);
	gtk_label_set_text(GTK_LABEL(lmet), ameth);
	tmp = fopen(ad->full_name, "r");
	if (tmp)
	{
		fseek(tmp, 0L, SEEK_END);
		asize = ftell(tmp);
		fclose(tmp);
		g_snprintf(buff,20,_("%d"), asize);
		gtk_label_set_text(GTK_LABEL(lsize), buff);
	}
}

void
gnochive_arch_file_view(void)
{
	gchar *text=NULL;
	gchar *rtxt;
	guint i,j=0;
	gchar *tmpfile,*gc_type;
	
	for (i=0;i<gnochivemain->active_page->current_lines;i++) {
		if (gnochivemain->active_page->selected_rows[i]==1 && i!=0 && i!=1)
		{
			gtk_clist_get_text(GTK_CLIST(gnochivemain->active_page->clist), i, 1, &text);
			gtk_clist_unselect_row(GTK_CLIST(gnochivemain->active_page->clist), i, 0);
			rtxt = g_strconcat(gnochivemain->active_page->ad->root, text, NULL);
			tmpfile = extract_file_from_archive(gnochivemain->active_page->ad->full_name, rtxt);
			g_free(rtxt);
			gc_type = gnochive_get_archive_type(tmpfile);
			if (!gc_type) 
			{	
				if (!gnochivemain->gc_cfg->internal)
					gnochive_file_open_mime(tmpfile);
				else
					gnochive_file_open_internal(tmpfile);
			}
			j++;
		}
	}
}
