/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Lucian Langa
 *
 *	gnochive configure engine
 *  	Used for setting archivers path, autodetect archivers,
 *		user options. At some point this file will be replaced
 *		with GCONF.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gnome.h>
#include <unistd.h>
#include <pwd.h>
#include <wait.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "gc_setup.h"
#include "conf.h"
#include "misc.h"
#include "gnochive.h"

#define CFG_FILE ".gnochive"

gchar crt_script_file[0x100]="#!/bin/sh\n\n$1 a $2 $3 $4 $5 $6 $7 $8";
gchar del_script_file[0x100]="#!/bin/sh\n\n$1 d $2 $3 $4 $5";
gchar ext_script_file[0x100]="#!/bin/sh\n\n$1 x -o+ -y $2 $3 $4";
gchar gzip_view_script_file[0x100]="#!/bin/sh\n\n$1 $2 $3 \"$4\" |grep -e \".tar$\" > $5";
gchar tgz_extract_script_file[0x100]="#!/bin/sh\n\n$1 $2 -xf $3 --overwrite --files-from=$4";
gchar zip_op_file[0x100]="#!/bin/sh\n\n$1 $2 $3 $4 -@<$5 >/dev/null";
gchar zip_view[0x100]="#!/bin/sh\n\n$1 \"$2\" \"$3\" >$4";
gchar zip_parse[0x100]="#!/bin/sh\n\n$1 $2 \"$3\" \"$4\" <$5";
gchar stat_gz_compresion[0x100]="#!/bin/sh\n\n$1 -c $2 >/tmp/gzstat ; $1 -l /tmp/gzstat >/tmp/gzstat1 ; rm /tmp/gzstat";
gchar stat_bz2_compresion[0x100]="#!/bin/sh\n\n$1 $2 $3>/tmp/bzstat ; rm /tmp/bzstat";
gchar stat_rar_compresion[0x100]="#!/bin/sh\n\n$1 a $2 $3 >/dev/null; $1 l $2 >/tmp/strar ; rm $2";
gchar remove_filter[0x100]="#!/bin/sh\n\n$1 $2 $3 \"$4\" \"$5\" >$6";
gchar gen_test_file[0x100]="#!/bin/sh\n\n$1 $2 $3 > $4 2>&1";

gnochive_config*
config_gnochive			(void)
{
	FILE *cfgfile;
	guint cfgsize;
	gchar *cfgbuff;
	guint i,j;
	gchar cver[0x100];
	gchar *cfver;
	gnochive_config *gc=NULL;
	gchar *local_cfg;
	
	local_cfg = get_local_cfile();
	// search for local config file (.gnochive)
	cfgfile = fopen(local_cfg, "r");
	if (!cfgfile)
	{
		gc = create_cfile(local_cfg);
		if (!gc)
		{
			gc = NULL;
			g_error (_("Cannot create gnochive config dir...exiting\n"));
		}
	}
	else {
		fseek(cfgfile, 0L, SEEK_END);
		cfgsize = ftell(cfgfile);
		if (!cfgsize) {	// .gnochive cannot have 0 size
				g_print(_("Critical error! Config file corrupted!"));
				fclose(cfgfile);
				gc = create_cfile(local_cfg);
		}
		else {
			cfgbuff = malloc(cfgsize);
			fseek(cfgfile, 0L, SEEK_SET);
			fread(cfgbuff, cfgsize, 1, cfgfile);
			fclose(cfgfile);
			i=0;j=0;
			while (cfgbuff[j]!='\n') {
					cver[i]=cfgbuff[j];
					i++;j++;
					}
			j++;i = 0;
			cfver = cver;
			//if we do not have a vaild cfile create a new one by the defaults
			if (cmpare(cfver, GNOCHIVE_VER)!=0) {
						free(cfgbuff);
						gc = create_cfile(local_cfg);
						}
			else gc = read_cfile(cfgbuff);
    		}
	}
	if (gc->bz2flag) gc->tbz2parm = g_strdup("-j");
	else gc->tbz2parm = g_strdup("-I");
	gc->file_name = local_cfg;
return gc;
}

/* reads one archiver line & alocates mem for it */
gchar*
read_arc_line		(gnochive_config *var,gchar *cfgbuff)
{
	gchar tmp1[0x100]="\0";
	guint i=0;guint j=0;
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.rar_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.gzip_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.tar_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.zip_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.unzip_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.bz2_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.Z_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
		tmp1[j]=cfgbuff[i];
		i++;j++;
	}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.unarj_path=g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
		tmp1[j]=cfgbuff[i];
		i++;j++;
	}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.unace_path=g_strdup(tmp1);
		while (cfgbuff[i]!='\n') {
			    tmp1[j]=cfgbuff[i];
			    i++;j++;
			}
		tmp1[j]='\0';
		i++;j=0;
	var->archs.lha_path = g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
	    tmp1[j]=cfgbuff[i];
	    i++;j++;
		}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.zoo_path = g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
	    tmp1[j]=cfgbuff[i];
	    i++;j++;
		}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.rpm_path = g_strdup(tmp1);
	while (cfgbuff[i]!='\n') {
	    tmp1[j]=cfgbuff[i];
	    i++;j++;
		}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.cpio_path = g_strdup(tmp1);
		while (cfgbuff[i]!='\n') {
	    tmp1[j]=cfgbuff[i];
	    i++;j++;
		}
	tmp1[j]='\0';
	i++;j=0;
	var->archs.rpm2cpio_path = g_strdup(tmp1);
	return cfgbuff+i;
}

gchar*
parse_option		(gnochive_config *var,gchar *cfgbuff)
{
gchar opt[10]="\0";
gint i=0;guint j=0;
while(cfgbuff[i]!='=') i++;
i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
if(strcmp(opt,"yes")==0) var->tit=1;
else var->tit=0;
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
if(strcmp(opt,"yes")==0) var->res_syms=1;
else var->res_syms=0;
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
if(strcmp(opt, "yes")==0) var->popn=1;
else var->popn=0;
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!=',')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
var->gbstat=atoi(opt);
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
var->tstyle=atoi(opt);
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
var->tbstyle=atoi(opt);
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
if(strcmp(opt, "single")==0)
	var->stype = 1;
else var->stype = 0;
while(cfgbuff[i]!='=') i++;i++;
while(cfgbuff[i]!='\n')
{
	opt[j]=cfgbuff[i];
	i++;j++;
}
opt[j]='\0';
i++;j=0;
if(strcmp(opt, "yes")==0)
	var->internal = 1;
else var->internal = 0;
return cfgbuff+i;
}

/* reads gnochive configure file
 * returns: gc_config
 */

gnochive_config*
read_cfile				(gchar *cfgbuff)
{
	gchar tmp[0x10];
	gchar wdir[0x100];
	guint i=0;guint j=0;
	gchar *temp=NULL,*temp1=NULL;
	gnochive_config *gc_config;
	
	gc_config=(gnochive_config *) g_malloc(sizeof(gnochive_config));
	if (gc_config)
	{
		while (cfgbuff[i]!='\n') {i++;}
		i++;
		while (cfgbuff[i]!='\n') {i++;}
		i++;
		temp = read_arc_line(gc_config, cfgbuff+i);
		
		/*interface options*/
		temp1=parse_option(gc_config,temp);
		i=0;
		while(temp1[i]!='\n') i++;
		while(temp1[i]!='\n') i++;
		while(temp1[i]!='=') i++;
		i++;j=0;
		while(temp1[i]!='\n')
		{
 			wdir[j]=temp1[i];
 			i++;j++;
		}
		wdir[j]='\0';
		i++;
		gc_config->workdir=g_strdup(wdir);
		while(temp1[i]!='\n') i++;
		i++;
		while(temp1[i]!='=') i++;
		i++;j=0;
		while(temp1[i]!='\n')
		{
			tmp[j]=temp1[i];
			i++;j++;
		}
		tmp[j]='\0';
		i++;j=0;
		if(strcmp(tmp, "yes")==0)
			gc_config->bz2flag = 1;
		else
			gc_config->bz2flag = 0;
		while(temp1[i]!='\n')
		{
			tmp[j]=temp1[i];
			i++;j++;
		}
		tmp[j]='\0';
		i++;j=0;
		gc_config->rpm_handle = (gboolean) tmp;
		while(temp1[i]!='=') i++;
		i++;
		while(temp1[i]!='\n')
		{
 			tmp[j]=temp1[i];
 			i++;j++;
		}
		tmp[j]='\0';
		i++;
		gc_config->cclist_val=atoi(tmp);
		gc_config->defaults=0;
		free(cfgbuff);
	}
	else {
		gc_config = NULL;
		g_error (_("Cannot create gnochive...exiting\n"));
	}
	return gc_config;
}


gchar*
get_local_cfile(void)
{
struct passwd *pw;
pw=getpwuid(getuid());
return g_strconcat(pw->pw_dir,G_DIR_SEPARATOR_S,CFG_FILE,NULL);
}

int
find_arc			(gchar *arc)
{
int arc_ok=0;
FILE *myfile;
char buff[0x100];
memset(buff,0,sizeof(buff));
g_snprintf(buff,0x100,"/usr/bin/%s",arc);
if ((myfile=fopen(buff,"r"))!=NULL)
 {
    fclose(myfile);
    return 2;
 }
g_snprintf(buff,0x100,"/usr/local/bin/%s",arc);
if ((myfile=fopen(buff,"r"))!=NULL)
 {
    fclose(myfile);
    return 3;
 }
g_snprintf(buff,0x100,"/bin/%s",arc);
if ((myfile=fopen(buff,"r"))!=NULL)
 {
    fclose(myfile);
    return 4;
 }
return arc_ok;
}

gchar*
detect_arc_def_path			(gchar *arc_name,FILE *cfgfile)
{
int stat;
gchar tmpp[0x50];
gchar *mypath;
stat=find_arc(arc_name);
switch(stat) {
		case 2:	g_snprintf(tmpp,0x50,"/usr/bin/%s",arc_name);break;
		case 3:	g_snprintf(tmpp,0x50,"/usr/local/bin/%s",arc_name);break;
		case 4:	g_snprintf(tmpp,0x50,"/bin/%s",arc_name);break;
	    default   : g_snprintf(tmpp,0x50,"/usr/local/bin/%s",arc_name);
	    }
mypath=write_parse_arc_path(tmpp,cfgfile);
return mypath;
}

/* writes arch path to conf file and parses it's walue through a malloced var */
gchar*
write_parse_arc_path			(gchar *path,FILE *cfgfile)
{
	fwrite(path,strlen(path),1,cfgfile);
	fprintf(cfgfile,"\n");
	return g_strdup(path);
}

/* 
 * Creates a new configure file based on the defaults
 */
gnochive_config*
create_cfile				(gchar *cfg_file)
{
	struct passwd *pw;
	gchar *path;
	FILE *cfgfile;
	gnochive_config *gc_config;
	
	gc_config=(gnochive_config *) g_malloc(sizeof(gnochive_config));
	if (gc_config)
	{
		//me/gnome_app_flash(GNOME_APP(mainwindow),_("Configuration file not found , parsing defaults"));
		cfgfile=fopen(cfg_file,"w+");
		if (cfgfile)
		{
			fprintf(cfgfile,"[%s]\n",GNOCHIVE_VER);
			fprintf(cfgfile,"#archivers path\n");
			gc_config->archs.rar_path = detect_arc_def_path("rar", cfgfile);
			gc_config->archs.gzip_path = detect_arc_def_path("gzip", cfgfile);
			gc_config->archs.tar_path = detect_arc_def_path("tar", cfgfile);
			gc_config->archs.zip_path = detect_arc_def_path("zip", cfgfile);
			gc_config->archs.unzip_path = detect_arc_def_path("unzip", cfgfile);
			gc_config->archs.bz2_path = detect_arc_def_path("bzip2", cfgfile);
			gc_config->archs.Z_path = detect_arc_def_path("compress", cfgfile);
			gc_config->archs.unarj_path = detect_arc_def_path("unarj", cfgfile);
			gc_config->archs.unace_path = detect_arc_def_path("unace", cfgfile);
			gc_config->archs.lha_path = detect_arc_def_path("lha", cfgfile);
			gc_config->archs.zoo_path = detect_arc_def_path("zoo", cfgfile);
			gc_config->archs.rpm_path = detect_arc_def_path("rpm", cfgfile);
			gc_config->archs.cpio_path = detect_arc_def_path("cpio", cfgfile);
			gc_config->archs.rpm2cpio_path = detect_arc_def_path("rpm2cpio", cfgfile);
			fprintf(cfgfile,"#interface options\n");
			fprintf(cfgfile,"show-titles=yes\n");
			gc_config->tit=1;
			fprintf(cfgfile,"follow-syms=no\n");
			gc_config->res_syms=0;
			fprintf(cfgfile,"post-open=no\n");
			gc_config->popn=0;
			fprintf(cfgfile,"toolbar-style=0,0\n");
			gc_config->gbstat = 0;
			fprintf(cfgfile,"tabs-style=2\n");
			fprintf(cfgfile,"sel-type=single\n");
			fprintf(cfgfile,"internal-viewer=no\n");
			gc_config->internal = 0;
			fprintf(cfgfile,"#working directory (used for overwriting the path parsed by nautilus)\n");
			fprintf(cfgfile,"#by default we use our $HOME dir\n");
			pw=getpwuid(getuid());
			path=g_strdup(pw->pw_dir);
			fprintf(cfgfile,"own-working-dir=%s\n",path);
			fprintf(cfgfile,"#compat flags\n");
			fprintf(cfgfile,"bz2_j_flag=no\n");
			fprintf(cfgfile, "rpm_handle=1\n");
			gc_config->bz2flag=0;
			fprintf(cfgfile,"clist=%d\n",127);
			gc_config->cclist_val=127;
			gc_config->defaults=1;
			gc_config->tbstyle=2;
			fclose(cfgfile);
			g_free(path);
			return gc_config;
		}
	}
return NULL;
}

gboolean
create_script				(gchar *sfile,gchar *dtafile)
{
	FILE *scrfile=NULL;
	gchar *res;
	gint imask;
	scrfile=fopen(sfile,"w+");
	if (scrfile)
	{
		fwrite(dtafile,strlen(dtafile),1,scrfile);
		fclose(scrfile);
		imask=umask(0666);
		chmod(sfile,0777 & ~imask);
		umask(imask);
	}
	else {
			res=g_strdup_printf(_("\nError: couldn't create scripts: %s files! Wrong /tmp permisions maybe?\n"),sfile);
			g_print(res);
			g_free(res);
			return FALSE;
		}
	return TRUE;
}

void
write_scripts		(void)
{
create_script("/tmp/grarcrt",crt_script_file);
create_script("/tmp/grardel",del_script_file);
create_script("/tmp/grarextr",ext_script_file);
create_script("/tmp/gzipview",gzip_view_script_file);
create_script("/tmp/tgzextr",tgz_extract_script_file);
create_script("/tmp/zpcrt",zip_op_file);
create_script("/tmp/zpvw",zip_view);
create_script("/tmp/zpps", zip_parse);
create_script("/tmp/statgz",stat_gz_compresion);
create_script("/tmp/statbz",stat_bz2_compresion);
create_script("/tmp/statrar",stat_rar_compresion);
create_script("/tmp/remflt",remove_filter);
create_script("/tmp/gtest",gen_test_file);
}

void
remove_scripts		(void)
{
	static char *scripts[] = {"/tmp/grarcrt", "/tmp/grardel", "/tmp/grarextr", 
							"/tmp/grarview", "/tmp/gzipview", "/tmp/tgzview",
							"/tmp/tgzextr", "/tmp/zpcrt", "/tmp/zpvw", "/tmp/zpps",
							"/tmp/statgz","/tmp/statbz","/tmp/statrar","/tmp/remflt","/tmp/gtest"};
	guint i;

	for (i=0;i<=14;i++)
		remove(scripts[i]);
}
