/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Langa Lucian
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef USE_GNOME
#include <gnome.h>
#	ifdef ENABLE_BONOBO
#	include <bonobo.h>
#	endif
#endif

#include <libgnomevfs/gnome-vfs-init.h>
#include "interface.h"
#include "support.h"

#include "gnochive.h"
#include "conf.h"
#include "misc.h"
#include "mime.h"


int i;
gchar arg_fn;
static int arg_extr = 0;		//if user passed gnochive extract command
gchar *arga[];					//files

static void get_command_line(int argc, char *argv[]);
static guint gc_init_from_main_loop(gpointer data);
void usage(void);

#include "gnochive_session.h"

#ifdef USE_GNOME
static poptContext pctx;

static struct poptOption opt[] = {
 {
  "extract-archive",
  'x',
  0,
  &arg_extr,
  0,
  N_("Extract archive and exit"),
  NULL
 },
 {
  NULL,
  '\0',
  0,
  NULL,
  0,
  NULL,
  NULL
 }
};
#endif

int
main (int argc, char *argv[])
{

	GnomeClient *client;
	
#ifdef ENABLE_BONOBO
CORBA_ORB	orb;
#endif

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

	gnome_vfs_init();
	
  putenv(BASE_DIR);
  
#ifdef USE_GNOME
#	ifdef ENABLE_BONOBO
	if bonobo_init(orb,NULL,NULL) == FALSE)
	    g_error("Bonobo failed");
#	else
	 gnome_init_with_popt_table ("gnochive", VERSION, argc, argv,
			       opt,0,&pctx);
#	endif
#else
  gnome_init("gnochive", VERSION, argc, argv);
#endif
  	get_command_line(argc,argv);
	client = gnome_master_client();
	gtk_signal_connect (GTK_OBJECT(client),"save_yourself",
		GTK_SIGNAL_FUNC (gnochive_save_yourself), argv[0]);
	gtk_signal_connect (GTK_OBJECT(client),"die",
		GTK_SIGNAL_FUNC (gnochive_die), NULL);
  	gtk_idle_add((GtkFunction)gc_init_from_main_loop,NULL);

#ifdef ENABLE_BONOBO
    bonobo_main();
#else  
    gtk_main ();
#endif
  return 0;
}

#ifdef USE_GNOME
static void
get_command_line(int argc, char *argv[])
{
    const gchar **args;
    gint i;
    
    args= poptGetArgs (pctx);
	
    for (i=0;args && args[i];i++)
      {
		  arga[i] = g_strdup((gchar *)args[i]);
		  arg_fn=i;
       //if (arg_fn==NULL)
        //arg_fn=(gchar *)args[i];
       //else
    	    //usage();
      }
      
    poptFreeContext(pctx);
}

#else
static void
get_command_line(int argc, char *argv[])
{
gint i;

    for (i=1;i<argc;i++)
    {
    if (!argv[i]) continue;
    if (!strcmp (argv[i],"-x") || !strcmp (argv[i],"--extract"))
	arg_extr=1;
    else if (arg_fn==NULL)
        arg_fn=(gchar *)argv[i];
    else
	usage();
    }
}
#endif

static guint
gc_init_from_main_loop(gpointer data)
{
	GtkWidget *gnochive_mainwindow;

	gnochive_mainwindow = create_gnochive ();
	gtk_widget_show (gnochive_mainwindow);
	if (arga[i])
		process_archive(gnochive_mainwindow,arga);
  	else normal_startup(gnochive_mainwindow);
	return FALSE;
}

void
usage (void)
{
	fprintf(stderr,"Usage: gnochive [-x|--extract] [<files...>] [<--files-from=filelist>] \n");
	exit(0);
}
