/*  GUI frontend for common compresion programs
 *  Copyright 2000-2001  Langa Lucian
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include <libgnomevfs/gnome-vfs-application-registry.h>
#include <sys/wait.h>
#include "support.h"
#include "gnochive.h"
#include "conf.h"

#define GNOME_VFS_MIME_TYPE_UNKNOWN "application/octet-stream"

extern gchivemain *gnochivemain;

extern GdkPixmap 	*dirpix,*docpix,*xpmpix,*makpix;
extern GdkPixmap	*rpmpix,*comppix,*dotpix,*lockpix;
extern GdkPixmap	*hdrpix,*sympix,*rdpix,*rpm1pix;

extern GdkBitmap 	*dirmask,*docmask,*xpmmask,*makmask;
extern GdkBitmap	*rpmmask,*compmask,*dotmask,*lockmask;
extern GdkBitmap *hdrmask,*symmask,*rdmask,*rpm1mask;

/* opens a file via mime type detection */

void
gnochive_file_open_mime(char *mime_file)
{
	pid_t pid;
	GnomeVFSMimeApplication *mygv;
	const char *mymime;
	
	mymime = gnome_vfs_get_file_mime_type(mime_file, NULL, FALSE);
	g_print("Detected mime: %s\n",mymime);
	mygv = gnome_vfs_mime_get_default_application(mymime);
	if (mygv)
	{
		if ((pid = fork()) == 0)
		{
			execlp(mygv->command, mygv->command, mime_file, NULL);
			g_error (_("Cannot launch %s"), mygv->command);
		}
		waitpid (pid, NULL, 0);
		gnome_vfs_mime_application_free(mygv);
	}
	else 
	{
		if ((pid = fork()) == 0)
		{
			execl("/usr/bin/gedit", "gedit", mime_file, NULL);
			g_error (_("Cannot launch %s"), mygv->command);
		}
		waitpid (pid, NULL, 0);
	}
}

/* Wrapper for gnochive_get_archive_type */

gchar*
gnochive_vfs_get_archive_type(char *file_name)
{
	gchar *arch_mime[] = {"zip", "gzip", "bzip2", "zoo", "lha", "rar", "arj", "tar", "rpm"};
	gchar *arch_vfs[] = {"application/zip", "application/x-gzip", "application/x-bzip",
						"application/x-zoo", "application/x-lha", "application/x-rar",
						"application/x-arj", "application/x-tar", "application/x-rpm"};
	guint i;
	const char *mymime = gnome_vfs_get_file_mime_type(file_name, NULL, FALSE);
	for (i=0;i<=8;i++)
	{
		if (!cmpare((gchar *)mymime, arch_vfs[i]))
			return g_strdup(arch_mime[i]);
	}
	return NULL;
}

void
gnochive_read_pwd_files(gchar *pwd)
{

	int line=0;
	gchar *clist_line[3][8];
	gchar *cur_dir;
	gchar buff[0x100];
	struct stat statbuf;
	gchar attr[11];
	gint pixset=1;
	gchar *full=NULL;
	GnomeVFSResult result;
	GList *list, *node;
	GnomeVFSFileInfo *info;
	gchar *uri_pwd;
	gchar mytime[30];
	
	if (is_readable(pwd))
	{
		gnochivemain->active_page->current_lines=0;
		cur_dir = g_get_current_dir();
		gtk_entry_set_text(GTK_ENTRY(gnochivemain->location_entry), cur_dir);
		memset(buff,0,strlen(buff));
		full = g_strconcat(GNOCHIVE_VER,_(":"),cur_dir,NULL);
		gtk_window_set_title(GTK_WINDOW(gnochivemain->mainwin),full);
		g_free(full);
		gtk_clist_freeze(GTK_CLIST(gnochivemain->active_page->clist));
		gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
		memset(clist_line,0,sizeof(clist_line));
		
		uri_pwd = gnome_vfs_get_uri_from_local_path(pwd);
		result = gnome_vfs_directory_list_load(&list,
					uri_pwd, GNOME_VFS_FILE_INFO_DEFAULT | 
							 GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
					NULL);
		for (node = list; node != NULL ; node = node->next)
		{
			info = node->data;
			if ((info->name[0] == '.' && !info->name[1]) ||
				(info->name[0] == '.' && info->name[1] == '.'))
				continue;
			stat((char *)info->name, &statbuf); // do a normal stat for sorting sym from symdirectories
			/* FIXME symdirectories should not be shown as normal directories */
			xfer_mode(statbuf.st_mode,attr);
			if (attr[0]=='d') (gchar*)clist_line[0][2]="dir";
			else (gchar*)clist_line[0][2]=0;
			clist_line[0][1] = info->name;
			if (!gnochivemain->gc_cfg->res_syms && 
				!(attr[0]=='l' || attr[0]=='d'))
				lstat((char *)clist_line[0][1],&statbuf);
			else
				stat((char *)clist_line[0][1],&statbuf);
			g_snprintf(buff,50,"%-10ld",statbuf.st_size);
			(gchar*)clist_line[0][3]=(gchar *)buff;
			strftime(&mytime[0], 30, "%a, %b %e %Y %I:%M:%S %p", localtime(&statbuf.st_atime));
			(gchar*)clist_line[0][5]=mytime;
			xfer_mode(statbuf.st_mode,attr);
			(gchar*)clist_line[0][6]=(gchar*)attr;
			gnochivemain->active_page->current_lines++;
			if (attr[0]=='d') {
					gtk_clist_insert(GTK_CLIST(gnochivemain->active_page->clist), 0, clist_line[0]);
					gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist), 0, 0, dirpix, dirmask);
					}
				else {
						gtk_clist_insert(GTK_CLIST(gnochivemain->active_page->clist), 0, clist_line[0]);//append
						if (cmpare(clist_line[0][1],".xpm")==0) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,xpmpix,xpmmask);pixset=0;}
						if (cmpare(clist_line[0][1],".rpm")==0) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,rpm1pix,rpm1mask);pixset=0;}
						if (clist_line[0][6][0]=='l') {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,sympix,symmask);pixset=0;}
						if (pixset==1) {gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,docpix,docmask);pixset=1;}
						pixset=1;
					}
			line++;
			
		}
		gtk_clist_set_sort_column(GTK_CLIST(gnochivemain->active_page->clist),1);
		gtk_clist_set_sort_type(GTK_CLIST(gnochivemain->active_page->clist),GTK_SORT_ASCENDING);
		gtk_clist_set_auto_sort(GTK_CLIST(gnochivemain->active_page->clist),1);
		gtk_clist_set_sort_column(GTK_CLIST(gnochivemain->active_page->clist),2);
		gtk_clist_set_sort_type(GTK_CLIST(gnochivemain->active_page->clist),GTK_SORT_DESCENDING);
		gtk_clist_sort(GTK_CLIST(gnochivemain->active_page->clist));
		gtk_clist_set_auto_sort(GTK_CLIST(gnochivemain->active_page->clist),0);
		memset(clist_line,0,sizeof(clist_line));
		clist_line[0][1]="..";
		gtk_clist_prepend(GTK_CLIST(gnochivemain->active_page->clist),clist_line[0]);
		gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,dotpix,dotmask);
		clist_line[0][1]=".";
		gtk_clist_prepend(GTK_CLIST(gnochivemain->active_page->clist),clist_line[0]);
		gtk_clist_set_pixmap(GTK_CLIST(gnochivemain->active_page->clist),0,0,rdpix,rdmask);
		gtk_clist_thaw(GTK_CLIST(gnochivemain->active_page->clist));
		gtk_clist_set_column_width(GTK_CLIST(gnochivemain->active_page->clist), 5, 
				gtk_clist_optimal_column_width(GTK_CLIST(gnochivemain->active_page->clist), 5));
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar), 0);
		g_free(cur_dir);
		//files_selected=0;
		gnochivemain->active_page->current_lines+=2;
	}
}
