/*  GUI frontend for common compression programms
 *  Copyright (C) 2000-2001 Lucian Langa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _MISC_H_
#define _MISC_H_

#include <gtk/gtkfeatures.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkfixed.h>
#include <gtk/gtkobject.h>

#include "gc_bk_end.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Glade Filename Utility Functions.
 */

/* This turns a relative pathname into an absolute one based on the given
   base directory (which MUST be absolute).
   e.g. "/home/damon" + "../dave/test" -> "/home/dave/test"
   The returned path should be freed when no longer needed. */
gchar*	  make_absolute_path	(const gchar	    *dir,
				 const gchar	    *file);

/* This turns an absolute pathname into an relative one based on the given
   base directory. Both arguments must be absolute paths, and should be in
   canonical form, i.e. not containing '..', '.' or multiple '/'s together.
   The returned value may or may not end with a '/', depending on the
   arguments. The returned path should be freed when no longer needed. */
gchar*	  make_relative_path	(const gchar	    *base_dir,
				 const gchar	    *file);

/* This returns the parent directory of the given directory, which may or may
   not end in a G_DIR_SEPARATOR. The returned string should be freed. */
gchar*    get_parent_directory	(const gchar	    *dir);

/* Extracts the file name from a given path
   the path MUST be absolute */
gchar*	  extract_filename	(const gchar	*fullpath);

gchar*	  extract_extension	(const gchar	*file_name);

int	  f_type		(const gchar	*path);

int	sort_dirs		(gchar *line);
gboolean sort_type		(gchar *line, gchar *attr);
int	sort_subdirs		(gchar *parent,gchar *child);

int	process_line		(gchar *line);

gchar* extract_root_word	(gchar *line,int seps);

gchar* trim_root_word		(gchar *file1,gchar *file2);

gboolean
gnochive_check_file(gchar *file_name,guint mode);

gboolean is_dir (const gchar *file);
gboolean is_regular (const gchar *file);
gboolean is_readable (const gchar *file);
gboolean is_writable (const gchar *file);
gboolean gnochive_can_create(const gchar *file);
gboolean gnochive_istr_has_suffix (const char *haystack, const char *needle);
gint gnochive_exec_arch	(gchar *path, gchar *param, ...);

void open_xpm (char *filename, GtkWidget *parent, GdkPixmap **pixmap, GdkBitmap **mask, int quit_on_err);
void xfer_mode(mode_t my,gchar *mode);
gchar* gnochive_remove_ext (archivedesc *ad);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

