/*  gnochive - GUI frontend for common compresion programs  
 *  Copyright 2001-2002 Lucian Langa
 *
 *	rpm handling functions
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <fcntl.h>
#include <wait.h>

#include "archs.h"
#include "rpm.h"
#include "cpio.h"
#include "components.h"
#include "interface.h"
#include "support.h"
#include "gnochive.h"

#define TIMEOUT_GTK 5

extern gchivemain *gnochivemain;

extern GdkPixmap 	*dirpix,*docpix,*xpmpix,*makpix;
extern GdkPixmap	*rpmpix,*comppix,*dotpix,*lockpix;
extern GdkPixmap *hdrpix,*sympix,*rdpix,*rpm1pix;

extern GdkBitmap 	*dirmask,*docmask,*xpmmask,*makmask;
extern GdkBitmap	*rpmmask,*compmask,*dotmask,*lockmask;
extern GdkBitmap *hdrmask,*symmask,*rdmask,*rpm1mask;

extern gchar *arc_type;

extern GtkWidget *arcmenu1;

extern gchar *directory_read;
extern gchar *directory_up;

extern gfloat step;
extern float progr;
extern gchar *opd_archive_name;
extern int last_select_row;

static guint mytime(gpointer data);

void
rpm_dummy_info(gchar *rpm_file)
{
	FILE *tempfile;
	gchar *arcfile = NULL;
	gint arcsize;
	gchar *temp_name; 
	int tf;
	GtkWidget *rpm_info;
	GtkWidget *lbl1, *lbl2;
	GtkWidget *txt;
	gint textlen;
	gint i, j, q;
	gchar xfr[0x100];
	gchar *xfrwd[3] = {NULL, NULL, NULL};
	gchar *label[10][2] = 	{{"label314", "label320"},
							{"label315", "label321"},
							{"label316", "label322"},
							{"label317", "label323"},
							{"label318", "label324"},
							{"label319", "label325"},
							{"label326", "label326"},
							{"label327", "label327"},
							{"label328", "label328"}};
	gchar *rpm_tags[4] = {"Packager", "URL", "Summary", "Description"};
	
	temp_name = g_concat_dir_and_file(TMPDIR,"gchiveXXXXXX");
	tf = mkstemp(temp_name);
	close(tf);
	
	if (gnochive_exec_arch("/tmp/remflt", "remflt", 
			gnochivemain->gc_cfg->archs.rpm_path,
			"-q", "-p", "-i", rpm_file, temp_name, NULL))
	
	while (!waitpid(0, NULL, WNOHANG))
	{
		gtk_main_iteration();
	}
	while(gtk_events_pending())
		gtk_main_iteration();
	tempfile = fopen(temp_name, "rb");
	if (tempfile)
	{
		fseek(tempfile, 0L, SEEK_END);
		arcsize = ftell(tempfile);
		arcfile = g_malloc(arcsize);
		fseek(tempfile, 0L, SEEK_SET);
		fread(arcfile, 1, arcsize, tempfile);
		fclose(tempfile);
		rpm_info = create_rpm_info();
		gtk_widget_show(rpm_info);
		
		i = 0;
		j = 0;
		for (q=0;q<=5;q++)
		{
			lbl1 = lookup_widget(GTK_WIDGET(rpm_info), label[q][0]);
			lbl2 = lookup_widget(GTK_WIDGET(rpm_info), label[q][1]);
			while (arcfile[i] != ':') i++;
			i+=2;
			while (arcfile[i] != 0x20 || arcfile[i+1] != 0x20) {
						xfr[j] = arcfile[i];
						i++;j++;
						}
			xfr[j] = '\0';
			xfrwd[0] = g_strdup(xfr);
			i++;
			gtk_label_set_text(GTK_LABEL(lbl1), xfrwd[0]);
			while (arcfile[i] != ':') i++;
			i+=2;
			j = 0;
			while (arcfile[i] != 0x0A) {
						xfr[j] = arcfile[i];
						i++;j++;
						}
						xfr[j]='\0';
			xfrwd[1] = g_strdup(xfr);
			gtk_label_set_text(GTK_LABEL(lbl2), xfrwd[1]);
			i++;j=0;
			g_free(xfrwd[0]);
			g_free(xfrwd[1]);
		}
		for (q=6;q<=8;q++)
		{
			lbl1 = lookup_widget(GTK_WIDGET(rpm_info), label[q][0]);
			j = 0;
			while (arcfile[i] != 0x20)
			{
				xfr[j] = arcfile[i];
				i++;j++;
			}
			xfr[j] = '\0';
			xfrwd[0] = g_strdup(xfr);
			if (!cmpare(xfrwd[0], rpm_tags[q-6]))
			{
				while (arcfile[i] != ':') i++;
				i+=2;
				j = 0;
				while (arcfile[i] != 0x0A) {
								xfr[j] = arcfile[i];
								i++;j++;
								}
				xfr[j] = '\0';
				xfrwd[1] = g_strdup(xfr);
				i++;
				gtk_label_set_text(GTK_LABEL(lbl1), xfrwd[1]);
				g_free(xfrwd[1]);
			}
			else 
			{
				while (arcfile[i] != 0x0A) i++;
				i++;
				gtk_label_set_text(GTK_LABEL(lbl1), "\0");
			}
			g_free(xfrwd[0]);
		}
		txt = lookup_widget(GTK_WIDGET(rpm_info), "text6");
		j = 0;
		while (arcfile[i] != ':') i++;
		i+=2;
		while (arcfile[i] != 0x0A)
		{
			xfr[j] = arcfile[i];
			i++;j++;
		}
		xfr[j] = '\0';
		xfrwd[0] = g_strdup(xfr);
		i++;
		textlen = gtk_text_get_length(GTK_TEXT(txt));
		gtk_text_set_point(GTK_TEXT(txt), 0);
		gtk_text_forward_delete(GTK_TEXT(txt), textlen);
		gtk_text_freeze(GTK_TEXT(txt));
		gtk_text_insert(GTK_TEXT(txt), NULL,
			&txt->style->black, NULL, xfrwd[0], strlen(xfrwd[0]));
		gtk_text_thaw(GTK_TEXT(txt));
		g_free(xfrwd[0]);
	}
}

gint
rpm_open(pager *page, gchar *file_name)
{
	int status=0;
	int vline=0;
	gchar *myfile = NULL;
	gchar *arcfile = NULL;
	gint i,j, w,line = 0;
	guint time_id = 0, arcsize;
	FILE *tempfile;
	gchar xfr[0x100];
	gchar *xfrwd[6] = {NULL,NULL,NULL,NULL,NULL,NULL};
	gchar *clist_line[9] = {"","","","","","","","",""};
	gchar *temp_name = g_concat_dir_and_file(TMPDIR,"gchiveXXXXXX");
	int tf;
	gboolean is_dr = FALSE;
	gboolean dirprs = FALSE;
	gchar *dirs[512];
	guint e, k;
	gchar *tdir = NULL, *rbase2 = NULL, *fl = NULL;
	
	tf = mkstemp(temp_name);
	close(tf);
	
	step = 0.003;
	progr = 0.1;
		
	time_id = gtk_timeout_add(TIMEOUT_GTK, (GtkFunction)mytime, NULL);
	myfile = g_strdup(file_name);

	if (gnochive_exec_arch("/tmp/remflt", "remflt", 
			gnochivemain->gc_cfg->archs.rpm_path,
			"-q", "-pl", "-v", file_name, temp_name, NULL))
{
	while(gtk_events_pending())
		gtk_main_iteration();
	gtk_clist_freeze(GTK_CLIST(page->clist));
	gtk_clist_clear(GTK_CLIST(page->clist));
	clist_line[1] = directory_read;
	gtk_clist_append(GTK_CLIST(page->clist), clist_line);
	clist_line[1] = directory_up;
	gtk_clist_append(GTK_CLIST(page->clist), clist_line);
	gtk_clist_set_pixmap(GTK_CLIST(page->clist),
						1, 0, dotpix, dotmask);
	vline = line = 2;
	while (!waitpid(0, NULL, WNOHANG))
	{
		gtk_main_iteration();
	}
	while(gtk_events_pending())
		gtk_main_iteration();
	tempfile = fopen(temp_name, "rb");
	if (tempfile)
	{
		fseek(tempfile, 0L, SEEK_END);
		arcsize = ftell(tempfile);
		arcfile = g_malloc(arcsize);
		fseek(tempfile, 0L, SEEK_SET);
		fread(arcfile, 1, arcsize, tempfile);
		fclose(tempfile);
		i=0;e=0;
		while (i<=arcsize-2) {
				j=0;line++;
				while (arcfile[i]!=0x20) {
					xfr[j]=arcfile[i];
					i++;j++;
				}
				xfr[j] = '\0';
				xfrwd[0] = g_strdup(xfr);
				(gchar *)clist_line[6] = xfrwd[0];
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) i++;
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) i++;
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) i++;
				while (arcfile[i] == 0x20) i++;
				j=0;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
							xfr[j] = '\0';
				xfrwd[1] = g_strdup(xfr);
				(gchar *)clist_line[3] = xfrwd[1];
				j=0;
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
				xfr[j]=' ';
				while (arcfile[i] == 0x20) i++;
				j++;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
				xfr[j]=' ';
				while (arcfile[i] == 0x20) i++;
				j++;
				while (arcfile[i] != 0x20) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
							xfr[j]='\0';
				xfrwd[2]=g_strdup(xfr);
				(gchar *)clist_line[5]=xfrwd[2];
				j=0;
				while (arcfile[i] == 0x20) i++;
				while (arcfile[i] != 0x0A) {
							xfr[j] = arcfile[i];
							i++;j++;
							}
							xfr[j]='\0';
				xfrwd[3] = g_strdup(xfr);
				(gchar *)clist_line[1] = xfrwd[3];
				i++;j=0;
				if (clist_line[6][1]=='d')
					is_dr = TRUE;
				else
					is_dr = FALSE;
				if ((fl = g_path_skip_root(clist_line[1])) == NULL)
					fl = clist_line[1];
				tdir = extract_root_word(fl, 1);
				if (e==0)
				{
					if (is_dr) 
					{
						dirs[e] = extract_root_word(fl, 1);
						e++;
						(gchar *)clist_line[1] = tdir;
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, dirpix, dirmask);
						vline++;
						is_dr = FALSE;
					}
					else
					{
						if (!process_line(clist_line[1])) {
							gtk_clist_append(GTK_CLIST(page->clist), clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(page->clist),
								vline, 0, docpix, docmask);
							vline++;
						}
						else
						{
							rbase2 = extract_root_word(fl, 1);
							dirprs = FALSE;
							for (k=0;k<e;k++)
							{
								if (!cmpare(dirs[k], rbase2))
									dirprs = TRUE;
							}
							g_free(rbase2);
							if (!dirprs)
							{
								dirs[e] = extract_root_word(fl, 1);
								e++;
								/* found a new dir and append it in the clist */
								(gchar *)clist_line[1] =
									extract_root_word(fl, 1);
								(gchar *)clist_line[3] = g_strdup("0");
								(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
								gtk_clist_append(GTK_CLIST(page->clist), clist_line);
								gtk_clist_set_pixmap(GTK_CLIST(page->clist),
									vline, 0, dirpix, dirmask);
								vline++;
							}
						}
					}
				}
				else {
				if (is_dr) 
				{
					is_dr = FALSE;
					dirprs = FALSE;
					for (k=0;k<e;k++)
					{
						if (!cmpare(dirs[k], tdir))
							dirprs = TRUE;
					}
					if (!dirprs)
					{
						dirs[e] = extract_root_word(fl, 1);
						e++;
						/* found a new dir and append it in the clist */
						(gchar *)clist_line[1] = tdir;
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, dirpix, dirmask);
						vline++;
					}
				}
				else
				{
					if (!process_line(clist_line[1])) {
						gtk_clist_append(GTK_CLIST(page->clist), clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							vline, 0, docpix, docmask);
						vline++;
					}
					else
					{
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(dirs[k], tdir))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(fl, 1);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = tdir;
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							gtk_clist_append(GTK_CLIST(page->clist), clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(page->clist),
								vline, 0, dirpix, dirmask);
							vline++;
						}
					}
				}
			}
			for (w=0;w<=3;w++)
				g_free(xfrwd[w]);
			}
			gtk_clist_thaw(GTK_CLIST(page->clist));
			gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
			gtk_timeout_remove(time_id);
			page->current_lines =line;
			page->selected_rows =
							g_malloc0(page->current_lines);
			page->ad->data = arcfile;
			page->ad->root = g_strdup("\0");
			page->ad->depth = 1;
			status=1;
	}
	else status=0;
	g_free(myfile);
	remove(temp_name);
	g_free(temp_name);
	return status;
}
g_free(temp_name);
g_free(myfile);
return -1;
}

gint
rpm_open_filesystem(pager *page, gchar *file_name)
{
	gchar *clist_line[9] = {"","","","","","","","",""};
	gchar *rpm_fs_names[6] = {"HEADER", "INSTALL", "UPGRADE", "ERROR", "INFO", "CONTENTS.cpio"};
	gint i, row;
	struct stat statbuf;
	gchar mytime[30];
	
	lstat(file_name, &statbuf);
	strftime(&mytime[0], 30, "%a, %b %e %Y %I:%M:%S %p", localtime(&statbuf.st_atime));
		
	//gtk_clist_freeze(GTK_CLIST(page->clist));
	gtk_clist_clear(GTK_CLIST(page->clist));
	clist_line[1] = directory_read;
	gtk_clist_append(GTK_CLIST(page->clist), clist_line);
	clist_line[1] = directory_up;
	gtk_clist_append(GTK_CLIST(page->clist), clist_line);
	gtk_clist_set_pixmap(GTK_CLIST(page->clist),
						1, 0, dotpix, dotmask);
	row = 2;
	for (i=0;i<=5;i++)
	{
		clist_line[1] = rpm_fs_names[i];
		(gchar*)clist_line[3]= g_strdup("0");
		(gchar*)clist_line[5]=mytime;
		gtk_clist_append(GTK_CLIST(page->clist), clist_line);
		if (i<4)
			gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							row, 0, docpix,docmask);
		else
			gtk_clist_set_pixmap(GTK_CLIST(page->clist),
							row, 0, dirpix,dirmask);
		row++;
	}
	return -1;
}

void
rpm_cd		(GtkWidget *clst, gchar *child)
{
	int vline = 0;
	int line = 0;
	int i = 0, j = 0, e = 0, k;
	gchar *tword = NULL;
	gchar *temp = NULL;
	gchar *arcfile = NULL;
	gchar xfr[100];
	gint row, arcsize;
	gchar *xfrwd[6] = {NULL,NULL,NULL,NULL,NULL,NULL};
	gchar *clist_line[9] = {"", "", "", "", "", "", "", "", ""};
	gchar *dirs[512];
	gchar *tdir = NULL;
	gboolean is_dr = FALSE, dirprs;
	gchar *wr_xfr = FALSE;
	
		
	line = vline = 2;
	
	//while (gtk_events_pending())
		//gtk_main_iteration();
	gtk_clist_freeze(GTK_CLIST(clst));
	gtk_clist_clear(GTK_CLIST(gnochivemain->active_page->clist));
	g_free(gnochivemain->active_page->selected_rows);
	
	gtk_clist_set_auto_sort(GTK_CLIST(clst), 1);
	gtk_clist_set_sort_column(GTK_CLIST(clst), 2);
	gtk_clist_set_sort_type(GTK_CLIST(clst), GTK_SORT_DESCENDING);
	gtk_clist_sort(GTK_CLIST(clst));
	
	arcfile = gnochivemain->active_page->ad->data;
	arcsize = strlen(arcfile);
	arcsize-=1;
	i=0;
	while (i<=arcsize-2) {
		j=0;line++;
		while (arcfile[i]!=0x20) {
			xfr[j]=arcfile[i];
			i++;j++;
		}
		xfr[j] = '\0';
		xfrwd[0] = g_strdup(xfr);
		(gchar *)clist_line[6] = xfrwd[0];
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) i++;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) i++;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) i++;
		while (arcfile[i] == 0x20) i++;
		j=0;
		while (arcfile[i] != 0x20) {
					xfr[j] = arcfile[i];
					i++;j++;
					}
					xfr[j] = '\0';
		xfrwd[1] = g_strdup(xfr);
		(gchar *)clist_line[3] = xfrwd[1];
		j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x20) {
					xfr[j] = arcfile[i];
					i++;j++;
					}
		xfr[j]=' ';
		while (arcfile[i] == 0x20) i++;
		j++;
		while (arcfile[i] != 0x20) {
					xfr[j] = arcfile[i];
					i++;j++;
					}
		xfr[j]=' ';
		while (arcfile[i] == 0x20) i++;
		j++;
		while (arcfile[i] != 0x20) {
					xfr[j] = arcfile[i];
					i++;j++;
					}
					xfr[j]='\0';
		xfrwd[2]=g_strdup(xfr);
		(gchar *)clist_line[5]=xfrwd[2];
		j=0;
		while (arcfile[i] == 0x20) i++;
		while (arcfile[i] != 0x0A) {
					xfr[j] = arcfile[i];
					i++;j++;
					}
					xfr[j]='\0';
		xfrwd[3] = g_strdup(xfr);
		(gchar *)clist_line[1] = xfrwd[3];
		i++;j=0;
		if (clist_line[6][0]=='d')
		{
			is_dr = TRUE;
			wr_xfr = g_strconcat(g_path_skip_root(xfr), G_DIR_SEPARATOR_S, NULL);
		}
		else
		{
			is_dr = FALSE;
			wr_xfr = g_path_skip_root(xfr);
		}
		if (cmpare(wr_xfr, child) == 0)
		{
			if (e==0 && strcmp(wr_xfr, child) != 0)
			{
				if (is_dr && strcmp(wr_xfr, child) != 0)
				{
					tdir = extract_root_word(wr_xfr,
						gnochivemain->active_page->ad->depth);
					dirs[e] = extract_root_word(wr_xfr,
						gnochivemain->active_page->ad->depth);
					e++;
					clist_line[1] = trim_root_word(child, tdir);
					clist_line[2] = "dir";
					row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
					gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
					g_free(clist_line[1]);
					vline++;
					g_free(tdir);
					is_dr = FALSE;
				}
				else
				{
					gchar *rbase2 = trim_root_word(child, wr_xfr);
					if (!process_line(rbase2)) {
						clist_line[1] = trim_root_word(child, wr_xfr);
						clist_line[2] = " ";
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst),
							row, 0, docpix,docmask);
						g_free(clist_line[1]);
						vline++;
					}
					else
					{
						gchar *rbase3 = extract_root_word(wr_xfr,
							gnochivemain->active_page->ad->depth);
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(dirs[k], rbase3))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(wr_xfr,
								gnochivemain->active_page->ad->depth);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = trim_root_word(child, rbase3);
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
							g_free(clist_line[1]);
							vline++;
						}
						g_free(rbase3);
					}
					g_free(rbase2);
				}
			}
			else if (e>0  && strcmp(wr_xfr, child) != 0)
			{
				if (is_dr && strcmp(wr_xfr, child) != 0)
				{
					tdir = extract_root_word(wr_xfr,
						gnochivemain->active_page->ad->depth);
					is_dr = FALSE;
					dirprs = FALSE;
					for (k=0;k<e;k++)
					{
						if (!cmpare(dirs[k], tdir))
							dirprs = TRUE;
					}
					if (!dirprs)
					{
						dirs[e] = extract_root_word(wr_xfr,
							gnochivemain->active_page->ad->depth);
						e++;
						/* found a new dir and append it in the clist */
						(gchar *)clist_line[1] = trim_root_word(child, tdir);
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
						g_free(clist_line[1]);
						vline++;
					}
					g_free(tdir);
				}
				else
				{
					gchar *rbase2 = trim_root_word(child, wr_xfr);
					if (!process_line(rbase2)) {
						clist_line[1] = trim_root_word(child, wr_xfr);
						clist_line[2] = " ";
						row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
						gtk_clist_set_pixmap(GTK_CLIST(clst),
							row, 0, docpix,docmask);
						g_free(clist_line[1]);
						vline++;
					}
					else
					{
						tdir = extract_root_word(wr_xfr,
							gnochivemain->active_page->ad->depth);
						dirprs = FALSE;
						for (k=0;k<e;k++)
						{
							if (!cmpare(dirs[k], tdir))
								dirprs = TRUE;
						}
						if (!dirprs)
						{
							dirs[e] = extract_root_word(wr_xfr,
								gnochivemain->active_page->ad->depth);
							e++;
							/* found a new dir and append it in the clist */
							(gchar *)clist_line[1] = trim_root_word(child, tdir);
							(gchar *)clist_line[3] = g_strdup("0");
							(gchar *)clist_line[6] = g_strdup("drwxrwxr-x");
							row = gtk_clist_insert(GTK_CLIST(clst), 0, clist_line);
							gtk_clist_set_pixmap(GTK_CLIST(clst), row, 0, dirpix, dirmask);
							g_free(clist_line[1]);
							vline++;
						}
						g_free(tdir);
					}
					g_free(rbase2);
				}
				//g_free(tdir);
			}
		}
		g_free(clist_line[6]);g_free(clist_line[3]);g_free(clist_line[5]);
	}
	for (i=0;i<=9;i++)
		clist_line[i] = "\0";
	gtk_clist_set_auto_sort(GTK_CLIST(clst), 0);	
	clist_line[1] = g_strdup(directory_up);
	gtk_clist_prepend(GTK_CLIST(clst), clist_line);
	gtk_clist_set_pixmap(GTK_CLIST(clst), 0, 0, dotpix, dotmask);
	g_free(clist_line[1]);
	clist_line[1] = g_strdup(directory_read);
	gtk_clist_prepend(GTK_CLIST(clst), clist_line);
	g_free(clist_line[1]);
	gnochivemain->active_page->current_lines = vline;
	gnochivemain->active_page->selected_rows = 
						g_malloc0(gnochivemain->active_page->current_lines);
	temp = g_strdup(child);
	if (temp[0] == '/')
	{
		g_free(gnochivemain->active_page->ad->root);
		gtk_entry_set_text(GTK_ENTRY(arcmenu1), "\0");
		gnochivemain->active_page->ad->root = g_strdup("\0");
	}
	else
	{
		g_free(gnochivemain->active_page->ad->root);
		gtk_entry_set_text(GTK_ENTRY(arcmenu1), temp);
		gnochivemain->active_page->ad->root = g_strdup(child);
	}
	g_free(temp);
	gtk_clist_thaw(GTK_CLIST(clst));
	g_free(tword);
}


/*
 * 0 - unknown
 * 1 - rpm file
 */
 
gint
detect_rpm(gchar *file)
{
	int status = 0;
	FILE *tmp;
	gchar buff[0x30];
	
	tmp = fopen(file,"r");
	fread(buff, 4, 1, tmp);
	fclose(tmp);
	if (cmpare(buff, RPM_MAGIC)==0)
		return 1;
	return status;
}

gchar*
rpm_extract_with_cpio(gchar *arc_name, gchar *file)
{
	gchar *tfile = g_concat_dir_and_file(TMPDIR, "gchive.ext.XXXXXX");
	gchar *tfile2 = g_concat_dir_and_file(TMPDIR, "gchive.ext.XXXXXX");
	gchar *tfile3 = NULL;
	int tf, tf2;
	FILE *tmp;
	
	tf = mkstemp(tfile);
	tf2 = mkstemp(tfile2);
		
	if (tf && tf2)
	{
		close(tf);
		close(tf2);
		tmp = fopen(tfile2, "w");
		fprintf(tmp, "#!/bin/sh\n");
		fprintf(tmp, "%s \"%s\" >%s \n",
		gnochivemain->gc_cfg->archs.rpm2cpio_path,
		arc_name, tfile);
		fclose(tmp);
		chmod(tfile2, S_IRUSR | S_IWUSR | S_IXUSR);
		if (gnochive_exec_arch(tfile2, tfile2, NULL))
			wait(0);
	}
	tfile3 = cpio_extract_file_from_archive(tfile, file);
	remove(tfile2);
	remove(tfile);
	g_free(tfile2);
	g_free(tfile);
	if (tfile3)
		return tfile3;
	else
		return NULL;
}

static 
guint mytime(gpointer data)
{
	GDK_THREADS_ENTER();
	progr=progr+step;
	if (progr<=1)
		gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),progr);
	else gnome_appbar_set_progress(GNOME_APPBAR(gnochivemain->mainbar),1);
	GDK_THREADS_LEAVE();
	return TRUE;
}
